@@ -447,7 +447,7 @@ QVariant QQmlSettings::value(const QString &key, const QVariant &defaultValue) c
 }
 
 /*!
-   \qmlmethod Settings::setValue(string key, var value)
+   \qmlmethod void Settings::setValue(string key, var value)
 
    Sets the value of setting \a key to \a value. If the key already exists,
    the previous value is overwritten.
@@ -462,7 +462,7 @@ void QQmlSettings::setValue(const QString &key, const QVariant &value)
 }
 
 /*!
-   \qmlmethod Settings::sync()
+   \qmlmethod void Settings::sync()
 
     Writes any unsaved changes to permanent storage, and reloads any
     settings that have been changed in the meantime by another
