@@ -1,5 +1,6 @@
 // Copyright (C) 2019 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #include "qquickboundaryrule_p.h"
 
@@ -13,7 +14,7 @@
 
 QT_BEGIN_NAMESPACE
 
-Q_LOGGING_CATEGORY(lcBR, "qt.quick.boundaryrule")
+Q_STATIC_LOGGING_CATEGORY(lcBR, "qt.quick.boundaryrule")
 
 class QQuickBoundaryReturnJob;
 class QQuickBoundaryRulePrivate : public QObjectPrivate
@@ -344,7 +345,7 @@ void QQuickBoundaryRule::setOvershootFilter(OvershootFilter overshootFilter)
 }
 
 /*!
-    \qmlmethod bool Qt.labs.animation::BoundaryRule::returnToBounds
+    \qmlmethod bool Qt.labs.animation::BoundaryRule::returnToBounds()
 
     Returns the intercepted property to a value between \l minimum and
     \l maximum, such that \l currentOvershoot and \l peakOvershoot are both
