@@ -1,5 +1,6 @@
 // Copyright (C) 2016 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant
 
 #include "qqmlboundsignal_p.h"
 
@@ -28,6 +29,12 @@ Q_TRACE_POINT(qtqml, QQmlHandlingSignal_entry, const QQmlEngine *engine, const Q
               const QString &fileName, int line, int column)
 Q_TRACE_POINT(qtqml, QQmlHandlingSignal_exit)
 
+/*!
+    \class QQmlBoundSignal
+    \inmodule QtQml
+    \internal
+*/
+
 QQmlBoundSignalExpression::QQmlBoundSignalExpression(const QObject *target, int index, const QQmlRefPointer<QQmlContextData> &ctxt, QObject *scope,
         const QString &expression, const QString &fileName, quint16 line, quint16 column,
         const QString &handlerName, const QString &parameterString)
@@ -51,7 +58,7 @@ QQmlBoundSignalExpression::QQmlBoundSignalExpression(const QObject *target, int
         //TODO: look at using the property cache here (as in the compiler)
         //      for further optimization
         QMetaMethod signal = QMetaObjectPrivate::signal(m_target->metaObject(), m_index);
-        function += QQmlPropertyCache::signalParameterStringForJS(v4, signal.parameterNames(), &error);
+        function += QQmlPropertyCache::signalParameterStringForJS(signal.parameterNames(), &error);
 
         if (!error.isEmpty()) {
             qmlWarning(scopeObject()) << error;
@@ -62,7 +69,8 @@ QQmlBoundSignalExpression::QQmlBoundSignalExpression(const QObject *target, int
 
     function += QLatin1String(") { ") + expression + QLatin1String(" })");
     QV4::Scope valueScope(v4);
-    QV4::ScopedFunctionObject f(valueScope, evalFunction(context(), scopeObject(), function, fileName, line));
+    QV4::Scoped<QV4::JavaScriptFunctionObject> f(
+            valueScope, evalFunction(context(), scopeObject(), function, fileName, line));
     QV4::ScopedContext context(valueScope, f->scope());
     setupFunction(context, f->function());
 }
@@ -82,7 +90,7 @@ QQmlBoundSignalExpression::QQmlBoundSignalExpression(const QObject *target, int
         QList<QByteArray> signalParameters = QMetaObjectPrivate::signal(m_target->metaObject(), m_index).parameterNames();
         if (!signalParameters.isEmpty()) {
             QString error;
-            QQmlPropertyCache::signalParameterStringForJS(engine, signalParameters, &error);
+            QQmlPropertyCache::signalParameterStringForJS(signalParameters, &error);
             if (!error.isEmpty()) {
                 qmlWarning(scopeObject) << error;
                 return;
@@ -107,7 +115,7 @@ QQmlBoundSignalExpression::QQmlBoundSignalExpression(const QObject *target, int
         // we need to run the outer function to get the nested one.
         if (function->isClosureWrapper()) {
             bool isUndefined = false;
-            QV4::ScopedFunctionObject result(
+            QV4::Scoped<QV4::JavaScriptFunctionObject> result(
                         valueScope, QQmlJavaScriptExpression::evaluate(&isUndefined));
 
             Q_ASSERT(!isUndefined);
@@ -179,7 +187,7 @@ void QQmlBoundSignalExpression::evaluate(void **a)
                     QMetaObjectPrivate::signal(targetMeta, m_index).methodIndex());
 
         int argCount = metaMethod.parameterCount();
-        QQmlMetaObject::ArgTypeStorage storage;
+        QQmlMetaObject::ArgTypeStorage<9> storage;
         storage.reserve(argCount + 1);
         storage.append(QMetaType()); // We're not interested in the return value
         for (int i = 0; i < argCount; ++i) {
