@@ -1,5 +1,6 @@
 // Copyright (C) 2021 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant
 
 #include "qqmlbuiltinfunctions_p.h"
 
@@ -13,6 +14,7 @@
 #include <private/qqmlstringconverters_p.h>
 
 #include <private/qv4dateobject_p.h>
+#include <private/qv4domerrors_p.h>
 #include <private/qv4engine_p.h>
 #include <private/qv4functionobject_p.h>
 #include <private/qv4include_p.h>
@@ -1225,25 +1227,163 @@ QString QtObject::md5(const QString &data) const
 }
 
 /*!
-\qmlmethod string Qt::btoa(data)
-Binary to ASCII - this function returns a base64 encoding of \a data.
+\qmlmethod string Qt::btoa(string data)
+\deprecated [6.11] This method performs a UTF-8 conversion of the string before encoding it.
+
+Binary to ASCII --- this function returns a base64 encoding of \a data.
 */
 QString QtObject::btoa(const QString &data) const
 {
+    qWarning("Qt.btoa(string): This method is deprecated. "
+             "Its output differs from the common Web API. "
+             "Use the overloads that take array-likes.");
     return QLatin1String(data.toUtf8().toBase64());
 }
 
 /*!
-\qmlmethod string Qt::atob(data)
-ASCII to binary - this function decodes the base64 encoded \a data string and returns it.
+\qmlmethod string Qt::atob(string data)
+\deprecated [6.11] This method performs a Latin-1 conversion of the string before decoding it
+                   and then interprets the result as UTF-8.
+
+ASCII to binary --- this function decodes the base64 encoded \a data string and returns it.
 */
 QString QtObject::atob(const QString &data) const
 {
+    qWarning("Qt.atob(string): This method is deprecated. "
+             "Its output differs from the common Web API. "
+             "Use the overloads that take array-likes.");
     return QString::fromUtf8(QByteArray::fromBase64(data.toLatin1()));
 }
 
 /*!
-    \qmlmethod Qt::quit()
+\qmlmethod ArrayBuffer Qt::btoa(ArrayBuffer data)
+\since 6.11
+
+Binary to ASCII --- this function returns a base64 encoding of \a data.
+
+You can pass any array-like as \a data, and it will try to convert
+it into a byte array. In particular this works with a list of numbers
+and a list of one-character strings. The most efficient way to do this,
+however, is passing either a QByteArray or a JavaScript ArrayBuffer
+object.
+
+If the conversion fails and it turns out that the \a data is not of
+the expected form, an \c{Invalid Character} exception is thrown and an
+empty array is returned.
+
+*/
+QByteArray QtObject::btoa(const QByteArray &data) const
+{
+    return data.toBase64();
+}
+
+static QV4::ReturnedValue throwInvalidCharacter(QV4::ExecutionEngine *engine)
+{
+    QV4::Scope scope(engine);
+    THROW_DOM(DOMEXCEPTION_INVALID_CHARACTER_ERR, "Invalid character");
+}
+
+/*!
+\qmlmethod ArrayBuffer Qt::atob(ArrayBuffer data)
+\since 6.11
+
+ASCII to binary --- this function decodes the base64 encoded \a data and returns it.
+
+You can pass any array-like as \a data, and it will try to convert
+it into a byte array. In particular this works with a list of numbers
+and a list of one-character strings. The most efficient way to do this,
+however, is passing either a QByteArray or a JavaScript ArrayBuffer
+object.
+
+If the conversion fails and it turns out that the \a data is not of
+the expected form, an \c{Invalid Character} exception is thrown and an
+empty array is returned.
+
+*/
+QByteArray QtObject::atob(const QByteArray &data) const
+{
+    const auto result
+            = QByteArray::fromBase64Encoding(data, QByteArray::AbortOnBase64DecodingErrors);
+    if (result.decodingStatus == QByteArray::Base64DecodingStatus::Ok)
+        return result.decoded;
+
+    throwInvalidCharacter(v4Engine());
+    return QByteArray();
+}
+
+static QByteArray convertVariantList(const QVariantList &data, QV4::ExecutionEngine *engine)
+{
+    const auto fail = [&]() {
+        throwInvalidCharacter(engine);
+        return QByteArray();
+    };
+
+    QByteArray result;
+
+    const auto append = [&](auto value) {
+        if (value < 0 || value >= 256)
+            return false;
+        result.append(char(value));
+        return true;
+    };
+
+    for (const QVariant &entry : data) {
+        switch (entry.typeId()) {
+        case QMetaType::Char:
+            result.append(*static_cast<const char *>(entry.constData()));
+            break;
+        case QMetaType::Int: {
+            if (!append(*static_cast<const int *>(entry.constData())))
+                return fail();
+            break;
+        }
+        case QMetaType::Double: {
+            if (!append(*static_cast<const double *>(entry.constData())))
+                return fail();
+            break;
+        }
+        case QMetaType::QString: {
+            const QString *string = static_cast<const QString *>(entry.constData());
+            if (string->length() != 1)
+                return fail();
+            if (!append(string->at(0).unicode()))
+                return fail();
+            break;
+        }
+        default:
+            return fail();
+        }
+    }
+
+    return result;
+}
+
+/*!
+\qmlmethod var Qt::btoa(var data)
+\overload
+\since 6.11
+
+Binary to ASCII --- this function returns a base64 encoding of \a data.
+*/
+QByteArray QtObject::btoa(const QVariantList &data) const
+{
+    return btoa(convertVariantList(data, v4Engine()));
+}
+
+/*!
+\qmlmethod var Qt::atob(var data)
+\overload
+\since 6.11
+
+ASCII to binary --- this function decodes the base64 encoded \a data and returns it.
+*/
+QByteArray QtObject::atob(const QVariantList &data) const
+{
+    return atob(convertVariantList(data, v4Engine()));
+}
+
+/*!
+    \qmlmethod void Qt::quit()
 
     This function causes the QQmlEngine::quit() signal to be emitted.
     Within the \l {Prototyping with the QML Runtime Tool}{qml tool},
@@ -1260,7 +1400,7 @@ void QtObject::quit() const
 }
 
 /*!
-    \qmlmethod Qt::exit(int retCode)
+    \qmlmethod void Qt::exit(int retCode)
 
     This function causes the QQmlEngine::exit(int) signal to be emitted.
     Within the \l {Prototyping with the QML Runtime Tool}{qml tool},
@@ -1374,7 +1514,7 @@ QObject *QtObject::createQmlObject(const QString &qml, QObject *parent, const QU
         return nullptr;
     }
 
-    QQmlRefPointer<QQmlTypeData> typeData = QQmlEnginePrivate::get(engine)->typeLoader.getType(
+    QQmlRefPointer<QQmlTypeData> typeData = v4Engine()->typeLoader()->getType(
                 qml.toUtf8(), resolvedUrl, QQmlTypeLoader::Synchronous);
 
     if (!typeData->isCompleteOrError()) {
@@ -1422,6 +1562,7 @@ QObject *QtObject::createQmlObject(const QString &qml, QObject *parent, const QU
     }
     component.completeCreate();
 
+    v4Engine()->trimCompilationUnitsForUrl(resolvedUrl);
     if (component.isError()) {
         ScopedValue v(scope, Error::create(scope.engine, component.errors()));
         scope.engine->throwError(v);
@@ -1600,7 +1741,7 @@ QBindable<QString> QtObject::uiLanguageBindable()
 
 #if QT_CONFIG(qml_locale)
 /*!
-    \qmlmethod Qt::locale(name)
+    \qmlmethod locale Qt::locale(name)
 
     Returns a JS object representing the locale with the specified
     \a name, which has the format "language[_territory][.codeset][@modifier]"
@@ -1617,6 +1758,8 @@ QBindable<QString> QtObject::uiLanguageBindable()
     is not present, or is not a valid ISO 3166 code, the most
     appropriate country is chosen for the specified language.
 
+    The returned object is of an anonymous QML type backed by \l QLocale.
+
     \sa Locale
 */
 QLocale QtObject::locale() const
@@ -1657,7 +1800,7 @@ QQmlSourceLocation QQmlBindingFunction::currentLocation() const
 DEFINE_OBJECT_VTABLE(QQmlBindingFunction);
 
 /*!
-    \qmlmethod Qt::binding(function)
+    \qmlmethod var Qt::binding(var function)
 
     Returns a JavaScript object representing a \l{Property Binding}{property binding},
     with a \a function that evaluates the binding.
@@ -1719,7 +1862,7 @@ void QtObject::callLater(QQmlV4FunctionPtr args)
 }
 
 /*!
-    \qmlmethod Qt::enumStringToValue(enumType, keyName)
+    \qmlmethod real Qt::enumStringToValue(enumType, keyName)
 
     Returns the numeric value of key \a keyName in enum \a enumType. If the
     enum could not be found, a \c TypeError is thrown. If the key is not an
@@ -1737,7 +1880,7 @@ double QtObject::enumStringToValue(const QJSManagedValue &enumType, const QStrin
 }
 
 /*!
-    \qmlmethod Qt::enumValueToString(enumType, keyValue)
+    \qmlmethod string Qt::enumValueToString(enumType, keyValue)
 
     Returns the string representation of a key of enum \a enumType that has the
     value \a keyValue. If the enum could not be found, a \c TypeError is
@@ -1766,7 +1909,7 @@ QString QtObject::enumValueToString(const QJSManagedValue &enumType, double valu
 }
 
 /*!
-    \qmlmethod Qt::enumValueToStrings(enumType, keyValue)
+    \qmlmethod list<string> Qt::enumValueToStrings(enumType, keyValue)
 
     Returns a list of the string representation of all the keys of enum
     \a enumType that have the value \a keyValue. If the enum could not be
@@ -2523,8 +2666,8 @@ ReturnedValue GlobalExtensions::method_string_arg(const FunctionObject *b, const
 }
 
 /*!
-\qmlmethod Qt::callLater(function)
-\qmlmethod Qt::callLater(function, argument1, argument2, ...)
+\qmlmethod void Qt::callLater(function)
+\qmlmethod void Qt::callLater(function, argument1, argument2, ...)
 \since 5.8
 Use this function to eliminate redundant calls to a function or signal.
 
