@@ -1,5 +1,6 @@
 // Copyright (C) 2016 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant
 
 #include "qqmlpropertycache_p.h"
 
@@ -10,7 +11,9 @@
 #include <private/qmetaobject_p.h>
 #include <private/qmetaobjectbuilder_p.h>
 #include <private/qqmlpropertycachemethodarguments_p.h>
+#include <private/qqmlsignalnames_p.h>
 
+#include <private/qv4codegen_p.h>
 #include <private/qv4value_p.h>
 
 #include <QtCore/qdebug.h>
@@ -29,6 +32,129 @@ QT_BEGIN_NAMESPACE
 
 #define Q_INT16_MAX 32767
 
+namespace OverrideSemantics {
+namespace detail {
+
+static inline bool hasInvalidModifierCombintation(const QQmlPropertyData &overridingProperty)
+{
+    return (overridingProperty.isVirtual() && overridingProperty.isFinal())
+            || (overridingProperty.doesOverride() && overridingProperty.isFinal())
+            || (overridingProperty.isVirtual() && overridingProperty.doesOverride());
+}
+
+/*
+ * Performs minimal validation of property override semantics.
+ *
+ * This function checks whether an existing property can be overridden.
+ * It distinguishes between the following cases:
+ *  - No base property exists          → Status::NoOverride
+ *  - Base property is marked final    → Status::OverridingFinal
+ *  - Otherwise                        → Status::Valid
+ *
+ * The minimal check is used in contexts where only basic inheritance
+ * constraints (existence and finality) must be verified.
+ */
+static inline Status checkMinimal(const QQmlPropertyData *const existingProperty)
+{
+    if (!existingProperty)
+        return Status::NoOverride;
+
+    if (existingProperty->isFinal()) {
+        return Status::OverridingFinal;
+    }
+
+    return Status::Valid;
+}
+
+/*
+ * Performs full validation of property override semantics.
+ *
+ * This function enforces the full set of rules for `virtual`, `override`,
+ * and `final` keyword combinations when resolving property overrides.
+ * It verifies:
+ *
+ *  - If `override` is specified but no base property exists,
+ *    the override is invalid (Status::MissingBase).
+ *
+ *  - If `override` is NOT specified and no base property exists,
+ *    then there is no override (Status::NoOverride).
+ *
+ *  - If the base property is final, overriding is not allowed
+ *    (Status::OverridingFinal).
+ *
+ *  - If the base property is invokable and overriding is not (and vice-versa),
+ *    override is invalid (Status::InvokabilityMismatch).
+ *
+ *  - If the base property is not virtual, but 'override' is present
+ *    overriding is not allowed (Status::OverridingNonVirtualError),
+ *    otherwise it returns Status::OverridingNonVirtual
+ *
+ *  - If no `override` or `final` keyword is specified for an existing virtual base,
+ *    the override specifier is missing (Status::MissingOverrideOrFinalSpecifier).
+ *
+ * Returns Status::Valid if the combination is semantically correct.
+ */
+static inline Status checkFull(const QQmlPropertyData &overridingProperty,
+                               const QQmlPropertyData *const existingProperty)
+{
+    const auto overrideKeyword = overridingProperty.doesOverride();
+    if (overrideKeyword && !existingProperty) {
+        return Status::MissingBase;
+    }
+
+    const auto minimalCheckRes = checkMinimal(existingProperty);
+    if (minimalCheckRes != Status::Valid) {
+        return minimalCheckRes;
+    }
+
+    // if the property doesn't exist we should have returned MissingBase or NoOverride already
+    Q_ASSERT(existingProperty);
+    if (overridingProperty.isFunction() != existingProperty->isFunction()) {
+        return Status::InvokabilityMismatch;
+    }
+
+    if (!existingProperty->isVirtual()) {
+        return overrideKeyword ? Status::OverridingNonVirtualError
+                               : Status::OverridingNonVirtual;
+    }
+
+    const auto overrideOrFinal = overrideKeyword || overridingProperty.isFinal();
+    if (!overrideOrFinal) {
+        return Status::MissingOverrideOrFinalSpecifier;
+    }
+
+    return Status::Valid;
+}
+
+static inline Status check(const QQmlPropertyData &overridingProperty,
+                           const QQmlPropertyData *const existingProperty, CheckMode mode)
+{
+    Q_ASSERT(!hasInvalidModifierCombintation(overridingProperty));
+
+    switch (mode) {
+    case CheckMode::Minimal:
+        return detail::checkMinimal(existingProperty);
+    case CheckMode::Full:
+        return detail::checkFull(overridingProperty, existingProperty);
+    default:
+        Q_UNREACHABLE_RETURN(Status::Unknown);
+    }
+}
+} // namespace detail
+
+Status handleOverride(QQmlPropertyData &overridingProperty, QQmlPropertyData *existingProperty,
+                      CheckMode mode)
+{
+    const auto status = detail::check(overridingProperty, existingProperty, mode);
+
+    if (isValidOverride(status)) {
+        overridingProperty.markAsOverrideOf(existingProperty);
+    }
+    return status;
+}
+
+} // namespace OverrideSemantics
+
 static int metaObjectSignalCount(const QMetaObject *metaObject)
 {
     int signalCount = 0;
@@ -46,6 +172,8 @@ QQmlPropertyData::flagsForProperty(const QMetaProperty &p)
     flags.setIsWritable(p.isWritable());
     flags.setIsResettable(p.isResettable());
     flags.setIsFinal(p.isFinal());
+    flags.setIsVirtual(p.isVirtual());
+    flags.setDoesOverride(p.isOverride());
     flags.setIsRequired(p.isRequired());
     flags.setIsBindable(p.isBindable());
 
@@ -53,19 +181,13 @@ QQmlPropertyData::flagsForProperty(const QMetaProperty &p)
     const QMetaType metaType = p.metaType();
     int propType = metaType.id();
     if (p.isEnumType()) {
-        flags.type = QQmlPropertyData::Flags::EnumType;
+        flags.setType(QQmlPropertyData::Flags::EnumType);
     } else if (metaType.flags() & QMetaType::PointerToQObject) {
-        flags.type = QQmlPropertyData::Flags::QObjectDerivedType;
+        flags.setType(QQmlPropertyData::Flags::QObjectDerivedType);
     } else if (propType == QMetaType::QVariant) {
-        flags.type = QQmlPropertyData::Flags::QVariantType;
-    } else if (propType < static_cast<int>(QMetaType::User)) {
-        // nothing to do
-    } else if (propType == qMetaTypeId<QQmlBinding *>()) {
-        flags.type = QQmlPropertyData::Flags::QmlBindingType;
-    } else if (propType == qMetaTypeId<QJSValue>()) {
-        flags.type = QQmlPropertyData::Flags::QJSValueType;
+        flags.setType(QQmlPropertyData::Flags::QVariantType);
     } else if (metaType.flags() & QMetaType::IsQmlList) {
-        flags.type = QQmlPropertyData::Flags::QListType;
+        flags.setType(QQmlPropertyData::Flags::QListType);
     }
 
     return flags;
@@ -85,34 +207,52 @@ void QQmlPropertyData::load(const QMetaProperty &p)
 void QQmlPropertyData::load(const QMetaMethod &m)
 {
     setCoreIndex(m.methodIndex());
-    setArguments(nullptr);
+    m_flags.setType(Flags::FunctionType);
 
-    setPropType(m.returnMetaType());
+    // We need to set the constructor, signal, constant, arguments, V4Function, cloned flags.
+    // These are specific to methods and change with each method.
+    // The same QQmlPropertyData may be loaded with multiple methods in sequence.
 
-    m_flags.type = Flags::FunctionType;
-    if (m.methodType() == QMetaMethod::Signal) {
+    switch (m.methodType()) {
+    case QMetaMethod::Signal:
         m_flags.setIsSignal(true);
-    } else if (m.methodType() == QMetaMethod::Constructor) {
+        m_flags.setIsConstructor(false);
+        setPropType(m.returnMetaType());
+        break;
+    case QMetaMethod::Constructor:
+        m_flags.setIsSignal(false);
         m_flags.setIsConstructor(true);
-        setPropType(QMetaType::fromType<QObject *>());
+        break;
+    default:
+        m_flags.setIsSignal(false);
+        m_flags.setIsConstructor(false);
+        setPropType(m.returnMetaType());
+        break;
     }
+
     m_flags.setIsConstant(m.isConst());
 
     const int paramCount = m.parameterCount();
     if (paramCount) {
         m_flags.setHasArguments(true);
-        if ((paramCount == 1) && (m.parameterMetaType(0) == QMetaType::fromType<QQmlV4Function *>()))
-            m_flags.setIsV4Function(true);
+        m_flags.setIsV4Function(
+                paramCount == 1 &&
+                m.parameterMetaType(0) == QMetaType::fromType<QQmlV4FunctionPtr>());
+    } else {
+        m_flags.setHasArguments(false);
+        m_flags.setIsV4Function(false);
     }
 
-    if (m.attributes() & QMetaMethod::Cloned)
-        m_flags.setIsCloned(true);
+    m_flags.setIsCloned(m.attributes() & QMetaMethod::Cloned);
 
     Q_ASSERT(m.revision() <= std::numeric_limits<quint16>::max());
     setRevision(QTypeRevision::fromEncodedVersion(m.revision()));
 }
 
+Q_LOGGING_CATEGORY(qqmlPropertyCacheAppend, "qt.qml.propertyCache.append", QtWarningMsg)
+
 /*!
+    \internal
     Creates a standalone QQmlPropertyCache of \a metaObject. It is separate from the usual
     QQmlPropertyCache hierarchy. It's parent is not equal to any other QQmlPropertyCache
     created from QObject::staticMetaObject, for example.
@@ -164,7 +304,7 @@ QQmlPropertyCache::~QQmlPropertyCache()
 QQmlPropertyCache::Ptr QQmlPropertyCache::copy(const QQmlMetaObjectPointer &mo, int reserve) const
 {
     QQmlPropertyCache::Ptr cache = QQmlPropertyCache::Ptr(
-            new QQmlPropertyCache(mo), QQmlPropertyCache::Ptr::Adopt);
+            new QQmlPropertyCache(mo, _handleOverride), QQmlPropertyCache::Ptr::Adopt);
     cache->_parent.reset(this);
     cache->propertyIndexCacheStart = propertyIndexCache.size() + propertyIndexCacheStart;
     cache->methodIndexCacheStart = methodIndexCache.size() + methodIndexCacheStart;
@@ -194,31 +334,21 @@ QQmlPropertyCache::Ptr QQmlPropertyCache::copyAndReserve(
     return rv;
 }
 
-/*! \internal
-
-    \a notifyIndex MUST be in the signal index range (see QObjectPrivate::signalIndex()).
-    This is different from QMetaMethod::methodIndex().
-*/
-void QQmlPropertyCache::appendProperty(const QString &name, QQmlPropertyData::Flags flags,
-                                       int coreIndex, QMetaType propType, QTypeRevision version,
-                                       int notifyIndex)
+QQmlPropertyCache::AppendResult
+QQmlPropertyCache::appendAlias(const QString &name, QQmlPropertyData::Flags flags, int coreIndex,
+                               QMetaType propType, QTypeRevision version, int notifyIndex,
+                               int encodedTargetIndex)
 {
     QQmlPropertyData data;
     data.setPropType(propType);
     data.setCoreIndex(coreIndex);
     data.setNotifyIndex(notifyIndex);
+    flags.setIsAlias(true);
     data.setFlags(flags);
+    data.setAliasTarget(encodedTargetIndex);
     data.setTypeVersion(version);
 
-    const OverrideResult overrideResult = handleOverride(name, &data);
-    if (overrideResult == InvalidOverride)
-        return;
-
-    int index = propertyIndexCache.size();
-    propertyIndexCache.append(data);
-
-    setNamedProperty(name, index + propertyOffset(), propertyIndexCache.data() + index,
-                     overrideResult == ValidOverride);
+    return appendPropertyAttr(name, std::move(data));
 }
 
 void QQmlPropertyCache::appendSignal(const QString &name, QQmlPropertyData::Flags flags,
@@ -242,9 +372,24 @@ void QQmlPropertyCache::appendSignal(const QString &name, QQmlPropertyData::Flag
         data.setArguments(args);
     }
 
-    const OverrideResult overrideResult = handleOverride(name, &data);
-    if (overrideResult == InvalidOverride)
+    QQmlPropertyData *old = findNamedProperty(name);
+    const auto overrideStatus = _handleOverride(data, old, OverrideSemantics::CheckMode::Minimal);
+    maybeLog(overrideStatus, name);
+    // remove assert when checkMode is expanded and adjust handling correspondingly. For now it
+    // verifies that some code-path work in the same way as before introduction of virtual and
+    // override keywords
+    Q_ASSERT(overrideStatus == OverrideSemantics::Status::NoOverride
+             || overrideStatus == OverrideSemantics::Status::Valid
+             || overrideStatus == OverrideSemantics::Status::OverridingFinal);
+    if (overrideStatus == OverrideSemantics::Status::OverridingFinal) {
+        // TODO QTBUG-141728
+        // Insert the overridden member and its signal once more, to keep the counts in sync
+        methodIndexCache.append(*old);
+        handler = *old;
+        handler.m_flags.setIsSignalHandler(true);
+        signalHandlerIndexCache.append(handler);
         return;
+    }
 
     int methodIndex = methodIndexCache.size();
     methodIndexCache.append(data);
@@ -252,20 +397,17 @@ void QQmlPropertyCache::appendSignal(const QString &name, QQmlPropertyData::Flag
     int signalHandlerIndex = signalHandlerIndexCache.size();
     signalHandlerIndexCache.append(handler);
 
-    QString handlerName = QLatin1String("on") + name;
-    handlerName[2] = handlerName.at(2).toUpper();
+    const QString handlerName = QQmlSignalNames::signalNameToHandlerName(name);
 
-    setNamedProperty(name, methodIndex + methodOffset(), methodIndexCache.data() + methodIndex,
-                     overrideResult == ValidOverride);
+    setNamedProperty(name, methodIndex + methodOffset(), methodIndexCache.data() + methodIndex);
     setNamedProperty(handlerName, signalHandlerIndex + signalOffset(),
-                     signalHandlerIndexCache.data() + signalHandlerIndex,
-                     overrideResult == ValidOverride);
+                     signalHandlerIndexCache.data() + signalHandlerIndex);
 }
 
 void QQmlPropertyCache::appendMethod(const QString &name, QQmlPropertyData::Flags flags,
                                      int coreIndex, QMetaType returnType,
                                      const QList<QByteArray> &names,
-                                     const QVector<QMetaType> &parameterTypes)
+                                     const QList<QMetaType> &parameterTypes)
 {
     int argumentCount = names.size();
 
@@ -273,9 +415,21 @@ void QQmlPropertyCache::appendMethod(const QString &name, QQmlPropertyData::Flag
     data.setPropType(returnType);
     data.setCoreIndex(coreIndex);
     data.setFlags(flags);
-    const OverrideResult overrideResult = handleOverride(name, &data);
-    if (overrideResult == InvalidOverride)
+    QQmlPropertyData *old = findNamedProperty(name);
+    const auto overrideStatus = _handleOverride(data, old, OverrideSemantics::CheckMode::Minimal);
+    maybeLog(overrideStatus, name);
+    // remove assert when checkMode is expanded and adjust handling correspondingly. For now it
+    // verifies that some code-path work in the same way as before introduction of virtual and
+    // override keywords
+    Q_ASSERT(overrideStatus == OverrideSemantics::Status::NoOverride
+             || overrideStatus == OverrideSemantics::Status::Valid
+             || overrideStatus == OverrideSemantics::Status::OverridingFinal);
+    if (overrideStatus == OverrideSemantics::Status::OverridingFinal) {
+        // TODO QTBUG-141728
+        // Insert the overridden member once more, to keep the counts in sync
+        methodIndexCache.append(*old);
         return;
+    }
 
     QQmlPropertyCacheMethodArguments *args = createArgumentsObject(argumentCount, names);
     new (args->types) QMetaType(returnType);
@@ -286,11 +440,10 @@ void QQmlPropertyCache::appendMethod(const QString &name, QQmlPropertyData::Flag
     int methodIndex = methodIndexCache.size();
     methodIndexCache.append(data);
 
-    setNamedProperty(name, methodIndex + methodOffset(), methodIndexCache.data() + methodIndex,
-                     overrideResult == ValidOverride);
+    setNamedProperty(name, methodIndex + methodOffset(), methodIndexCache.data() + methodIndex);
 }
 
-void QQmlPropertyCache::appendEnum(const QString &name, const QVector<QQmlEnumValue> &values)
+void QQmlPropertyCache::appendEnum(const QString &name, const QList<QQmlEnumValue> &values)
 {
     QQmlEnumData data;
     data.name = name;
@@ -358,6 +511,28 @@ QQmlPropertyCache::copyAndAppend(const QMetaObject *metaObject,
     return rv;
 }
 
+static QHashedString signalNameToHandlerName(const QHashedString &methodName)
+{
+    return QQmlSignalNames::signalNameToHandlerName(methodName);
+}
+
+static QHashedString signalNameToHandlerName(const QHashedCStringRef &methodName)
+{
+    return QQmlSignalNames::signalNameToHandlerName(
+            QLatin1StringView{ methodName.constData(), methodName.length() });
+}
+
+static inline std::pair<bool, int> deriveEncodingAndLength(const char *str)
+{
+    char utf8 = 0;
+    const char *cptr = str;
+    while (*cptr != 0) {
+        utf8 |= *cptr & 0x80;
+        ++cptr;
+    }
+    return std::make_pair(utf8, cptr - str);
+}
+
 void QQmlPropertyCache::append(const QMetaObject *metaObject,
                                QTypeRevision typeVersion,
                                QQmlPropertyData::Flags propertyFlags,
@@ -410,20 +585,13 @@ void QQmlPropertyCache::append(const QMetaObject *metaObject,
     for (int ii = methodOffset; ii < methodCount; ++ii) {
         if (preventDestruction && (ii == destroyedIdx1 || ii == destroyedIdx2 || ii == deleteLaterIdx))
             continue;
-        QMetaMethod m = metaObject->method(ii);
+        const QMetaMethod &m = metaObject->method(ii);
         if (m.access() == QMetaMethod::Private)
             continue;
 
         // Extract method name
         // It's safe to keep the raw name pointer
         Q_ASSERT(QMetaObjectPrivate::get(metaObject)->revision >= 7);
-        const char *rawName = m.name().constData();
-        const char *cptr = rawName;
-        char utf8 = 0;
-        while (*cptr) {
-            utf8 |= *cptr & 0x80;
-            ++cptr;
-        }
 
         QQmlPropertyData *data = &methodIndexCache[ii - methodIndexCacheStart];
         QQmlPropertyData *sigdata = nullptr;
@@ -444,51 +612,51 @@ void QQmlPropertyCache::append(const QMetaObject *metaObject,
             sigdata->m_flags.setIsSignalHandler(true);
         }
 
-        QQmlPropertyData *old = nullptr;
-
-        if (utf8) {
-            QHashedString methodName(QString::fromUtf8(rawName, cptr - rawName));
+        const auto doSetNamedProperty = [&](const auto &methodName) {
+            QQmlPropertyData *old = nullptr;
             if (StringCache::mapped_type *it = stringCache.value(methodName)) {
-                if (handleOverride(methodName, data, (old = it->second)) == InvalidOverride)
-                    continue;
+                const auto overrideStatus = _handleOverride(*data, (old = it->second),
+                                                            OverrideSemantics::CheckMode::Minimal);
+                maybeLog(overrideStatus, methodName);
+                // remove assert when checkMode is expanded and adjust handling correspondingly. For
+                // now it verifies that some code-path work in the same way as before introduction
+                // of virtual and override keywords
+                Q_ASSERT(overrideStatus == OverrideSemantics::Status::NoOverride
+                         || overrideStatus == OverrideSemantics::Status::Valid
+                         || overrideStatus == OverrideSemantics::Status::OverridingFinal);
+                if (overrideStatus == OverrideSemantics::Status::OverridingFinal) {
+                    // TODO QTBUG-141728
+                    *data = *old;
+                    if (sigdata) {
+                        // Keep the signal counts in sync,
+                        // even if the "old" data has no real signal.
+                        *sigdata = *old;
+                        sigdata->m_flags.setIsSignalHandler(true);
+                        ++signalHandlerIndex;
+                    }
+                    return;
+                }
             }
-            setNamedProperty(methodName, ii, data, (old != nullptr));
+
+            setNamedProperty(methodName, ii, data);
 
             if (data->isSignal()) {
-                QHashedString on(QLatin1String("on") % methodName.at(0).toUpper() % QStringView{methodName}.mid(1));
-                setNamedProperty(on, ii, sigdata, (old != nullptr));
+
+                // TODO: Remove this once we can. Signals should not be overridable.
+                if constexpr (std::is_same_v<std::decay_t<decltype(methodName)>, QHashedCStringRef>)
+                    data->m_flags.setIsOverridableSignal(true);
+
+                setNamedProperty(signalNameToHandlerName(methodName), ii, sigdata);
                 ++signalHandlerIndex;
             }
-        } else {
-            QHashedCStringRef methodName(rawName, cptr - rawName);
-            if (StringCache::mapped_type *it = stringCache.value(methodName)) {
-                if (handleOverride(methodName, data, (old = it->second)) == InvalidOverride)
-                    continue;
-            }
-            setNamedProperty(methodName, ii, data, (old != nullptr));
+        };
 
-            if (data->isSignal()) {
-                int length = methodName.length();
-
-                QVarLengthArray<char, 128> str(length+3);
-                str[0] = 'o';
-                str[1] = 'n';
-                str[2] = QtMiscUtils::toAsciiUpper(rawName[0]);
-                if (length > 1)
-                    memcpy(&str[3], &rawName[1], length - 1);
-                str[length + 2] = '\0';
-
-                QHashedString on(QString::fromLatin1(str.data()));
-                setNamedProperty(on, ii, data, (old != nullptr));
-                ++signalHandlerIndex;
-            }
-        }
-
-        if (old) {
-            // We only overload methods in the same class, exactly like C++
-            if (old->isFunction() && old->coreIndex() >= methodOffset)
-                data->m_flags.setIsOverload(true);
-        }
+        const char *str = m.nameView().constData();
+        const auto [isUtf8, len] = deriveEncodingAndLength(str);
+        if (isUtf8)
+            doSetNamedProperty(QHashedString(QString::fromUtf8(str, len)));
+        else
+            doSetNamedProperty(QHashedCStringRef(str, len));
     }
 
     int propCount = metaObject->propertyCount();
@@ -502,14 +670,7 @@ void QQmlPropertyCache::append(const QMetaObject *metaObject,
         if (!p.isScriptable())
             continue;
 
-        const char *str = p.name();
-        char utf8 = 0;
-        const char *cptr = str;
-        while (*cptr != 0) {
-            utf8 |= *cptr & 0x80;
-            ++cptr;
-        }
-
+        // TODO QTBUG-141728
         QQmlPropertyData *data = &propertyIndexCache[ii - propertyIndexCacheStart];
 
         data->setFlags(propertyFlags);
@@ -519,23 +680,28 @@ void QQmlPropertyCache::append(const QMetaObject *metaObject,
         Q_ASSERT((allowedRevisionCache.size() - 1) < Q_INT16_MAX);
         data->setMetaObjectOffset(allowedRevisionCache.size() - 1);
 
-        QQmlPropertyData *old = nullptr;
+        const auto doSetNamedProperty = [this](const auto &propName, int index, auto *propData) {
+            QQmlPropertyData *existingPropData = findNamedProperty(propName);
+            const auto overrideStatus = _handleOverride(*propData, existingPropData,
+                                                        OverrideSemantics::CheckMode::Full);
+            maybeLog(overrideStatus, propName);
+            if (!OverrideSemantics::isValidOverride(overrideStatus)) {
+                if (existingPropData) {
+                    // TODO QTBUG-141728
+                    *propData = *existingPropData;
+                }
+                return;
+            }
 
-        if (utf8) {
-            QHashedString propName(QString::fromUtf8(str, cptr - str));
-            if (StringCache::mapped_type *it = stringCache.value(propName)) {
-                if (handleOverride(propName, data, (old = it->second)) == InvalidOverride)
-                    continue;
-            }
-            setNamedProperty(propName, ii, data, (old != nullptr));
-        } else {
-            QHashedCStringRef propName(str, cptr - str);
-            if (StringCache::mapped_type *it = stringCache.value(propName)) {
-                if (handleOverride(propName, data, (old = it->second)) == InvalidOverride)
-                    continue;
-            }
-            setNamedProperty(propName, ii, data, (old != nullptr));
-        }
+            setNamedProperty(propName, index, propData);
+        };
+
+        const char *str = p.name();
+        const auto [isUtf8, len] = deriveEncodingAndLength(str);
+        if (isUtf8)
+            doSetNamedProperty(QHashedString(QString::fromUtf8(str, len)), ii, data);
+        else
+            doSetNamedProperty(QHashedCStringRef(str, len), ii, data);
 
         bool isGadget = true;
         for (const QMetaObject *it = metaObject; it != nullptr; it = it->superClass()) {
@@ -544,7 +710,7 @@ void QQmlPropertyCache::append(const QMetaObject *metaObject,
         }
 
         // otherwise always dispatch over a 'normal' meta-call so the QQmlValueType can intercept
-        if (!isGadget)
+        if (!isGadget && !data->isAlias())
             data->trySetStaticMetaCallFunction(metaObject->d.static_metacall, ii - propOffset);
     }
 }
@@ -582,7 +748,6 @@ void QQmlPropertyCache::invalidate(const QMetaObject *metaObject)
     methodIndexCache.clear();
     signalHandlerIndexCache.clear();
 
-    _hasPropertyOverrides = false;
     argumentsCache = nullptr;
 
     int pc = metaObject->propertyCount();
@@ -690,40 +855,42 @@ const QQmlPropertyData *QQmlPropertyCache::findProperty(
     return nullptr;
 }
 
-QString QQmlPropertyData::name(QObject *object) const
+// Note, this function is called when adding aliases, hence data.isEnum() can possibly be true
+QQmlPropertyCache::AppendResult QQmlPropertyCache::appendPropertyAttr(const QString &name,
+                                                                      QQmlPropertyData &&data)
 {
-    if (!object)
-        return QString();
-
-    return name(object->metaObject());
-}
-
-QString QQmlPropertyData::name(const QMetaObject *metaObject) const
-{
-    if (!metaObject || coreIndex() == -1)
-        return QString();
-
-    if (isFunction()) {
-        QMetaMethod m = metaObject->method(coreIndex());
-
-        return QString::fromUtf8(m.name().constData());
-    } else {
-        QMetaProperty p = metaObject->property(coreIndex());
-        return QString::fromUtf8(p.name());
+    QQmlPropertyData *old = findNamedProperty(name);
+    const auto overrideStatus = _handleOverride(data, old, OverrideSemantics::CheckMode::Full);
+    maybeLog(overrideStatus, name);
+    if (!OverrideSemantics::isValidOverride(overrideStatus)) {
+        // TODO QTBUG-141728
+        // Insert the overridden member once more, to keep the counts in sync
+        propertyIndexCache.append(old ? *old : data);
+        return q23::make_unexpected(overrideStatus);
     }
+
+    const int index = propertyIndexCache.size();
+    propertyIndexCache.append(std::move(data));
+
+    setNamedProperty(name, index + propertyOffset(), propertyIndexCache.data() + index);
+    return {};
 }
 
-bool QQmlPropertyData::markAsOverrideOf(QQmlPropertyData *predecessor)
+void QQmlPropertyData::markAsOverrideOf(QQmlPropertyData *predecessor)
 {
     Q_ASSERT(predecessor != this);
-    if (predecessor->isFinal())
-        return false;
+
+    if (!predecessor) {
+        return;
+    }
 
     setOverrideIndexIsProperty(!predecessor->isFunction());
     setOverrideIndex(predecessor->coreIndex());
+    // propagate "virtuality"
+    m_flags.setIsVirtual(predecessor->isVirtual());
     predecessor->m_flags.setIsOverridden(true);
     Q_ASSERT(predecessor->isOverridden());
-    return true;
+    return;
 }
 
 QQmlPropertyCacheMethodArguments *QQmlPropertyCache::createArgumentsObject(
@@ -737,23 +904,27 @@ QQmlPropertyCacheMethodArguments *QQmlPropertyCache::createArgumentsObject(
     return args;
 }
 
-QString QQmlPropertyCache::signalParameterStringForJS(QV4::ExecutionEngine *engine, const QList<QByteArray> &parameterNameList, QString *errorString)
+QString QQmlPropertyCache::signalParameterStringForJS(
+        const QList<QByteArray> &parameterNameList, QString *errorString)
 {
     bool unnamedParameter = false;
-    const QSet<QString> &illegalNames = engine->illegalNames();
     QString parameters;
 
-    for (int i = 0; i < parameterNameList.size(); ++i) {
+    const qsizetype count = parameterNameList.size();
+    if (count > std::numeric_limits<quint16>::max())
+        *errorString = QCoreApplication::translate("QQmlRewrite", "Signal has an excessive number of parameters: %1").arg(count);
+
+    for (qsizetype i = 0; i < count; ++i) {
         if (i > 0)
             parameters += QLatin1Char(',');
         const QByteArray &param = parameterNameList.at(i);
-        if (param.isEmpty())
+        if (param.isEmpty()) {
             unnamedParameter = true;
-        else if (unnamedParameter) {
+        } else if (unnamedParameter) {
             if (errorString)
                 *errorString = QCoreApplication::translate("QQmlRewrite", "Signal uses unnamed parameter followed by named parameter.");
             return QString();
-        } else if (illegalNames.contains(QString::fromUtf8(param))) {
+        } else if (QV4::Compiler::Codegen::isNameGlobal(param)) {
             if (errorString)
                 *errorString = QCoreApplication::translate("QQmlRewrite", "Signal parameter \"%1\" hides global variable.").arg(QString::fromUtf8(param));
             return QString();
@@ -932,14 +1103,14 @@ const char *QQmlPropertyCache::className() const
 
 void QQmlPropertyCache::toMetaObjectBuilder(QMetaObjectBuilder &builder) const
 {
-    struct Sort { static bool lt(const QPair<QString, const QQmlPropertyData *> &lhs,
-                                 const QPair<QString, const QQmlPropertyData *> &rhs) {
+    struct Sort { static bool lt(const std::pair<QString, const QQmlPropertyData *> &lhs,
+                                 const std::pair<QString, const QQmlPropertyData *> &rhs) {
         return lhs.second->coreIndex() < rhs.second->coreIndex();
     } };
 
     struct Insert { static void in(const QQmlPropertyCache *This,
-                                   QList<QPair<QString, const QQmlPropertyData *> > &properties,
-                                   QList<QPair<QString, const QQmlPropertyData *> > &methods,
+                                   QList<std::pair<QString, const QQmlPropertyData *> > &properties,
+                                   QList<std::pair<QString, const QQmlPropertyData *> > &methods,
                                    StringCache::ConstIterator iter, const QQmlPropertyData *data) {
         if (data->isSignalHandler())
             return;
@@ -948,7 +1119,7 @@ void QQmlPropertyCache::toMetaObjectBuilder(QMetaObjectBuilder &builder) const
             if (data->coreIndex() < This->methodIndexCacheStart)
                 return;
 
-            QPair<QString, const QQmlPropertyData *> entry = qMakePair((QString)iter.key(), data);
+            std::pair<QString, const QQmlPropertyData *> entry = std::make_pair((QString)iter.key(), data);
             // Overrides can cause the entry to already exist
             if (!methods.contains(entry)) methods.append(entry);
 
@@ -958,7 +1129,7 @@ void QQmlPropertyCache::toMetaObjectBuilder(QMetaObjectBuilder &builder) const
             if (data->coreIndex() < This->propertyIndexCacheStart)
                 return;
 
-            QPair<QString, const QQmlPropertyData *> entry = qMakePair((QString)iter.key(), data);
+            std::pair<QString, const QQmlPropertyData *> entry = std::make_pair((QString)iter.key(), data);
             // Overrides can cause the entry to already exist
             if (!properties.contains(entry)) properties.append(entry);
 
@@ -970,14 +1141,16 @@ void QQmlPropertyCache::toMetaObjectBuilder(QMetaObjectBuilder &builder) const
 
     builder.setClassName(_dynamicClassName);
 
-    QList<QPair<QString, const QQmlPropertyData *> > properties;
-    QList<QPair<QString, const QQmlPropertyData *> > methods;
+    QList<std::pair<QString, const QQmlPropertyData *> > properties;
+    QList<std::pair<QString, const QQmlPropertyData *> > methods;
 
     for (StringCache::ConstIterator iter = stringCache.begin(), cend = stringCache.end(); iter != cend; ++iter)
         Insert::in(this, properties, methods, iter, iter.value().second);
 
-    Q_ASSERT(properties.size() == propertyIndexCache.size());
-    Q_ASSERT(methods.size() == methodIndexCache.size());
+    // Any invalid overrides are not linked by name into the properties and methods hashes.
+    // Therefore there can be more properties and methods than present in the hashes.
+    Q_ASSERT(properties.size() <= propertyIndexCache.size());
+    Q_ASSERT(methods.size() <= methodIndexCache.size());
 
     std::sort(properties.begin(), properties.end(), Sort::lt);
     std::sort(methods.begin(), methods.end(), Sort::lt);
@@ -997,7 +1170,7 @@ void QQmlPropertyCache::toMetaObjectBuilder(QMetaObjectBuilder &builder) const
         property.setReadable(true);
         property.setWritable(data->isWritable());
         property.setResettable(data->isResettable());
-        property.setBindable(data->isBindable());
+        property.setBindable(data->notifiesViaBindable());
         property.setAlias(data->isAlias());
     }
 
@@ -1069,14 +1242,18 @@ int visitMethods(const QMetaObject &mo, int methodOffset, int methodCount,
 {
     int fieldsForParameterData = 0;
 
-    bool hasRevisionedMethods = false;
+    bool hasOldStyleRevisionedMethods = false;
 
     for (int i = 0; i < methodCount; ++i) {
         const int handle = methodOffset + i * QMetaObjectPrivate::IntsPerMethod;
 
         const uint flags = mo.d.data[handle + 4];
-        if (flags & MethodRevisioned)
-            hasRevisionedMethods = true;
+        if (flags & MethodRevisioned) {
+            if (mo.d.data[0] < 13)
+                hasOldStyleRevisionedMethods = true;
+            else
+                fieldsForParameterData += 1;    // revision
+        }
 
         visitString(mo.d.data[handle + 0]); // name
         visitString(mo.d.data[handle + 3]); // tag
@@ -1099,7 +1276,7 @@ int visitMethods(const QMetaObject &mo, int methodOffset, int methodCount,
     }
 
     int fieldsForRevisions = 0;
-    if (hasRevisionedMethods)
+    if (hasOldStyleRevisionedMethods)
         fieldsForRevisions = methodCount;
 
     return methodCount * QMetaObjectPrivate::IntsPerMethod
@@ -1189,13 +1366,13 @@ int countMetaObjectFields(const QMetaObject &mo, StringVisitor stringVisitor)
 
 } // anonymous namespace
 
-static_assert(QMetaObjectPrivate::OutputRevision == 10, "Check and adjust determineMetaObjectSizes");
+static_assert(QMetaObjectPrivate::OutputRevision == 13, "Check and adjust determineMetaObjectSizes");
 
 bool QQmlPropertyCache::determineMetaObjectSizes(const QMetaObject &mo, int *fieldCount,
                                                  int *stringCount)
 {
     const QMetaObjectPrivate *priv = reinterpret_cast<const QMetaObjectPrivate*>(mo.d.data);
-    if (priv->revision != 10)
+    if (priv->revision != QMetaObjectPrivate::OutputRevision)
         return false;
 
     uint highestStringIndex = 0;
