@@ -1,9 +1,11 @@
 // Copyright (C) 2020 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// Qt-Security score:significant
 
 #include "qqmljscompiler_p.h"
 
 #include <private/qqmlirbuilder_p.h>
+#include <private/qqmljsaotirbuilder_p.h>
 #include <private/qqmljsbasicblocks_p.h>
 #include <private/qqmljscodegenerator_p.h>
 #include <private/qqmljscompilerstats_p.h>
@@ -21,6 +23,7 @@
 #include <QtCore/qfile.h>
 #include <QtCore/qfileinfo.h>
 #include <QtCore/qloggingcategory.h>
+#include <QtCore/qelapsedtimer.h>
 
 #include <QtQml/private/qqmlsignalnames_p.h>
 
@@ -169,7 +172,7 @@ private:
     const QmlIR::Function *m_function = nullptr;
 };
 
-bool qCompileQmlFile(const QString &inputFileName, QQmlJSSaveFunction saveFunction,
+bool qCompileQmlFile(const QString &inputFileName, const QQmlJSSaveFunction &saveFunction,
                      QQmlJSAotCompiler *aotCompiler, QQmlJSCompileError *error,
                      bool storeSourceLocation, QV4::Compiler::CodegenWarningInterface *wInterface,
                      const QString *fileContents)
@@ -180,7 +183,7 @@ bool qCompileQmlFile(const QString &inputFileName, QQmlJSSaveFunction saveFuncti
 }
 
 bool qCompileQmlFile(QmlIR::Document &irDocument, const QString &inputFileName,
-                     QQmlJSSaveFunction saveFunction, QQmlJSAotCompiler *aotCompiler,
+                     const QQmlJSSaveFunction &saveFunction, QQmlJSAotCompiler *aotCompiler,
                      QQmlJSCompileError *error, bool storeSourceLocation,
                      QV4::Compiler::CodegenWarningInterface *wInterface, const QString *fileContents)
 {
@@ -202,9 +205,11 @@ bool qCompileQmlFile(QmlIR::Document &irDocument, const QString &inputFileName,
     }
 
     {
-        QmlIR::IRBuilder irBuilder;
-        if (!irBuilder.generateFromQml(sourceCode, inputFileName, &irDocument)) {
-            error->appendDiagnostics(inputFileName, irBuilder.errors);
+        // For now, only use the AOT IRBuilder when linting
+        std::unique_ptr<QmlIR::IRBuilder> irBuilder = aotCompiler && aotCompiler->isLintCompiler()
+                ? std::make_unique<QQmlJSAOTIRBuilder>() : std::make_unique<QmlIR::IRBuilder>();
+        if (!irBuilder->generateFromQml(sourceCode, inputFileName, &irDocument)) {
+            error->appendDiagnostics(inputFileName, irBuilder->errors);
             return false;
         }
     }
@@ -319,6 +324,9 @@ bool qCompileQmlFile(QmlIR::Document &irDocument, const QString &inputFileName,
                                            << irDocument.stringAt(binding->propertyNameIndex);
                     result = aotCompiler->compileBinding(context, *binding, node);
                 } else if (const auto *function = bindingOrFunction.function()) {
+                    if (!aotCompiler->isLintCompiler() && !function->isQmlFunction)
+                        return;
+
                     Q_ASSERT(quint32(functionsToCompile.size()) > function->index);
                     auto *node = functionsToCompile[function->index].node;
                     Q_ASSERT(node);
@@ -371,8 +379,8 @@ bool qCompileQmlFile(QmlIR::Document &irDocument, const QString &inputFileName,
 }
 
 bool qCompileJSFile(
-        const QString &inputFileName, const QString &inputFileUrl, QQmlJSSaveFunction saveFunction,
-        QQmlJSCompileError *error)
+        const QString &inputFileName, const QString &inputFileUrl,
+        const QQmlJSSaveFunction &saveFunction, QQmlJSCompileError *error)
 {
     Q_UNUSED(inputFileUrl);
 
@@ -551,14 +559,6 @@ bool qSaveQmlJSUnitAsCpp(const QString &inputFileName, const QString &outputFile
     if (!writeStr("};\n"))
         return false;
 
-    // Suppress the following warning generated by MSVC 2019:
-    //     "the usage of 'QJSNumberCoercion::toInteger' requires the compiler to capture 'this'
-    //      but the current default capture mode does not allow it"
-    // You clearly don't have to capture 'this' in order to call 'QJSNumberCoercion::toInteger'.
-    // TODO: Remove when we don't have to support MSVC 2019 anymore. Mind the QT_WARNING_POP below.
-    if (!writeStr("QT_WARNING_PUSH\nQT_WARNING_DISABLE_MSVC(4573)\n"))
-        return false;
-
     writeStr(aotFunctions[FileScopeCodeIndex].code.toUtf8().constData());
     if (aotFunctions.size() <= 1) {
         // FileScopeCodeIndex is always there, but it may be the only one.
@@ -583,7 +583,7 @@ bool qSaveQmlJSUnitAsCpp(const QString &inputFileName, const QString &outputFile
                                     "QMetaType *argTypes) {\n%3}, %4 },")
                      .arg(func.key())
                      .arg(func->numArguments)
-                     .arg(func->signature, function)
+                     .arg(func->signature.isEmpty() ? u"    Q_UNUSED(contextUnit);\n    Q_UNUSED(argTypes);\n"_s : func->signature, function)
                      .toUtf8().constData());
         }
 
@@ -592,9 +592,6 @@ bool qSaveQmlJSUnitAsCpp(const QString &inputFileName, const QString &outputFile
         writeStr("};\n");
     }
 
-    if (!writeStr("QT_WARNING_POP\n"))
-        return false;
-
     if (!writeStr("}\n}\n"))
         return false;
 
@@ -629,8 +626,7 @@ void QQmlJSAotCompiler::setDocument(
         m_logger->setFilePath(resourcePathInfo.fileName());
     m_logger->setCode(irDocument->code);
     m_unitGenerator = &irDocument->jsGenerator;
-    QQmlJSScope::Ptr target = QQmlJSScope::create();
-    QQmlJSImportVisitor visitor(target, m_importer, m_logger,
+    QQmlJSImportVisitor visitor(m_importer, m_logger,
                                 resourcePathInfo.canonicalPath() + u'/',
                                 m_qmldirFiles);
     m_typeResolver.init(&visitor, irDocument->program);
@@ -802,15 +798,16 @@ QQmlJSAotFunction QQmlJSAotCompiler::doCompileAndRecordAotStats(
         const QV4::Compiler::Context *context, QQmlJSCompilePass::Function *function,
         const QString &name, QQmlJS::SourceLocation location)
 {
-    auto t1 = std::chrono::high_resolution_clock::now();
+    QElapsedTimer timer {};
+    timer.start();
     QQmlJSAotFunction result;
     if (!m_logger->currentFunctionHasCompileError())
         result = doCompile(context, function);
-    auto t2 = std::chrono::high_resolution_clock::now();
+    auto elapsed = std::chrono::milliseconds { timer.elapsed() };
 
     if (QQmlJS::QQmlJSAotCompilerStats::recordAotStats()) {
         QQmlJS::AotStatsEntry entry;
-        entry.codegenDuration = std::chrono::duration_cast<std::chrono::microseconds>(t2 - t1);
+        entry.codegenDuration = elapsed;
         entry.functionName = name;
         entry.message = m_logger->currentFunctionWasSkipped()
                 ? m_logger->currentFunctionCompileSkipMessage()
