@@ -1,5 +1,6 @@
 // Copyright (C) 2020 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// Qt-Security score:significant
 
 #include "qqmljsimporter_p.h"
 #include "qqmljstypedescriptionreader_p.h"
@@ -163,6 +164,10 @@ static QStringList aliases(const QQmlJSScope::ConstPtr &scope)
         : scope->aliases();
 }
 
+/*!
+    \class QQmlJSImporter
+    \internal
+*/
 QQmlJSImporter::QQmlJSImporter(const QStringList &importPaths, QQmlJSResourceFileMapper *mapper,
                                QQmlJSImporterFlags flags)
     : m_importPaths(importPaths),
@@ -289,7 +294,7 @@ QQmlJSImporter::Import QQmlJSImporter::readQmldir(const QString &modulePath)
 
         auto mo = qmlComponents.find(it->fileName);
         if (mo == qmlComponents.end()) {
-            QQmlJSScope::Ptr imported = localFile2ScopeTree(filePath);
+            QQmlJSScope::Ptr imported = localFile2QQmlJSScope(filePath);
             if (auto *factory = imported.factory()) {
                 if (it->singleton) {
                     factory->setIsSingleton(true);
@@ -309,7 +314,7 @@ QQmlJSImporter::Import QQmlJSImporter::readQmldir(const QString &modulePath)
         const QString filePath = resolvedPath + script.fileName;
         auto mo = result.scripts.find(script.fileName);
         if (mo == result.scripts.end())
-            mo = result.scripts.insert(script.fileName, { localFile2ScopeTree(filePath), {} });
+            mo = result.scripts.insert(script.fileName, { localFile2QQmlJSScope(filePath), {} });
 
         mo->exports.append(QQmlJSScope::Export(
                                reader.typeNamespace(), script.nameSpace,
@@ -329,7 +334,7 @@ QQmlJSImporter::Import QQmlJSImporter::readDirectory(const QString &directory)
                 const QString name = QFileInfo(entry.resourcePath).baseName();
                 if (name.front().isUpper()) {
                     import.objects.append({
-                        localFile2ScopeTree(entry.filePath),
+                        localFile2QQmlJSScope(entry.filePath),
                         { QQmlJSScope::Export(QString(), name, QTypeRevision(), QTypeRevision()) }
                     });
                 }
@@ -342,13 +347,17 @@ QQmlJSImporter::Import QQmlJSImporter::readDirectory(const QString &directory)
         return import;
     }
 
-    QDirIterator it {
-        directory,
-        QStringList() << QLatin1String("*.qml"),
-        QDir::NoFilter
-    };
-    while (it.hasNext()) {
-        QString name = it.nextFileInfo().completeBaseName();
+    // The engine doesn't really care whether a file is hidden, so we need to include them
+    QDirListing::IteratorFlags listingFlags = QDirListing::IteratorFlag::Default
+            | QDirListing::IteratorFlag::IncludeHidden
+            | QDirListing::IteratorFlag::FilesOnly;
+    QDirListing dirListing(
+            directory,
+            QStringList() << QLatin1String("*.qml"),
+            listingFlags
+    );
+    for (const QDirListing::DirEntry &entry: dirListing) {
+        QString name = entry.completeBaseName();
 
         // Non-uppercase names cannot be imported anyway.
         if (!name.front().isUpper())
@@ -363,7 +372,7 @@ QQmlJSImporter::Import QQmlJSImporter::readDirectory(const QString &directory)
             continue;
 
         import.objects.append({
-                localFile2ScopeTree(it.filePath()),
+                localFile2QQmlJSScope(entry.filePath()),
                 { QQmlJSScope::Export(QString(), name, QTypeRevision(), QTypeRevision()) }
         });
     }
@@ -919,7 +928,7 @@ bool QQmlJSImporter::importHelper(const QString &module, AvailableTypes *types,
     return false;
 }
 
-QQmlJSScope::Ptr QQmlJSImporter::localFile2ScopeTree(const QString &filePath)
+QQmlJSScope::Ptr QQmlJSImporter::localFile2QQmlJSScope(const QString &filePath)
 {
     const QString sourceFolderFile = preferQmlFilesFromSourceFolder()
             ? QQmlJSUtils::qmlSourcePathFromBuildPath(m_mapper, filePath)
@@ -957,7 +966,7 @@ bool QQmlJSImporter::registerScope(const QQmlJSScope::Ptr &scope)
 
 QQmlJSScope::Ptr QQmlJSImporter::importFile(const QString &file)
 {
-    return localFile2ScopeTree(file);
+    return localFile2QQmlJSScope(file);
 }
 
 QQmlJSImporter::ImportedTypes QQmlJSImporter::importDirectory(
@@ -972,6 +981,9 @@ QQmlJSImporter::ImportedTypes QQmlJSImporter::importDirectory(
 
 void QQmlJSImporter::setImportPaths(const QStringList &importPaths)
 {
+    if (m_importPaths == importPaths)
+        return;
+
     m_importPaths = importPaths;
 
     // We have to get rid off all cache elements directly referencing modules, since changing
