@@ -1,5 +1,6 @@
 // Copyright (C) 2020 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// Qt-Security score:significant
 
 #include "qmetatypesjsonprocessor_p.h"
 
@@ -624,7 +625,7 @@ void MetaTypesJsonProcessor::addRelatedTypes()
     }
 }
 
-void MetaTypesJsonProcessor::sortTypes(QVector<MetaType> &types)
+void MetaTypesJsonProcessor::sortTypes(QList<MetaType> &types)
 {
     std::sort(types.begin(), types.end(), qualifiedClassNameLessThan);
 }
@@ -760,6 +761,8 @@ Property::Property(const QCborMap &cbor)
     , lineNumber(cbor[S_LINENUMBER].toInteger(0))
     , revision(getRevision(cbor))
     , isFinal(cbor[S_FINAL].toBool())
+    , isVirtual(cbor[S_VIRTUAL].toBool())
+    , isOverride(cbor[S_OVERRIDE].toBool())
     , isConstant(cbor[S_CONSTANT].toBool())
     , isRequired(cbor[S_REQUIRED].toBool())
 {
