@@ -1,5 +1,6 @@
 // Copyright (C) 2019 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// Qt-Security score:significant
 
 #include "qqmltyperegistrarutils_p.h"
 #include "qqmltypesclassdescription_p.h"
@@ -70,12 +71,12 @@ MetaType FoundType::select(const MetaType &category, QAnyStringView relation) co
 }
 
 FoundType QmlTypesClassDescription::findType(
-        const QVector<MetaType> &types, const QVector<MetaType> &foreign,
+        const QList<MetaType> &types, const QList<MetaType> &foreign,
         const QAnyStringView &name, const QList<QAnyStringView> &namespaces)
 {
     const auto tryFindType = [&](QAnyStringView qualifiedName) -> FoundType {
         FoundType result;
-        for (const QVector<MetaType> &t : {types, foreign}) {
+        for (const QList<MetaType> &t : {types, foreign}) {
             const auto [first, last] = std::equal_range(
                     t.begin(), t.end(), qualifiedName, Compare());
             for (auto it = first; it != last; ++it) {
@@ -129,8 +130,8 @@ FoundType QmlTypesClassDescription::findType(
 }
 
 void QmlTypesClassDescription::collectSuperClasses(
-        const MetaType &classDef, const QVector<MetaType> &types,
-        const QVector<MetaType> &foreign, CollectMode mode,  QTypeRevision defaultRevision)
+        const MetaType &classDef, const QList<MetaType> &types,
+        const QList<MetaType> &foreign, CollectMode mode,  QTypeRevision defaultRevision)
 {
     const QList<QAnyStringView> namespaces = MetaTypesJsonProcessor::namespaces(classDef);
     QAnyStringView superClassCandidate;
@@ -175,10 +176,11 @@ void QmlTypesClassDescription::handleRegisterEnumClassesUnscoped(
 }
 
 void QmlTypesClassDescription::collectLocalAnonymous(
-        const MetaType &classDef, const QVector<MetaType> &types,
-        const QVector<MetaType> &foreign, QTypeRevision defaultRevision)
+        const MetaType &classDef, const QList<MetaType> &types,
+        const QList<MetaType> &foreign, QTypeRevision defaultRevision)
 {
     file = classDef.inputFile();
+    lineNumber = classDef.lineNumber();
 
     resolvedClass = classDef;
     className = classDef.qualifiedClassName();
@@ -210,11 +212,13 @@ void QmlTypesClassDescription::collectLocalAnonymous(
 }
 
 void QmlTypesClassDescription::collect(
-        const MetaType &classDef, const QVector<MetaType> &types,
-        const QVector<MetaType> &foreign, CollectMode mode, QTypeRevision defaultRevision)
+        const MetaType &classDef, const QList<MetaType> &types,
+        const QList<MetaType> &foreign, CollectMode mode, QTypeRevision defaultRevision)
 {
-    if (file.isEmpty())
+    if (file.isEmpty()) {
         file = classDef.inputFile();
+        lineNumber = classDef.lineNumber();
+    }
 
     const QAnyStringView classDefName = classDef.className();
     const QList<QAnyStringView> namespaces = MetaTypesJsonProcessor::namespaces(classDef);
@@ -463,7 +467,7 @@ void QmlTypesClassDescription::collect(
 }
 
 FoundType QmlTypesClassDescription::collectRelated(
-        QAnyStringView related, const QVector<MetaType> &types, const QVector<MetaType> &foreign,
+        QAnyStringView related, const QList<MetaType> &types, const QList<MetaType> &foreign,
         QTypeRevision defaultRevision, const QList<QAnyStringView> &namespaces)
 {
     if (FoundType other = findType(types, foreign, related, namespaces)) {
