@@ -44,7 +44,7 @@ QT_BEGIN_NAMESPACE
     \inqmlmodule QtQml.XmlListModel
     \brief For specifying a role to an \l XmlListModel.
 
-    \sa {All QML Types}{Qt QML}
+    \sa {All QML Types}{Qt Qml}
 */
 
 /*!
@@ -594,7 +594,7 @@ qreal QQmlXmlListModel::progress() const
 }
 
 /*!
-    \qmlmethod QtQml.XmlListModel::XmlListModel::errorString()
+    \qmlmethod string QtQml.XmlListModel::XmlListModel::errorString()
 
     Returns a string description of the last error that occurred
     if \l status is \l {XmlListModel}.Error.
@@ -616,7 +616,7 @@ void QQmlXmlListModel::componentComplete()
 }
 
 /*!
-    \qmlmethod QtQml.XmlListModel::XmlListModel::reload()
+    \qmlmethod void QtQml.XmlListModel::XmlListModel::reload()
 
     Reloads the model.
 */
@@ -680,23 +680,9 @@ void QQmlXmlListModel::reload()
     }
 }
 
-#define XMLLISTMODEL_MAX_REDIRECT 16
-
 #if QT_CONFIG(qml_network)
 void QQmlXmlListModel::requestFinished()
 {
-    m_redirectCount++;
-    if (m_redirectCount < XMLLISTMODEL_MAX_REDIRECT) {
-        QVariant redirect = m_reply->attribute(QNetworkRequest::RedirectionTargetAttribute);
-        if (redirect.isValid()) {
-            QUrl url = m_reply->url().resolved(redirect.toUrl());
-            deleteReply();
-            setSource(url);
-            return;
-        }
-    }
-    m_redirectCount = 0;
-
     if (m_reply->error() != QNetworkReply::NoError) {
         m_errorString = m_reply->errorString();
         deleteReply();
@@ -903,14 +889,14 @@ void QQmlXmlListModelQueryRunnable::processElement(QQmlXmlListModelQueryResult *
     readSubTree(QString(), reader, results, &currentResult->errors);
 
     if (reader.hasError())
-        currentResult->errors.push_back(qMakePair(this, reader.errorString()));
+        currentResult->errors.push_back(std::make_pair(this, reader.errorString()));
 
     currentResult->data << results;
 }
 
 void QQmlXmlListModelQueryRunnable::readSubTree(const QString &prefix, QXmlStreamReader &reader,
                                                 QFlatMap<int, QString> &results,
-                                                QList<QPair<void *, QString>> *errors)
+                                                QList<std::pair<void *, QString>> *errors)
 {
     const QStringList &elementNames = m_job.elementNames;
     const QStringList &attributes = m_job.elementAttributes;
@@ -939,7 +925,7 @@ void QQmlXmlListModelQueryRunnable::readSubTree(const QString &prefix, QXmlStrea
                         if (elementAttributes.hasAttribute(attribute)) {
                             roleResult = elementAttributes.value(attributes.at(index)).toString();
                         } else {
-                            errors->push_back(qMakePair(m_job.roleQueryErrorId.at(index),
+                            errors->push_back(std::make_pair(m_job.roleQueryErrorId.at(index),
                                                         QLatin1String("Attribute %1 not found")
                                                                 .arg(attributes[index])));
                         }
