@@ -1,41 +1,6 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtQuick module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #include <private/qsgadaptationlayer_p.h>
 #include "qquickcanvasitem_p.h"
@@ -45,7 +10,7 @@
 #include <private/qquickcontext2dtexture_p.h>
 #include <private/qsgadaptationlayer_p.h>
 #include <qsgtextureprovider.h>
-#include <QtQuick/private/qquickpixmapcache_p.h>
+#include <QtQuick/private/qquickpixmap_p.h>
 #include <QtGui/QGuiApplication>
 #include <qsgtextureprovider.h>
 
@@ -59,6 +24,7 @@
 #include <private/qv4scopedvalue_p.h>
 #include <private/qv4jscall_p.h>
 #include <private/qv4qobjectwrapper_p.h>
+#include <private/qjsvalue_p.h>
 
 QT_BEGIN_NAMESPACE
 
@@ -207,7 +173,7 @@ QQuickCanvasItemPrivate::~QQuickCanvasItemPrivate()
 
 /*!
     \qmltype Canvas
-    \instantiates QQuickCanvasItem
+    \nativetype QQuickCanvasItem
     \inqmlmodule QtQuick
     \since 5.0
     \inherits Item
@@ -245,34 +211,20 @@ QQuickCanvasItemPrivate::~QQuickCanvasItemPrivate()
 
     \section1 Threaded Rendering and Render Target
 
-    The Canvas item supports two render targets: \c Canvas.Image and
-    \c Canvas.FramebufferObject.
+    In Qt 6.0 the Canvas item supports one render target: \c Canvas.Image.
 
     The \c Canvas.Image render target is a \a QImage object. This render target
     supports background thread rendering, allowing complex or long running
     painting to be executed without blocking the UI. This is the only render
     target that is supported by all Qt Quick backends.
 
-    The Canvas.FramebufferObject render target utilizes OpenGL hardware
-    acceleration rather than rendering into system memory, which in many cases
-    results in faster rendering. Canvas.FramebufferObject relies on the OpenGL
-    extensions \c GL_EXT_framebuffer_multisample and \c GL_EXT_framebuffer_blit
-    for antialiasing. It will also use more graphics memory when rendering
-    strategy is anything other than Canvas.Cooperative. Framebuffer objects may
-    not be available with Qt Quick backends other than OpenGL.
-
     The default render target is Canvas.Image and the default renderStrategy is
     Canvas.Immediate.
 
     \section1 Pixel Operations
     All HTML5 2D context pixel operations are supported. In order to ensure
     improved pixel reading/writing performance the \a Canvas.Image render
-    target should be chosen. The \a Canvas.FramebufferObject render target
-    requires the pixel data to be exchanged between the system memory and the
-    graphic card, which is significantly more expensive.  Rendering may also be
-    synchronized with the V-sync signal (to avoid
-    \l{http://en.wikipedia.org/wiki/Screen_tearing}{screen tearing}) which will further
-    impact pixel operations with \c Canvas.FrambufferObject render target.
+    target should be chosen.
 
     \section1 Tips for Porting Existing HTML5 Canvas Applications
 
@@ -301,7 +253,7 @@ QQuickCanvasItemPrivate::~QQuickCanvasItemPrivate()
     QPainter instead of the more expensive and likely less performing
     JavaScript and Context2D approach.
 
-    \sa Context2D QQuickPaintedItem
+    \sa Context2D, QQuickPaintedItem, {Qt Quick Examples - Pointer Handlers}
 */
 
 QQuickCanvasItem::QQuickCanvasItem(QQuickItem *parent)
@@ -381,7 +333,7 @@ void QQuickCanvasItem::setContextType(const QString &contextType)
 QJSValue QQuickCanvasItem::context() const
 {
     Q_D(const QQuickCanvasItem);
-    return d->context ? QJSValue(d->context->v4Engine(), d->context->v4value()) : QJSValue();
+    return d->context ? QJSValuePrivate::fromReturnedValue(d->context->v4value()) : QJSValue();
 }
 
 /*!
@@ -430,7 +382,7 @@ void QQuickCanvasItem::setCanvasSize(const QSizeF & size)
 
     By default the tileSize is the same as the canvasSize.
 
-    \obsolete This feature is incomplete. For details, see QTBUG-33129.
+    \deprecated This feature is incomplete. For details, see QTBUG-33129.
 
     \sa canvasSize, canvasWindow
 */
@@ -465,7 +417,7 @@ void QQuickCanvasItem::setTileSize(const QSize & size)
      can display different visible areas by changing the canvas windowSize
      and/or position.
 
-    \obsolete This feature is incomplete. For details, see QTBUG-33129
+    \deprecated This feature is incomplete. For details, see QTBUG-33129.
 
     \sa canvasSize, tileSize
 */
@@ -493,10 +445,8 @@ void QQuickCanvasItem::setCanvasWindow(const QRectF& rect)
     \qmlproperty enumeration QtQuick::Canvas::renderTarget
     Holds the current canvas render target.
 
-    \list
-    \li Canvas.Image  - render to an in memory image buffer.
-    \li Canvas.FramebufferObject - render to an OpenGL frame buffer
-    \endlist
+    \value Canvas.Image                 Render to an in-memory image buffer.
+    \value Canvas.FramebufferObject     As of Qt 6.0, this value is ignored.
 
     This hint is supplied along with renderStrategy to the graphics context to
     determine the method of rendering. A renderStrategy, renderTarget or a
@@ -530,11 +480,9 @@ void QQuickCanvasItem::setRenderTarget(QQuickCanvasItem::RenderTarget target)
     \qmlproperty enumeration QtQuick::Canvas::renderStrategy
     Holds the current canvas rendering strategy.
 
-    \list
-    \li Canvas.Immediate - context will perform graphics commands immediately in the main UI thread.
-    \li Canvas.Threaded - context will defer graphics commands to a private rendering thread.
-    \li Canvas.Cooperative - context will defer graphics commands to the applications global render thread.
-    \endlist
+    \value Canvas.Immediate     context will perform graphics commands immediately in the main UI thread.
+    \value Canvas.Threaded      context will defer graphics commands to a private rendering thread.
+    \value Canvas.Cooperative   context will defer graphics commands to the applications global render thread.
 
     This hint is supplied along with renderTarget to the graphics context to
     determine the method of rendering. A renderStrategy, renderTarget or a
@@ -593,11 +541,11 @@ void QQuickCanvasItem::sceneGraphInitialized()
         QMetaObject::invokeMethod(this, "requestPaint", Qt::QueuedConnection);
 }
 
-void QQuickCanvasItem::geometryChanged(const QRectF &newGeometry, const QRectF &oldGeometry)
+void QQuickCanvasItem::geometryChange(const QRectF &newGeometry, const QRectF &oldGeometry)
 {
     Q_D(QQuickCanvasItem);
 
-    QQuickItem::geometryChanged(newGeometry, oldGeometry);
+    QQuickItem::geometryChange(newGeometry, oldGeometry);
 
     // Due to indirect recursion, newGeometry may be outdated
     // after this call, so we use width and height instead.
@@ -666,6 +614,10 @@ void QQuickCanvasItem::invalidateSceneGraph()
     d->textureProvider = nullptr;
     delete d->nodeTexture;
     d->nodeTexture = nullptr;
+
+    // As we can expect(/hope) that the SG will be "good again", we can requestPaint ( which does 'markDirty(canvasWindow);' )
+    // Otherwise this Canvas will be "blank" when SG comes back
+    requestPaint();
 }
 
 void QQuickCanvasItem::schedulePolish()
@@ -727,12 +679,12 @@ void QQuickCanvasItem::updatePolish()
         QV4::ExecutionEngine *v4 = qmlEngine(this)->handle();
         QV4::Scope scope(v4);
         QV4::ScopedFunctionObject function(scope);
-        QV4::JSCallData jsCall(scope, 1);
-        *jsCall->thisObject = QV4::QObjectWrapper::wrap(v4, this);
+        QV4::JSCallArguments jsCall(scope, 1);
+        *jsCall.thisObject = QV4::QObjectWrapper::wrap(v4, this);
 
         for (auto it = animationCallbacks.cbegin(), end = animationCallbacks.cend(); it != end; ++it) {
             function = it.value().value();
-            jsCall->args[0] = QV4::Value::fromUInt32(QDateTime::currentMSecsSinceEpoch());
+            jsCall.args[0] = QV4::Value::fromUInt32(QDateTime::currentMSecsSinceEpoch());
             function->call(jsCall);
         }
     }
@@ -826,14 +778,14 @@ QSGTextureProvider *QQuickCanvasItem::textureProvider() const
         return QQuickItem::textureProvider();
 
     Q_D(const QQuickCanvasItem);
-#if QT_CONFIG(opengl)
+
     QQuickWindow *w = window();
     if (!w || !w->isSceneGraphInitialized()
             || QThread::currentThread() != QQuickWindowPrivate::get(w)->context->thread()) {
         qWarning("QQuickCanvasItem::textureProvider: can only be queried on the rendering thread of an exposed window");
         return nullptr;
     }
-#endif
+
     if (!d->textureProvider)
         d->textureProvider = new QQuickCanvasTextureProvider;
     d->textureProvider->tex = d->nodeTexture;
@@ -859,7 +811,7 @@ QSGTextureProvider *QQuickCanvasItem::textureProvider() const
 
 */
 
-void QQuickCanvasItem::getContext(QQmlV4Function *args)
+void QQuickCanvasItem::getContext(QQmlV4FunctionPtr args)
 {
     Q_D(QQuickCanvasItem);
 
@@ -903,7 +855,7 @@ void QQuickCanvasItem::getContext(QQmlV4Function *args)
     scene.
 */
 
-void QQuickCanvasItem::requestAnimationFrame(QQmlV4Function *args)
+void QQuickCanvasItem::requestAnimationFrame(QQmlV4FunctionPtr args)
 {
     QV4::Scope scope(args->v4engine());
     QV4::ScopedFunctionObject f(scope, (*args)[0]);
@@ -927,12 +879,12 @@ void QQuickCanvasItem::requestAnimationFrame(QQmlV4Function *args)
 }
 
 /*!
-    \qmlmethod QtQuick::Canvas::cancelRequestAnimationFrame(int handle)
+    \qmlmethod void QtQuick::Canvas::cancelRequestAnimationFrame(int handle)
 
     This function will cancel the animation callback referenced by \a handle.
 */
 
-void QQuickCanvasItem::cancelRequestAnimationFrame(QQmlV4Function *args)
+void QQuickCanvasItem::cancelRequestAnimationFrame(QQmlV4FunctionPtr args)
 {
     QV4::Scope scope(args->v4engine());
     QV4::ScopedValue v(scope, (*args)[0]);
@@ -947,7 +899,7 @@ void QQuickCanvasItem::cancelRequestAnimationFrame(QQmlV4Function *args)
 
 
 /*!
-    \qmlmethod QtQuick::Canvas::requestPaint()
+    \qmlmethod void QtQuick::Canvas::requestPaint()
 
     Request the entire visible region be re-drawn.
 
@@ -960,7 +912,7 @@ void QQuickCanvasItem::requestPaint()
 }
 
 /*!
-    \qmlmethod QtQuick::Canvas::markDirty(rect area)
+    \qmlmethod void QtQuick::Canvas::markDirty(rect area)
 
     Marks the given \a area as dirty, so that when this area is visible the
     canvas renderer will redraw it. This will trigger the \c paint signal.
@@ -986,30 +938,35 @@ void QQuickCanvasItem::checkAnimationCallbacks()
 }
 
 /*!
-    \qmlmethod bool QtQuick::Canvas::save(string filename)
+    \qmlmethod bool QtQuick::Canvas::save(string filename, size imageSize = undefined)
 
     Saves the current canvas content into an image file \a filename.
-    The saved image format is automatically decided by the \a filename's
-    suffix. Returns \c true on success.
+    The saved image format is automatically decided by the \a filename's suffix.
+    Returns \c true on success. If \a imageSize is specified, the resulting
+    image will have this size, and will have a devicePixelRatio of \c 1.0.
+    Otherwise, the \l {QQuickWindow::}{devicePixelRatio()} of the window in
+    which the canvas is displayed is applied to the saved image.
 
     \note Calling this method will force painting the whole canvas, not just the
     current canvas visible window.
 
     \sa canvasWindow, canvasSize, toDataURL()
 */
-bool QQuickCanvasItem::save(const QString &filename) const
+bool QQuickCanvasItem::save(const QString &filename, const QSizeF &imageSize) const
 {
     Q_D(const QQuickCanvasItem);
-    QUrl url = d->baseUrl.resolved(QUrl::fromLocalFile(filename));
-    return toImage().save(url.toLocalFile());
+    QUrl url;
+    url.setPath(filename); // `filename` may contain # or % characters
+    url = d->baseUrl.resolved(url);
+    return toImage(QRectF(QPointF(0, 0), imageSize)).save(url.toLocalFile());
 }
 
-QQmlRefPointer<QQuickCanvasPixmap> QQuickCanvasItem::loadedPixmap(const QUrl& url)
+QQmlRefPointer<QQuickCanvasPixmap> QQuickCanvasItem::loadedPixmap(const QUrl& url, QSizeF sourceSize)
 {
     Q_D(QQuickCanvasItem);
     QUrl fullPathUrl = d->baseUrl.resolved(url);
     if (!d->pixmaps.contains(fullPathUrl)) {
-        loadImage(url);
+        loadImage(url, sourceSize);
     }
     return d->pixmaps.value(fullPathUrl);
 }
@@ -1019,13 +976,11 @@ QQmlRefPointer<QQuickCanvasPixmap> QQuickCanvasItem::loadedPixmap(const QUrl& ur
 
     This signal is emitted when an image has been loaded.
 
-    The corresponding handler is \c onImageLoaded.
-
     \sa loadImage()
 */
 
 /*!
-    \qmlmethod QtQuick::Canvas::loadImage(url image)
+    \qmlmethod void QtQuick::Canvas::loadImage(url image, size sourceSize = undefined)
 
     Loads the given \a image asynchronously.
 
@@ -1034,10 +989,14 @@ QQmlRefPointer<QQuickCanvasPixmap> QQuickCanvasItem::loadedPixmap(const QUrl& ur
 
     \note Only loaded images can be painted on the Canvas item.
 
+    If \a sourceSize is specified, the image will be scaled to that size during loading. This is
+    useful for loading scalable (vector) images (eg. SVGs) at their intended display size. This
+    parameter was introduced in Qt 6.7.
+
     \sa unloadImage(), imageLoaded(), isImageLoaded(),
         Context2D::createImageData(), Context2D::drawImage()
 */
-void QQuickCanvasItem::loadImage(const QUrl& url)
+void QQuickCanvasItem::loadImage(const QUrl& url, QSizeF sourceSize)
 {
     Q_D(QQuickCanvasItem);
     QUrl fullPathUrl = d->baseUrl.resolved(url);
@@ -1049,13 +1008,15 @@ void QQuickCanvasItem::loadImage(const QUrl& url)
 
         pix->load(qmlEngine(this)
                 , fullPathUrl
+                , QRect()
+                , sourceSize.toSize()
                 , QQuickPixmap::Cache | QQuickPixmap::Asynchronous);
         if (pix->isLoading())
             pix->connectFinished(this, SIGNAL(imageLoaded()));
     }
 }
 /*!
-    \qmlmethod QtQuick::Canvas::unloadImage(url image)
+    \qmlmethod void QtQuick::Canvas::unloadImage(url image)
 
     Unloads the \a image.
 
@@ -1072,7 +1033,7 @@ void QQuickCanvasItem::unloadImage(const QUrl& url)
 }
 
 /*!
-    \qmlmethod QtQuick::Canvas::isImageError(url image)
+    \qmlmethod bool QtQuick::Canvas::isImageError(url image)
 
     Returns \c true if the \a image failed to load, \c false otherwise.
 
@@ -1087,8 +1048,8 @@ bool QQuickCanvasItem::isImageError(const QUrl& url) const
 }
 
 /*!
-  \qmlmethod QtQuick::Canvas::isImageLoading(url image)
-  Returns true if the \a image is currently loading.
+  \qmlmethod bool QtQuick::Canvas::isImageLoading(url image)
+  Returns \c true if the \a image is currently loading.
 
   \sa loadImage()
 */
@@ -1100,8 +1061,8 @@ bool QQuickCanvasItem::isImageLoading(const QUrl& url) const
         && d->pixmaps.value(fullPathUrl)->pixmap()->isLoading();
 }
 /*!
-  \qmlmethod QtQuick::Canvas::isImageLoaded(url image)
-  Returns true if the \a image is successfully loaded and ready to use.
+  \qmlmethod bool QtQuick::Canvas::isImageLoaded(url image)
+  Returns \c true if the \a image is successfully loaded and ready to use.
 
   \sa loadImage()
 */
@@ -1113,6 +1074,12 @@ bool QQuickCanvasItem::isImageLoaded(const QUrl& url) const
         && d->pixmaps.value(fullPathUrl)->pixmap()->isReady();
 }
 
+/*!
+    \internal
+    Returns a QImage representing the requested \a rect which is in device independent pixels of the item.
+    If \a rect is empty, then it will use the whole item's rect by default.
+*/
+
 QImage QQuickCanvasItem::toImage(const QRectF& rect) const
 {
     Q_D(const QQuickCanvasItem);
@@ -1121,7 +1088,7 @@ QImage QQuickCanvasItem::toImage(const QRectF& rect) const
         return QImage();
 
     const QRectF &rectSource = rect.isEmpty() ? canvasWindow() : rect;
-    const qreal dpr = window() ? window()->effectiveDevicePixelRatio() : qreal(1);
+    const qreal dpr = window() && rect.isEmpty() ? window()->effectiveDevicePixelRatio() : qreal(1);
     const QRectF rectScaled(rectSource.topLeft() * dpr, rectSource.size() * dpr);
 
     QImage image = d->context->toImage(rectScaled);
@@ -1134,7 +1101,7 @@ static const char* mimeToType(const QString &mime)
     const QLatin1String imagePrefix("image/");
     if (!mime.startsWith(imagePrefix))
         return nullptr;
-    const QStringRef mimeExt = mime.midRef(imagePrefix.size());
+    const QStringView mimeExt = QStringView{mime}.mid(imagePrefix.size());
     if (mimeExt == QLatin1String("png"))
         return "png";
     else if (mimeExt == QLatin1String("bmp"))
@@ -1242,10 +1209,8 @@ QRect QQuickCanvasItem::tiledRect(const QRectF &window, const QSize &tileSize)
     This signal is emitted when the \a region needs to be rendered. If a context
     is active it can be referenced from the context property.
 
-    This signal can be triggered by markdirty(), requestPaint() or by changing
+    This signal can be triggered by markDirty(), requestPaint() or by changing
     the current canvas window.
-
-    The corresponding handler is \c onPaint.
 */
 
 /*!
@@ -1253,8 +1218,6 @@ QRect QQuickCanvasItem::tiledRect(const QRectF &window, const QSize &tileSize)
 
     This signal is emitted after all context painting commands are executed and
     the Canvas has been rendered.
-
-    The corresponding handler is \c onPainted.
 */
 
 QT_END_NAMESPACE
