@@ -998,7 +998,7 @@ void QQuickItemViewPrivate::applyPendingChanges()
         layout();
 }
 
-int QQuickItemViewPrivate::findMoveKeyIndex(QQmlChangeSet::MoveKey key, const QVector<QQmlChangeSet::Change> &changes) const
+int QQuickItemViewPrivate::findMoveKeyIndex(QQmlChangeSet::MoveKey key, const QList<QQmlChangeSet::Change> &changes) const
 {
     for (int i=0; i<changes.size(); i++) {
         for (int j=changes[i].index; j<changes[i].index + changes[i].count; j++) {
@@ -1654,6 +1654,7 @@ QQuickItemViewPrivate::QQuickItemViewPrivate()
     , isClearing(false)
     , explicitDelegate(false)
     , explicitDelegateModelAccess(false)
+    , inRefill(false)
 {
     bufferPause.addAnimationChangeListener(this, QAbstractAnimationJob::Completion);
     bufferPause.setLoopCount(1);
@@ -1907,6 +1908,10 @@ void QQuickItemViewPrivate::refill(qreal from, qreal to)
         return;
     }
 
+    if (inRefill)
+        return;
+    inRefill = true;
+
     do {
         bufferPause.stop();
         if (currentChanges.hasPendingChanges() || bufferedChanges.hasPendingChanges() || currentChanges.active) {
@@ -1953,6 +1958,7 @@ void QQuickItemViewPrivate::refill(qreal from, qreal to)
             emitCountChanged();
     } while (currentChanges.hasPendingChanges() || bufferedChanges.hasPendingChanges());
     storeFirstVisibleItemPosition();
+    inRefill = false;
 }
 
 void QQuickItemViewPrivate::regenerate(bool orientationChanged)
@@ -2161,8 +2167,8 @@ bool QQuickItemViewPrivate::applyModelChanges(ChangeResult *totalInsertionResult
     totalInsertionResult->visiblePos = prevFirstItemInViewPos;
     totalRemovalResult->visiblePos = prevFirstItemInViewPos;
 
-    const QVector<QQmlChangeSet::Change> &removals = currentChanges.pendingChanges.removes();
-    const QVector<QQmlChangeSet::Change> &insertions = currentChanges.pendingChanges.inserts();
+    const QList<QQmlChangeSet::Change> &removals = currentChanges.pendingChanges.removes();
+    const QList<QQmlChangeSet::Change> &insertions = currentChanges.pendingChanges.inserts();
     ChangeResult insertionResult(prevFirstItemInViewPos);
     ChangeResult removalResult(prevFirstItemInViewPos);
 
@@ -2648,8 +2654,10 @@ bool QQuickItemViewPrivate::releaseItem(FxViewItem *item, QQmlInstanceModel::Reu
         }
 
         QQuickItemPrivate::get(quickItem)->removeItemChangeListener(this, itemChangeListenerTypes);
+#if QT_CONFIG(quick_viewtransitions)
         delete item->transitionableItem;
         item->transitionableItem = nullptr;
+#endif
     }
 
     delete item;
