@@ -1,41 +1,6 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtQuick module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #include "qquickpositioners_p.h"
 #include "qquickpositioners_p_p.h"
@@ -46,9 +11,11 @@
 
 #include <QtQuick/private/qquicktransition_p.h>
 
+#include <algorithm>
+
 QT_BEGIN_NAMESPACE
 
-static const QQuickItemPrivate::ChangeTypes watchedChanges
+static const QQuickItemPrivate::ChangeTypes positionerWatchedChanges
     = QQuickItemPrivate::Geometry
     | QQuickItemPrivate::SiblingOrder
     | QQuickItemPrivate::Visibility
@@ -57,19 +24,21 @@ static const QQuickItemPrivate::ChangeTypes watchedChanges
 void QQuickBasePositionerPrivate::watchChanges(QQuickItem *other)
 {
     QQuickItemPrivate *otherPrivate = QQuickItemPrivate::get(other);
-    otherPrivate->addItemChangeListener(this, watchedChanges);
+    otherPrivate->addItemChangeListener(this, positionerWatchedChanges);
 }
 
 void QQuickBasePositionerPrivate::unwatchChanges(QQuickItem* other)
 {
     QQuickItemPrivate *otherPrivate = QQuickItemPrivate::get(other);
-    otherPrivate->removeItemChangeListener(this, watchedChanges);
+    otherPrivate->removeItemChangeListener(this, positionerWatchedChanges);
 }
 
 
 QQuickBasePositioner::PositionedItem::PositionedItem(QQuickItem *i)
     : item(i)
+#if QT_CONFIG(quick_viewtransitions)
     , transitionableItem(nullptr)
+#endif
     , index(-1)
     , isNew(false)
     , isVisible(true)
@@ -80,36 +49,42 @@ QQuickBasePositioner::PositionedItem::PositionedItem(QQuickItem *i)
 {
 }
 
-QQuickBasePositioner::PositionedItem::~PositionedItem()
-{
-    delete transitionableItem;
-}
-
 qreal QQuickBasePositioner::PositionedItem::itemX() const
 {
-    return transitionableItem ? transitionableItem->itemX() : item->x();
+    return
+#if QT_CONFIG(quick_viewtransitions)
+            transitionableItem ? transitionableItem->itemX() :
+#endif
+            item->x();
 }
 
 qreal QQuickBasePositioner::PositionedItem::itemY() const
 {
-    return transitionableItem ? transitionableItem->itemY() : item->y();
+    return
+#if QT_CONFIG(quick_viewtransitions)
+            transitionableItem ? transitionableItem->itemY() :
+#endif
+            item->y();
 }
 
 void QQuickBasePositioner::PositionedItem::moveTo(const QPointF &pos)
 {
+#if QT_CONFIG(quick_viewtransitions)
     if (transitionableItem)
         transitionableItem->moveTo(pos);
     else
+#endif
         item->setPosition(pos);
 }
 
+#if QT_CONFIG(quick_viewtransitions)
 void QQuickBasePositioner::PositionedItem::transitionNextReposition(QQuickItemViewTransitioner *transitioner, QQuickItemViewTransitioner::TransitionType type, bool asTarget)
 {
     if (!transitioner)
         return;
     if (!transitionableItem)
-        transitionableItem = new QQuickItemViewTransitionableItem(item);
-    transitioner->transitionNextReposition(transitionableItem, type, asTarget);
+        transitionableItem = std::make_unique<QQuickItemViewTransitionableItem>(item);
+    transitioner->transitionNextReposition(transitionableItem.get(), type, asTarget);
 }
 
 bool QQuickBasePositioner::PositionedItem::prepareTransition(QQuickItemViewTransitioner *transitioner, const QRectF &viewBounds)
@@ -122,6 +97,7 @@ void QQuickBasePositioner::PositionedItem::startTransition(QQuickItemViewTransit
     if (transitionableItem)
         transitionableItem->startTransition(transitioner, index);
 }
+#endif
 
 void QQuickBasePositioner::PositionedItem::updatePadding(qreal lp, qreal tp, qreal rp, qreal bp)
 {
@@ -168,13 +144,15 @@ QQuickBasePositioner::QQuickBasePositioner(QQuickBasePositionerPrivate &dd, Posi
 QQuickBasePositioner::~QQuickBasePositioner()
 {
     Q_D(QQuickBasePositioner);
+#if QT_CONFIG(quick_viewtransitions)
     delete d->transitioner;
-    for (int i = 0; i < positionedItems.count(); ++i)
-        d->unwatchChanges(positionedItems.at(i).item);
-    for (int i = 0; i < unpositionedItems.count(); ++i)
-        d->unwatchChanges(unpositionedItems.at(i).item);
-    clearPositionedItems(&positionedItems);
-    clearPositionedItems(&unpositionedItems);
+#endif
+    for (const PositionedItem &pi : positionedItems)
+        d->unwatchChanges(pi.item);
+    for (const PositionedItem &pi : unpositionedItems)
+        d->unwatchChanges(pi.item);
+    positionedItems.clear();
+    unpositionedItems.clear();
 }
 
 void QQuickBasePositioner::updatePolish()
@@ -200,6 +178,7 @@ void QQuickBasePositioner::setSpacing(qreal s)
     emit spacingChanged();
 }
 
+#if QT_CONFIG(quick_viewtransitions)
 QQuickTransition *QQuickBasePositioner::populate() const
 {
     Q_D(const QQuickBasePositioner);
@@ -252,17 +231,24 @@ void QQuickBasePositioner::setAdd(QQuickTransition *add)
     d->transitioner->addTransition = add;
     emit addChanged();
 }
+#endif
 
 void QQuickBasePositioner::componentComplete()
 {
+#if QT_CONFIG(quick_viewtransitions)
     Q_D(QQuickBasePositioner);
+#endif
     QQuickItem::componentComplete();
+#if QT_CONFIG(quick_viewtransitions)
     if (d->transitioner)
         d->transitioner->setPopulateTransitionEnabled(true);
-    positionedItems.reserve(childItems().count());
+#endif
+    positionedItems.reserve(childItems().size());
     prePositioning();
+#if QT_CONFIG(quick_viewtransitions)
     if (d->transitioner)
         d->transitioner->setPopulateTransitionEnabled(false);
+#endif
 }
 
 void QQuickBasePositioner::itemChange(ItemChange change, const ItemChangeData &value)
@@ -272,14 +258,16 @@ void QQuickBasePositioner::itemChange(ItemChange change, const ItemChangeData &v
         d->setPositioningDirty();
     } else if (change == ItemChildRemovedChange) {
         QQuickItem *child = value.item;
-        QQuickBasePositioner::PositionedItem posItem(child);
-        int idx = positionedItems.find(posItem);
-        if (idx >= 0) {
+        auto it = std::find(positionedItems.begin(), positionedItems.end(), child);
+        if (it != positionedItems.end()) {
             d->unwatchChanges(child);
-            removePositionedItem(&positionedItems, idx);
-        } else if ((idx = unpositionedItems.find(posItem)) >= 0) {
-            d->unwatchChanges(child);
-            removePositionedItem(&unpositionedItems, idx);
+            positionedItems.erase(it);
+        } else {
+            it = std::find(unpositionedItems.begin(), unpositionedItems.end(), child);
+            if (it != unpositionedItems.end()) {
+                d->unwatchChanges(child);
+                unpositionedItems.erase(it);
+            }
         }
         d->setPositioningDirty();
     }
@@ -306,82 +294,109 @@ void QQuickBasePositioner::prePositioning()
     //Need to order children by creation order modified by stacking order
     QList<QQuickItem *> children = childItems();
 
-    QPODVector<PositionedItem,8> oldItems;
-    positionedItems.copyAndClear(oldItems);
-    for (int ii = 0; ii < unpositionedItems.count(); ii++)
-        oldItems.append(unpositionedItems[ii]);
-    unpositionedItems.clear();
-    int addedIndex = -1;
+    std::vector<PositionedItem> oldItems;
+    oldItems.reserve(positionedItems.size() + unpositionedItems.size());
 
-    for (int ii = 0; ii < children.count(); ++ii) {
+    std::move(positionedItems.begin(), positionedItems.end(),
+              std::back_inserter(oldItems));
+    positionedItems.clear();
+
+    std::move(unpositionedItems.begin(), unpositionedItems.end(),
+              std::back_inserter(oldItems));
+    unpositionedItems.clear();
+
+#if QT_CONFIG(quick_viewtransitions)
+    int addedIndex = -1;
+#endif
+
+    for (int ii = 0; ii < children.size(); ++ii) {
         QQuickItem *child = children.at(ii);
         if (QQuickItemPrivate::get(child)->isTransparentForPositioner())
             continue;
         QQuickItemPrivate *childPrivate = QQuickItemPrivate::get(child);
         PositionedItem posItem(child);
-        int wIdx = oldItems.find(posItem);
-        if (wIdx < 0) {
+        auto it = std::find(oldItems.begin(), oldItems.end(), posItem);
+        if (it == oldItems.end()) {
+            // This is a newly added item.
             d->watchChanges(child);
             posItem.isNew = true;
             if (!childPrivate->explicitVisible || !child->width() || !child->height()) {
                 posItem.isVisible = false;
                 posItem.index = -1;
-                unpositionedItems.append(posItem);
+                // If we hide a zero-width or height item by setting visible to false,
+                // the !childPrivate->explicitVisible will then always trigger. We can't
+                // overwrite what the user has set, and we don't want to introduce a separate
+                // flag to track whether the visible property was actually explicitly set so
+                // that we can implicitly set it, so instead we use culled for this.
+                childPrivate->setCulled(true);
+                unpositionedItems.push_back(std::move(posItem));
             } else {
-                posItem.index = positionedItems.count();
-                positionedItems.append(posItem);
+                const int posIndex = int(positionedItems.size());
+                posItem.index = posIndex;
+                positionedItems.push_back(std::move(posItem));
 
+#if QT_CONFIG(quick_viewtransitions)
                 if (d->transitioner) {
                     if (addedIndex < 0)
-                        addedIndex = posItem.index;
-                    PositionedItem *theItem = &positionedItems[positionedItems.count()-1];
+                        addedIndex = posIndex;
+                    PositionedItem *theItem = &positionedItems.back();
                     if (d->transitioner->canTransition(QQuickItemViewTransitioner::PopulateTransition, true))
                         theItem->transitionNextReposition(d->transitioner, QQuickItemViewTransitioner::PopulateTransition, true);
                     else if (!d->transitioner->populateTransitionEnabled())
                         theItem->transitionNextReposition(d->transitioner, QQuickItemViewTransitioner::AddTransition, true);
                 }
+#endif
             }
         } else {
-            PositionedItem *item = &oldItems[wIdx];
+            // This item already existed within us.
+            PositionedItem *item = &*it;
             // Items are only omitted from positioning if they are explicitly hidden
             // i.e. their positioning is not affected if an ancestor is hidden.
             if (!childPrivate->explicitVisible || !child->width() || !child->height()) {
                 item->isVisible = false;
                 item->index = -1;
-                unpositionedItems.append(*item);
+                childPrivate->setCulled(true);
+                unpositionedItems.push_back(std::move(*item));
             } else if (!item->isVisible) {
                 // item changed from non-visible to visible, treat it as a "new" item
                 item->isVisible = true;
                 item->isNew = true;
-                item->index = positionedItems.count();
-                positionedItems.append(*item);
+                const int itemIndex = int(positionedItems.size());
+                item->index = itemIndex;
+                childPrivate->setCulled(false);
+                positionedItems.push_back(std::move(*item));
 
+#if QT_CONFIG(quick_viewtransitions)
                 if (d->transitioner) {
                     if (addedIndex < 0)
-                        addedIndex = item->index;
-                    positionedItems[positionedItems.count()-1].transitionNextReposition(d->transitioner, QQuickItemViewTransitioner::AddTransition, true);
+                        addedIndex = itemIndex;
+                    positionedItems.back().transitionNextReposition(d->transitioner, QQuickItemViewTransitioner::AddTransition, true);
                 }
+#endif
             } else {
                 item->isNew = false;
-                item->index = positionedItems.count();
-                positionedItems.append(*item);
+                const int itemIndex = int(positionedItems.size());
+                item->index = itemIndex;
+                positionedItems.push_back(std::move(*item));
             }
         }
     }
 
+#if QT_CONFIG(quick_viewtransitions)
     if (d->transitioner) {
-        for (int i=0; i<positionedItems.count(); i++) {
-            if (!positionedItems[i].isNew) {
+        for (PositionedItem &item : positionedItems) {
+            if (!item.isNew) {
                 if (addedIndex >= 0) {
-                    positionedItems[i].transitionNextReposition(d->transitioner, QQuickItemViewTransitioner::AddTransition, false);
+                    item.transitionNextReposition(d->transitioner, QQuickItemViewTransitioner::AddTransition, false);
                 } else {
                     // just queue the item for a move-type displace - if the item hasn't
                     // moved anywhere, it won't be transitioned anyway
-                    positionedItems[i].transitionNextReposition(d->transitioner, QQuickItemViewTransitioner::MoveTransition, false);
+                    item.transitionNextReposition(d->transitioner, QQuickItemViewTransitioner::MoveTransition, false);
                 }
             }
         }
     }
+#endif
 
     QSizeF contentSize(0,0);
     reportConflictingAnchors();
@@ -390,14 +405,16 @@ void QQuickBasePositioner::prePositioning()
         updateAttachedProperties();
     }
 
+#if QT_CONFIG(quick_viewtransitions)
     if (d->transitioner) {
         QRectF viewBounds(QPointF(), contentSize);
-        for (int i=0; i<positionedItems.count(); i++)
-            positionedItems[i].prepareTransition(d->transitioner, viewBounds);
-        for (int i=0; i<positionedItems.count(); i++)
-            positionedItems[i].startTransition(d->transitioner);
+        for (PositionedItem &item : positionedItems)
+            item.prepareTransition(d->transitioner, viewBounds);
+        for (PositionedItem &item : positionedItems)
+            item.startTransition(d->transitioner);
         d->transitioner->resetTargetLists();
     }
+#endif
 
     d->doingPositioning = false;
 
@@ -431,24 +448,6 @@ void QQuickBasePositioner::positionItemY(qreal y, PositionedItem *target)
     }
 }
 
-/*
-  Since PositionedItem values are stored by value, their internal transitionableItem pointers
-  must be cleaned up when a PositionedItem is removed from a QPODVector, otherwise the pointer
-  is never deleted since QPODVector doesn't invoke the destructor.
-  */
-void QQuickBasePositioner::removePositionedItem(QPODVector<PositionedItem,8> *items, int index)
-{
-    Q_ASSERT(index >= 0 && index < items->count());
-    delete items->at(index).transitionableItem;
-    items->remove(index);
-}
-void QQuickBasePositioner::clearPositionedItems(QPODVector<PositionedItem,8> *items)
-{
-    for (int i=0; i<items->count(); i++)
-        delete items->at(i).transitionableItem;
-    items->clear();
-}
-
 QQuickPositionerAttached *QQuickBasePositioner::qmlAttachedProperties(QObject *obj)
 {
     return new QQuickPositionerAttached(obj);
@@ -463,8 +462,9 @@ void QQuickBasePositioner::updateAttachedProperties(QQuickPositionerAttached *sp
     QQuickPositionerAttached *prevLastProperty = nullptr;
     QQuickPositionerAttached *lastProperty = nullptr;
 
-    for (int ii = 0; ii < positionedItems.count(); ++ii) {
-        const PositionedItem &child = positionedItems.at(ii);
+    const int positionedItemsSize = int(positionedItems.size());
+    for (int ii = 0; ii < positionedItemsSize; ++ii) {
+        const PositionedItem &child = positionedItems[ii];
         if (!child.item)
             continue;
 
@@ -498,8 +498,7 @@ void QQuickBasePositioner::updateAttachedProperties(QQuickPositionerAttached *sp
         lastProperty->setIsLastItem(true);
 
     // clear attached properties for unpositioned items
-    for (int ii = 0; ii < unpositionedItems.count(); ++ii) {
-        const PositionedItem &child = unpositionedItems.at(ii);
+    for (const PositionedItem &child : unpositionedItems) {
         if (!child.item)
             continue;
 
@@ -702,7 +701,7 @@ void QQuickBasePositionerPrivate::setBottomPadding(qreal value, bool reset)
 
 /*!
     \qmltype Positioner
-    \instantiates QQuickPositionerAttached
+    \nativetype QQuickPositionerAttached
     \inqmlmodule QtQuick
     \ingroup qtquick-positioners
     \brief Provides attached properties that contain details on where an item exists in a positioner.
@@ -785,7 +784,7 @@ void QQuickPositionerAttached::setIsLastItem(bool isLastItem)
 
 /*!
     \qmltype Column
-    \instantiates QQuickColumn
+    \nativetype QQuickColumn
     \inqmlmodule QtQuick
     \inherits Item
     \ingroup qtquick-positioners
@@ -911,7 +910,7 @@ void QQuickPositionerAttached::setIsLastItem(bool isLastItem)
   \sa Grid::spacing
 */
 /*!
-    \qmlmethod QtQuick::Column::forceLayout()
+    \qmlmethod void QtQuick::Column::forceLayout()
     \since 5.9
 
     Column typically positions its children once per frame. This means that
@@ -928,8 +927,6 @@ void QQuickPositionerAttached::setIsLastItem(bool isLastItem)
     \since 5.9
 
     This signal is emitted when positioning has been completed.
-
-    The corresponding handler is \c onPositioningComplete.
 */
 
 QQuickColumn::QQuickColumn(QQuickItem *parent)
@@ -944,8 +941,7 @@ void QQuickColumn::doPositioning(QSizeF *contentSize)
     const qreal padding = leftPadding() + rightPadding();
     contentSize->setWidth(qMax(contentSize->width(), padding));
 
-    for (int ii = 0; ii < positionedItems.count(); ++ii) {
-        PositionedItem &child = positionedItems[ii];
+    for (PositionedItem &child : positionedItems) {
         positionItem(child.itemX() + leftPadding() - child.leftPadding, voffset, &child);
         child.updatePadding(leftPadding(), topPadding(), rightPadding(), bottomPadding());
         contentSize->setWidth(qMax(contentSize->width(), child.item->width() + padding));
@@ -962,8 +958,7 @@ void QQuickColumn::doPositioning(QSizeF *contentSize)
 void QQuickColumn::reportConflictingAnchors()
 {
     QQuickBasePositionerPrivate *d = static_cast<QQuickBasePositionerPrivate*>(QQuickBasePositionerPrivate::get(this));
-    for (int ii = 0; ii < positionedItems.count(); ++ii) {
-        const PositionedItem &child = positionedItems.at(ii);
+    for (const PositionedItem &child : positionedItems) {
         if (child.item) {
             QQuickAnchors *anchors = QQuickItemPrivate::get(static_cast<QQuickItem *>(child.item))->_anchors;
             if (anchors) {
@@ -985,7 +980,7 @@ void QQuickColumn::reportConflictingAnchors()
 }
 /*!
     \qmltype Row
-    \instantiates QQuickRow
+    \nativetype QQuickRow
     \inqmlmodule QtQuick
     \inherits Item
     \ingroup qtquick-positioners
@@ -1096,7 +1091,7 @@ void QQuickColumn::reportConflictingAnchors()
   \sa Grid::spacing
 */
 /*!
-    \qmlmethod QtQuick::Row::forceLayout()
+    \qmlmethod void QtQuick::Row::forceLayout()
     \since 5.9
 
     Row typically positions its children once per frame. This means that
@@ -1113,8 +1108,6 @@ void QQuickColumn::reportConflictingAnchors()
     \since 5.9
 
     This signal is emitted when positioning has been completed.
-
-    The corresponding handler is \c onPositioningComplete.
 */
 
 class QQuickRowPrivate : public QQuickBasePositionerPrivate
@@ -1151,14 +1144,12 @@ QQuickRow::QQuickRow(QQuickItem *parent)
 
     Possible values:
 
-    \list
-    \li Qt.LeftToRight (default) - Items are laid out from left to right. If the width of the row is explicitly set,
-    the left anchor remains to the left of the row.
-    \li Qt.RightToLeft - Items are laid out from right to left. If the width of the row is explicitly set,
-    the right anchor remains to the right of the row.
-    \endlist
+    \value Qt.LeftToRight   (default) Items are laid out from left to right. If the width of the row is
+                            explicitly set, the left anchor remains to the left of the row.
+    \value Qt.RightToLeft   Items are laid out from right to left. If the width of the row is
+                            explicitly set, the right anchor remains to the right of the row.
 
-    \sa Grid::layoutDirection, Flow::layoutDirection, {Qt Quick Examples - Right to Left}
+    \sa Grid::layoutDirection, Flow::layoutDirection
 */
 
 Qt::LayoutDirection QQuickRow::layoutDirection() const
@@ -1204,9 +1195,7 @@ void QQuickRow::doPositioning(QSizeF *contentSize)
     contentSize->setHeight(qMax(contentSize->height(), padding));
 
     QList<qreal> hoffsets;
-    for (int ii = 0; ii < positionedItems.count(); ++ii) {
-        PositionedItem &child = positionedItems[ii];
-
+    for (PositionedItem &child : positionedItems) {
         if (d->isLeftToRight()) {
             positionItem(hoffset, child.itemY() + topPadding() - child.topPadding, &child);
             child.updatePadding(leftPadding(), topPadding(), rightPadding(), bottomPadding());
@@ -1235,8 +1224,7 @@ void QQuickRow::doPositioning(QSizeF *contentSize)
         end = width();
 
     int acc = 0;
-    for (int ii = 0; ii < positionedItems.count(); ++ii) {
-        PositionedItem &child = positionedItems[ii];
+    for (PositionedItem &child : positionedItems) {
         hoffset = end - hoffsets[acc++] - child.item->width();
         positionItem(hoffset, child.itemY() + topPadding() - child.topPadding, &child);
         child.updatePadding(leftPadding(), topPadding(), rightPadding(), bottomPadding());
@@ -1246,8 +1234,7 @@ void QQuickRow::doPositioning(QSizeF *contentSize)
 void QQuickRow::reportConflictingAnchors()
 {
     QQuickBasePositionerPrivate *d = static_cast<QQuickBasePositionerPrivate*>(QQuickBasePositionerPrivate::get(this));
-    for (int ii = 0; ii < positionedItems.count(); ++ii) {
-        const PositionedItem &child = positionedItems.at(ii);
+    for (const PositionedItem &child : positionedItems) {
         if (child.item) {
             QQuickAnchors *anchors = QQuickItemPrivate::get(static_cast<QQuickItem *>(child.item))->_anchors;
             if (anchors) {
@@ -1269,7 +1256,7 @@ void QQuickRow::reportConflictingAnchors()
 
 /*!
     \qmltype Grid
-    \instantiates QQuickGrid
+    \nativetype QQuickGrid
     \inqmlmodule QtQuick
     \inherits Item
     \ingroup qtquick-positioners
@@ -1373,7 +1360,7 @@ void QQuickRow::reportConflictingAnchors()
     \sa add, populate, ViewTransition, {Qt Quick Examples - Positioners}
 */
 /*!
-  \qmlproperty qreal QtQuick::Grid::spacing
+  \qmlproperty real QtQuick::Grid::spacing
 
   The spacing is the amount in pixels left empty between adjacent
   items. The amount of spacing applied will be the same in the
@@ -1391,7 +1378,7 @@ void QQuickRow::reportConflictingAnchors()
   \sa rows, columns
 */
 /*!
-    \qmlmethod QtQuick::Grid::forceLayout()
+    \qmlmethod void QtQuick::Grid::forceLayout()
     \since 5.9
 
     Grid typically positions its children once per frame. This means that
@@ -1408,8 +1395,6 @@ void QQuickRow::reportConflictingAnchors()
     \since 5.9
 
     This signal is emitted when positioning has been completed.
-
-    The corresponding handler is \c onPositioningComplete.
 */
 
 class QQuickGridPrivate : public QQuickBasePositionerPrivate
@@ -1514,7 +1499,7 @@ void QQuickGrid::setFlow(Flow flow)
 }
 
 /*!
-    \qmlproperty qreal QtQuick::Grid::rowSpacing
+    \qmlproperty real QtQuick::Grid::rowSpacing
 
     This property holds the spacing in pixels between rows.
 
@@ -1536,7 +1521,7 @@ void QQuickGrid::setRowSpacing(const qreal rowSpacing)
 }
 
 /*!
-    \qmlproperty qreal QtQuick::Grid::columnSpacing
+    \qmlproperty real QtQuick::Grid::columnSpacing
 
     This property holds the spacing in pixels between columns.
 
@@ -1573,7 +1558,7 @@ void QQuickGrid::setColumnSpacing(const qreal columnSpacing)
     \l Grid::flow property.
     \endlist
 
-    \sa Flow::layoutDirection, Row::layoutDirection, {Qt Quick Examples - Right to Left}
+    \sa Flow::layoutDirection, Row::layoutDirection
 */
 Qt::LayoutDirection QQuickGrid::layoutDirection() const
 {
@@ -1703,7 +1688,7 @@ void QQuickGrid::doPositioning(QSizeF *contentSize)
     QQuickBasePositionerPrivate *d = static_cast<QQuickBasePositionerPrivate*>(QQuickBasePositionerPrivate::get(this));
     int c = m_columns;
     int r = m_rows;
-    int numVisible = positionedItems.count();
+    const int numVisible = int(positionedItems.size());
 
     if (m_columns <= 0 && m_rows <= 0) {
         c = 4;
@@ -1720,6 +1705,10 @@ void QQuickGrid::doPositioning(QSizeF *contentSize)
         return; //Nothing else to do
     }
 
+    if (numVisible > r * c) {
+        qmlWarning(this) << "Grid contains more visible items (" << numVisible << ") than rows*columns (" << r * c << ")";
+    }
+
     QList<qreal> maxColWidth;
     QList<qreal> maxRowHeight;
     int childIndex =0;
@@ -1734,7 +1723,7 @@ void QQuickGrid::doPositioning(QSizeF *contentSize)
                 if (childIndex == numVisible)
                     break;
 
-                const PositionedItem &child = positionedItems.at(childIndex++);
+                const PositionedItem &child = positionedItems[childIndex++];
                 if (child.item->width() > maxColWidth[j])
                     maxColWidth[j] = child.item->width();
                 if (child.item->height() > maxRowHeight[i])
@@ -1752,7 +1741,7 @@ void QQuickGrid::doPositioning(QSizeF *contentSize)
                 if (childIndex == numVisible)
                     break;
 
-                const PositionedItem &child = positionedItems.at(childIndex++);
+                const PositionedItem &child = positionedItems[childIndex++];
                 if (child.item->width() > maxColWidth[j])
                     maxColWidth[j] = child.item->width();
                 if (child.item->height() > maxRowHeight[i])
@@ -1795,8 +1784,7 @@ void QQuickGrid::doPositioning(QSizeF *contentSize)
     qreal yoffset = topPadding();
     int curRow =0;
     int curCol =0;
-    for (int i = 0; i < positionedItems.count(); ++i) {
-        PositionedItem &child = positionedItems[i];
+    for (PositionedItem &child : positionedItems) {
         qreal childXOffset = xoffset;
 
         if (effectiveHAlign() == AlignRight)
@@ -1854,8 +1842,7 @@ void QQuickGrid::doPositioning(QSizeF *contentSize)
 void QQuickGrid::reportConflictingAnchors()
 {
     QQuickBasePositionerPrivate *d = static_cast<QQuickBasePositionerPrivate*>(QQuickBasePositionerPrivate::get(this));
-    for (int ii = 0; ii < positionedItems.count(); ++ii) {
-        const PositionedItem &child = positionedItems.at(ii);
+    for (const PositionedItem &child : positionedItems) {
         if (child.item) {
             QQuickAnchors *anchors = QQuickItemPrivate::get(static_cast<QQuickItem *>(child.item))->_anchors;
             if (anchors && (anchors->usedAnchors() || anchors->fill() || anchors->centerIn())) {
@@ -1870,7 +1857,7 @@ void QQuickGrid::reportConflictingAnchors()
 
 /*!
     \qmltype Flow
-    \instantiates QQuickFlow
+    \nativetype QQuickFlow
     \inqmlmodule QtQuick
     \inherits Item
     \ingroup qtquick-positioners
@@ -1974,7 +1961,7 @@ void QQuickGrid::reportConflictingAnchors()
   \sa Grid::spacing
 */
 /*!
-    \qmlmethod QtQuick::Flow::forceLayout()
+    \qmlmethod void QtQuick::Flow::forceLayout()
     \since 5.9
 
     Flow typically positions its children once per frame. This means that
@@ -1992,8 +1979,6 @@ void QQuickGrid::reportConflictingAnchors()
     \since 5.9
 
     This signal is emitted when positioning has been completed.
-
-    The corresponding handler is \c onPositioningComplete.
 */
 
 class QQuickFlowPrivate : public QQuickBasePositionerPrivate
@@ -2071,7 +2056,7 @@ void QQuickFlow::setFlow(Flow flow)
     \l Flow::flow property.
     \endlist
 
-    \sa Grid::layoutDirection, Row::layoutDirection, {Qt Quick Examples - Right to Left}
+    \sa Grid::layoutDirection, Row::layoutDirection
 */
 
 Qt::LayoutDirection QQuickFlow::layoutDirection() const
@@ -2123,9 +2108,7 @@ void QQuickFlow::doPositioning(QSizeF *contentSize)
     contentSize->setWidth(qMax(contentSize->width(), hoffset1 + hoffset2));
     contentSize->setHeight(qMax(contentSize->height(), voffset + bottomPadding()));
 
-    for (int i = 0; i < positionedItems.count(); ++i) {
-        PositionedItem &child = positionedItems[i];
-
+    for (PositionedItem &child : positionedItems) {
         if (d->flow == LeftToRight)  {
             if (widthValid() && hoffset != hoffset1 && hoffset + child.item->width() + hoffset2 > width()) {
                 hoffset = hoffset1;
@@ -2173,8 +2156,7 @@ void QQuickFlow::doPositioning(QSizeF *contentSize)
     else
         end = contentSize->width();
     int acc = 0;
-    for (int i = 0; i < positionedItems.count(); ++i) {
-        PositionedItem &child = positionedItems[i];
+    for (PositionedItem &child : positionedItems) {
         hoffset = end - hoffsets[acc++] - child.item->width();
         positionItemX(hoffset, &child);
         child.leftPadding = leftPadding();
@@ -2185,8 +2167,7 @@ void QQuickFlow::doPositioning(QSizeF *contentSize)
 void QQuickFlow::reportConflictingAnchors()
 {
     Q_D(QQuickFlow);
-    for (int ii = 0; ii < positionedItems.count(); ++ii) {
-        const PositionedItem &child = positionedItems.at(ii);
+    for (const PositionedItem &child : positionedItems) {
         if (child.item) {
             QQuickAnchors *anchors = QQuickItemPrivate::get(static_cast<QQuickItem *>(child.item))->_anchors;
             if (anchors && (anchors->usedAnchors() || anchors->fill() || anchors->centerIn())) {
