@@ -1,5 +1,6 @@
 // Copyright (C) 2023 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #include "qquickrhiitem_p.h"
 
@@ -14,11 +15,6 @@ QT_BEGIN_NAMESPACE
     QQuickFramebufferObject that is not tied to OpenGL, but rather allows
     integrating rendering with the QRhi APIs with Qt Quick.
 
-    \preliminary
-
-    \note QQuickRhiItem is in tech preview in Qt 6.7. \b {The API is under
-    development and subject to change.}
-
     QQuickRhiItem is effectively the counterpart of \l QRhiWidget in the world of
     Qt Quick. Both of these are meant to be subclassed, and they both enable
     recording QRhi-based rendering that targets an offscreen color buffer. The
@@ -602,6 +598,18 @@ void QQuickRhiItem::setSampleCount(int samples)
     update();
 }
 
+/*!
+    \enum QQuickRhiItem::TextureFormat
+    Specifies the format of the backing texture to which the QQuickRhiItem renders.
+
+    \value RGBA8 See QRhiTexture::RGBA8. This is the default.
+    \value RGBA16F See QRhiTexture::RGBA16F.
+    \value RGBA32F See QRhiTexture::RGBA32F.
+    \value RGB10A2 See QRhiTexture::RGB10A2.
+
+    \sa QRhiTexture
+ */
+
 /*!
     \property QQuickRhiItem::colorBufferFormat
 
@@ -727,13 +735,29 @@ void QQuickRhiItem::setMirrorVertically(bool enable)
     item's area.
 
     For example, setting a size that is exactly twice the item's (pixel) size
-    effectively performs 2x supersampling (rendering at twice the resolution
-    and then implicitly scaling down when texturing the quad corresponding to
-    the item in the scene).
+    effectively performs 2x supersampling (rendering at twice the resolution and
+    then implicitly scaling down when texturing the quad corresponding to the
+    item in the scene). On the other hand, setting a size that is half of the
+    item's pixel size effectively achieves rendering at half resolution and then
+    upscaling the results.
 
     By default the value is \c 0. A value of 0 means that texture's size
     follows the item's size. (\c{texture size} = \c{item size} * \c{device
     pixel ratio}).
+
+    \note The device pixel ratio (the system compositor's scale factor) can have
+    a big impact on performance, since a scale factor of 2 (200%) means
+    rendering at twice the resolution, so twice of what the developer and UI
+    designer perceives as the item's size, and then effectively downscaling the
+    content, similarly to what happens when setting this property to twice the
+    item's pixel size on a system where the device pixel ratio is 1. Therefore,
+    this property is expected to be rarely used with sizes bigger than the
+    item's pixel size, since many modern desktop systems have neither the need
+    nor have the performance budget for it, when a larger than 1 device pixel
+    ratio is used anyway by the system. Instead, the main use case for this
+    property is to set a smaller size, in order to render at a reasonable
+    smaller resolution instead of blindly following the item (and perhaps
+    window) geometry, however big that may be.
  */
 int QQuickRhiItem::fixedColorBufferWidth() const
 {
@@ -761,14 +785,12 @@ void QQuickRhiItem::setFixedColorBufferWidth(int width)
     within the scene), which means the texture's content will appear stretched
     (scaled up) or scaled down onto the item's area.
 
-    For example, setting a size that is exactly twice the item's (pixel) size
-    effectively performs 2x supersampling (rendering at twice the resolution
-    and then implicitly scaling down when texturing the quad corresponding to
-    the item in the scene).
-
     By default the value is \c 0. A value of 0 means that texture's size
     follows the item's size. (\c{texture size} = \c{item size} * \c{device
     pixel ratio}).
+
+    See \l fixedColorBufferWidth for more information on the use cases for
+    setting a fixed width and height.
  */
 
 int QQuickRhiItem::fixedColorBufferHeight() const
