@@ -7,6 +7,7 @@ import android.util.Log;
 import java.lang.ref.WeakReference;
 import java.util.HashMap;
 import java.util.HashSet;
+import java.util.concurrent.atomic.AtomicInteger;
 
 /**
  * The QtQuickViewContent represents a QML component that can be loaded by a QtQuickView instance.
@@ -14,15 +15,17 @@ import java.util.HashSet;
  * This abstract class should be extended to be used by a QtQuickView. It provides QtQuickView with
  * essential information to load the QML component it represents.
  * It also offers convenient methods for seamless interaction with the QtQuickView that loads it.
- * @Since 6.8
+ * @since 6.8
  **/
 public abstract class QtQuickViewContent
 {
     private final static String TAG = "QtQuickViewContent";
+    private static AtomicInteger m_nextSignalId = new AtomicInteger();
 
     private WeakReference<QtQuickView> m_viewReference;
     private QtQmlStatusChangeListener m_statusChangeListener = null;
     private HashSet<Integer> m_signalListenerIds = new HashSet<>();
+    private QtSignalQueue m_signalQueue = new QtSignalQueue();
 
     /**
      * Implement this to return the library name that this component belongs to.
@@ -76,8 +79,10 @@ public abstract class QtQuickViewContent
     protected void attachView(QtQuickView view)
     {
         m_viewReference = new WeakReference<>(view);
-        if (view != null)
+        if (view != null) {
             view.setStatusChangeListener(m_statusChangeListener);
+            m_signalQueue.connectQueuedSignalListeners(view);
+        }
     }
 
     /**
@@ -158,14 +163,16 @@ public abstract class QtQuickViewContent
 
     /**
      * Connects a SignalListener to a signal of the QML component if it has already been attached
-     * and loaded by a QtQuickView instance.
+     * and loaded by a QtQuickView instance. Otherwise, the connection is executed once the
+     * component is attached to a view.
      *
      * @param signalName the name of the root object signal
      * @param argType    the Class type of the signal argument
      * @param listener   an instance of the QtSignalListener interface
      * @return a connection ID between signal and listener or the existing connection ID if there is
      *         an existing connection between the same signal and listener. Return a negative value
-     *         if the signal does not exist on the QML root object.
+     *         if the signal does not exist on the QML root object. Always returns
+     *         <code>true</code> if the component is not yet attached to a QtQuickView instance.
      **/
     protected <T> int connectSignalListener(String signalName, Class<T> argType,
                                             QtSignalListener<T> listener)
@@ -175,27 +182,28 @@ public abstract class QtQuickViewContent
 
     /**
      * Connects a SignalListener to a signal of the QML component if it has already been attached
-     * and loaded by a QtQuickView instance.
+     * and loaded by a QtQuickView instance. Otherwise, the connection is executed once the
+     * component is attached to a view.
      *
      * @param signalName the name of the root object signal
      * @param argTypes   the Class types of the signal arguments
      * @param listener   an instance of the QtSignalListener interface
      * @return a connection ID between signal and listener or the existing connection ID if there is
      *         an existing connection between the same signal and listener. Return a negative value
-     *         if the signal does not exist on the QML root object.
+     *         if the signal does not exist on the QML root object. Always returns
+     *         <code>true</code> if the component is not yet attached to a QtQuickView instance.
      **/
     protected int connectSignalListener(String signalName, Class<?>[] argTypes, Object listener)
     {
-        QtQuickView view = getQuickView();
-        if (view == null) {
-            Log.w(TAG,
-                  "Cannot connect signal listener as the QQmlComponent is not loaded in a "
-                          + "QtQuickView.");
-            return -1;
+        final int id = QtQuickViewContent.generateSignalId();
+        if (isViewAttached()) {
+            QtQuickView view = getQuickView();
+            view.connectSignalListener(signalName, argTypes, listener, id);
+            m_signalListenerIds.add(id);
+        } else {
+            m_signalQueue.add(signalName, argTypes, listener, id);
         }
-        int signalListenerId = view.connectSignalListener(signalName, argTypes, listener);
-        m_signalListenerIds.add(signalListenerId);
-        return signalListenerId;
+        return id;
     }
 
     /**
@@ -210,14 +218,66 @@ public abstract class QtQuickViewContent
      **/
     public boolean disconnectSignalListener(int signalListenerId)
     {
+        if (isViewAttached()) {
+            QtQuickView view = getQuickView();
+            m_signalListenerIds.remove(signalListenerId);
+            return view.disconnectSignalListener(signalListenerId);
+        } else {
+            return m_signalQueue.remove(signalListenerId);
+        }
+    }
+
+    static int generateSignalId()
+    {
+        return m_nextSignalId.getAndIncrement();
+    }
+
+    /**
+     * Invokes a QML method of the root object.
+     *
+     * Supported parameter types are {@link java.lang.Integer},{@link java.lang.Double},
+     * {@link java.lang.Float}, {@link java.lang.Boolean} and {@link java.lang.String}.
+     * These types get converted to their corresponding types: <code>int</code>,
+     * <code>double</code>, <code>real</code>, <code>bool</code>, and <code>string</code>,
+     * respectively.
+     *
+     * @param name name of the method
+     * @param params array of parameters that are passed to the method
+     *
+     * @see <a href="https://doc.qt.io/qt-6/qml-int.html">QML int</a>
+     * @see <a href="https://doc.qt.io/qt-6/qml-double.html">QML double</a>
+     * @see <a href="https://doc.qt.io/qt-6/qml-real.html">QML real</a>
+     * @see <a href="https://doc.qt.io/qt-6/qml-bool.html">QML bool</a>
+     * @see <a href="https://doc.qt.io/qt-6/qml-string.html">QML string</a>
+     **/
+    public void invokeMethod(String name, Object[] params)
+    {
         QtQuickView view = getQuickView();
-        if (view == null) {
+        if (view != null) {
+            view.invokeMethod(name, params);
+        } else {
             Log.w(TAG,
-                  "Cannot disconnect signal listener as the QQmlComponent is not loaded in a "
-                          + "QtQuickView.");
-            return false;
+                  "Cannot call method " + name +
+                  " as the QQmlComponent is not loaded in a QtQuickView.");
+        }
+    }
+
+    /**
+     * Invokes a QML method of the root object.
+     *
+     * @param name name of the method
+     *
+     * @see QtQuickViewContent#invokeMethod(String, Object[])
+     **/
+    public void invokeMethod(String name)
+    {
+        QtQuickView view = getQuickView();
+        if (view != null) {
+            view.invokeMethod(name);
+        } else {
+            Log.w(TAG,
+                  "Cannot call method " + name +
+                  " as the QQmlComponent is not loaded in a QtQuickView.");
         }
-        m_signalListenerIds.remove(signalListenerId);
-        return view.disconnectSignalListener(signalListenerId);
     }
 }
