@@ -5,6 +5,7 @@
 #include "qquickdialog_p.h"
 #include "qquickdialog_p_p.h"
 #include "qquickdialogbuttonbox_p.h"
+#include "qquickdialogbuttonbox_p_p.h"
 #include "qquickabstractbutton_p.h"
 #include "qquickpopupitem_p_p.h"
 #include "qquickpopupwindow_p_p.h"
@@ -170,11 +171,6 @@ void QQuickDialogPrivate::handleClick(QQuickAbstractButton *button)
     }
 }
 
-Qt::WindowFlags QQuickDialogPrivate::popupWindowType() const
-{
-    return Qt::Dialog;
-}
-
 QQuickDialog::QQuickDialog(QObject *parent)
     : QQuickDialog(*(new QQuickDialogPrivate), parent)
 {
@@ -561,6 +557,25 @@ void QQuickDialog::done(int result)
     close();
 }
 
+void QQuickDialog::itemChange(QQuickItem::ItemChange change, const QQuickItem::ItemChangeData &data)
+{
+    QQuickPopup::itemChange(change, data);
+
+    if (change != QQuickItem::ItemVisibleHasChanged || !isComponentComplete() || !data.boolValue)
+        return;
+
+    Q_D(QQuickDialog);
+    if (d->buttonBox && QQuickItemPrivate::get(d->popupItem)->subFocusItem == nullptr) {
+        for (int i = 0; i < d->buttonBox->count(); ++i) {
+            QQuickItem *item = qobject_cast<QQuickItem *>(d->buttonBox->itemAt(i));
+            if (item && item->isEnabled()) {
+                d->buttonBox->setFocus(true, Qt::OtherFocusReason);
+                break;
+            }
+        }
+    }
+}
+
 #if QT_CONFIG(accessibility)
 QAccessible::Role QQuickDialog::accessibleRole() const
 {
