@@ -68,7 +68,7 @@ QQuickPopupWindow::QQuickPopupWindow(QQuickPopup *popup, QWindow *parent)
     setWidth(d->m_popupItem->implicitWidth());
     setHeight(d->m_popupItem->implicitHeight());
 
-    const auto flags = QQuickPopupPrivate::get(popup)->popupWindowType();
+    const auto flags = QQuickPopupPrivate::get(popup)->popupWindowFlags();
 
     // For popup windows, we'll need to draw everything, in order to have enough control over the styling.
     if (flags & Qt::Popup)
@@ -76,7 +76,7 @@ QQuickPopupWindow::QQuickPopupWindow(QQuickPopup *popup, QWindow *parent)
 
     setFlags(flags);
 
-    qCDebug(lcPopupWindow) << "Created popup window with parent:" << parent << "flags:" << flags;
+    qCDebug(lcPopupWindow) << "Created popup window" << this << "with parent" << parent;
 }
 
 QQuickPopup *QQuickPopupWindow::popup() const
@@ -115,7 +115,8 @@ void QQuickPopupWindow::resizeEvent(QResizeEvent *e)
     if (!d->m_popupItem)
         return;
 
-    qCDebug(lcPopupWindow) << "A window system event changed the popup's size to be " << e->size();
+    qCDebug(lcPopupWindow).nospace() << "A window system event changed the popup's ("
+        << d->m_popup << ") size to " << e->size();
     QQuickPopupPrivate *popupPrivate = QQuickPopupPrivate::get(d->m_popup);
 
     const auto topLeftFromSystem = global2Local(d->geometry.topLeft());
@@ -400,7 +401,8 @@ void QQuickPopupWindow::handlePopupPositionChangeFromWindowSystem(const QPoint &
 
     const auto windowPos = global2Local(pos);
     if (Q_LIKELY(windowPos)) {
-        qCDebug(lcPopupWindow) << "A window system event changed the popup's position to be " << *windowPos;
+        qCDebug(lcPopupWindow).nospace() << "A window system event changed the popup's"
+            << " (" << d->m_popup << ") position to " << *windowPos;
         QQuickPopupPrivate::get(popup)->setEffectivePosFromWindowPos(*windowPos);
     }
 }
