@@ -30,6 +30,7 @@ QT_BEGIN_NAMESPACE
     tip text can be any \l{Rich Text Processing}{rich text} formatted string.
 
     \image qtquickcontrols-tooltip.png
+           {Tooltip displaying helpful text}
 
     \section2 Attached Tool Tips
 
@@ -83,6 +84,7 @@ QT_BEGIN_NAMESPACE
     the value of a slider when the handle is dragged.
 
     \image qtquickcontrols-tooltip-slider.png
+           {Tooltip attached to slider showing current value}
 
     \snippet qtquickcontrols-tooltip-slider.qml 1
 
@@ -95,6 +97,7 @@ class QQuickToolTipPrivate : public QQuickPopupPrivate
     Q_DECLARE_PUBLIC(QQuickToolTip)
 
 public:
+    QQuickToolTipPrivate();
     void startDelay();
     void stopDelay();
 
@@ -103,8 +106,6 @@ public:
 
     void opened() override;
 
-    Qt::WindowFlags popupWindowType() const override;
-
     QPalette defaultPalette() const override { return QQuickTheme::palette(QQuickTheme::ToolTip); }
 
     int delay = 0;
@@ -114,6 +115,11 @@ public:
     QBasicTimer timeoutTimer;
 };
 
+QQuickToolTipPrivate::QQuickToolTipPrivate()
+{
+    windowFlags = Qt::ToolTip;
+}
+
 void QQuickToolTipPrivate::startDelay()
 {
     Q_Q(QQuickToolTip);
@@ -144,11 +150,6 @@ void QQuickToolTipPrivate::opened()
     startTimeout();
 }
 
-Qt::WindowFlags QQuickToolTipPrivate::popupWindowType() const
-{
-    return Qt::ToolTip;
-}
-
 QQuickToolTip::QQuickToolTip(QQuickItem *parent)
     : QQuickPopup(*(new QQuickToolTipPrivate), parent)
 {
