@@ -5,16 +5,17 @@
 info: |
     This test is actually testing the [[Delete]] internal method (8.12.8). Since the
     language provides no way to directly exercise [[Delete]], the tests are placed here.
-es5id: 11.4.1-4.a-7
-description: delete operator inside 'eval'
+es5id: 11.4.1-0-1
+description: delete operator as UnaryExpression
 flags: [noStrict]
 ---*/
 
 function testcase() {
   var x = 1;
-  var d = eval("delete x");
+  var y = 2;
+  var z = 3;
 
-  assert.sameValue(d, false, 'd');
-  assert.sameValue(x, 1, 'x');
+  assert((!delete x || delete y), '(!delete x || delete y)');
+  assert(delete delete z, 'delete delete z');
  }
 testcase();
