@@ -9,10 +9,11 @@
 
 #include <QtTest/qtest.h>
 
-#include <QtQml/private/qtqml-config_p.h>
 #include <QtQml/qqmlcomponent.h>
 #include <QtQml/qqmlengine.h>
 #include <QtQml/qqmlprivate.h>
+#include <QtQml/qtqmlglobal.h>
+#include <QtQml/private/qtqmlglobal_p.h>
 
 #include <QtCore/qcoreapplication.h>
 #include <QtCore/qfile.h>
@@ -92,6 +93,14 @@ void tst_qmltyperegistrar::qmltypesHasFileNames()
     QVERIFY(qmltypesData.contains("file: \"tst_qmltyperegistrar.h\""));
 }
 
+void tst_qmltyperegistrar::qmltypesHasVirtualAndOverride()
+{
+    QVERIFY(qmltypesData.contains(
+            R"(Property { name: "virtualP"; type: "int"; index: 0; lineNumber: 924; isVirtual: true })"));
+    QVERIFY(qmltypesData.contains(
+            R"(Property { name: "virtualP"; type: "QString"; index: 0; lineNumber: 933; isOverride: true })"));
+}
+
 void tst_qmltyperegistrar::qmltypesHasFlags()
 {
     QVERIFY(qmltypesData.contains("name: \"Flags\""));
@@ -104,9 +113,26 @@ void tst_qmltyperegistrar::superAndForeignTypes()
     QVERIFY(qmltypesData.contains("values: [\"Pixel\", \"Centimeter\", \"Inch\", \"Point\"]"));
     QVERIFY(qmltypesData.contains("name: \"SizeGadget\""));
     QVERIFY(qmltypesData.contains("prototype: \"SizeEnums\""));
-    QVERIFY(qmltypesData.contains("Property { name: \"height\"; type: \"int\"; read: \"height\"; write: \"setHeight\"; index: 0; isFinal: true }"));
-    QVERIFY(qmltypesData.contains("Property { name: \"width\"; type: \"int\"; read: \"width\"; write: \"setWidth\"; index: 0; isFinal: true }"));
-    QVERIFY(qmltypesData.contains("Method { name: \"sizeToString\"; type: \"QString\"; isMethodConstant: true }"));
+    QVERIFY(qmltypesData.contains(R"(Property {
+            name: "height"
+            type: "int"
+            read: "height"
+            write: "setHeight"
+            index: 0
+            lineNumber: 31
+            isFinal: true
+        })"));
+    QVERIFY(qmltypesData.contains(R"(Property {
+            name: "width"
+            type: "int"
+            read: "width"
+            write: "setWidth"
+            index: 0
+            lineNumber: 94
+            isFinal: true
+        })"));
+    QVERIFY(qmltypesData.contains("Method { name: \"sizeToString\"; type: \"QString\"; "
+                                  "isMethodConstant: true; lineNumber: 100 }"));
     QCOMPARE(qmltypesData.count("extension: \"SizeValueType\""), 1);
 }
 
@@ -118,7 +144,13 @@ void tst_qmltyperegistrar::accessSemantics()
 
 void tst_qmltyperegistrar::isBindable()
 {
-    QVERIFY(qmltypesData.contains(R"(Property { name: "someProperty"; type: "int"; bindable: "bindableSomeProperty"; index: 0 })"));
+    QVERIFY(qmltypesData.contains(R"(Property {
+            name: "someProperty"
+            type: "int"
+            bindable: "bindableSomeProperty"
+            index: 0
+            lineNumber: 162
+        })"));
 }
 
 void tst_qmltyperegistrar::doNotRestrictToImportVersion()
@@ -250,7 +282,7 @@ void tst_qmltyperegistrar::finalProperty()
 {
     QCOMPARE(qmltypesData.count("name: \"FinalProperty\""), 1);
     QCOMPARE(qmltypesData.count(
-                     "Property { name: \"fff\"; type: \"int\"; index: 0; isFinal: true }"),
+                     "Property { name: \"fff\"; type: \"int\"; index: 0; lineNumber: 244; isFinal: true }"),
              1);
 }
 
@@ -426,8 +458,8 @@ void tst_qmltyperegistrar::duplicateExportWarnings()
     MetaTypesJsonProcessor processor(true);
     QVERIFY(processor.processTypes({ ":/duplicatedExports.json" }));
     processor.postProcessTypes();
-    QVector<MetaType> types = processor.types();
-    QVector<MetaType> typesforeign = processor.foreignTypes();
+    QList<MetaType> types = processor.types();
+    QList<MetaType> typesforeign = processor.foreignTypes();
     r.setTypes(types, typesforeign);
 
     const auto expectWarning = [](const char *message) {
@@ -477,8 +509,8 @@ void tst_qmltyperegistrar::consistencyWarnings()
 
     processor.postProcessForeignTypes();
 
-    QVector<MetaType> types = processor.types();
-    QVector<MetaType> typesforeign = processor.foreignTypes();
+    QList<MetaType> types = processor.types();
+    QList<MetaType> typesforeign = processor.foreignTypes();
     r.setTypes(types, typesforeign);
 
     QString outputData;
@@ -555,8 +587,8 @@ void tst_qmltyperegistrar::enumWarnings()
     processor.postProcessTypes();
     processor.postProcessForeignTypes();
 
-    QVector<MetaType> types = processor.types();
-    QVector<MetaType> typesforeign = processor.foreignTypes();
+    QList<MetaType> types = processor.types();
+    QList<MetaType> typesforeign = processor.foreignTypes();
     r.setTypes(types, typesforeign);
 
     QString outputData;
@@ -574,16 +606,18 @@ void tst_qmltyperegistrar::clonedSignal()
 {
     QVERIFY(qmltypesData.contains(R"(Signal {
             name: "clonedSignal"
+            lineNumber: 548
             Parameter { name: "i"; type: "int" }
         })"));
 
-    QVERIFY(qmltypesData.contains(R"(Signal { name: "clonedSignal"; isCloned: true })"));
+    QVERIFY(qmltypesData.contains(R"(Signal { name: "clonedSignal"; isCloned: true; lineNumber: 548 })"));
 }
 
 void tst_qmltyperegistrar::hasIsConstantInParameters()
 {
     QVERIFY(qmltypesData.contains(R"(        Signal {
             name: "mySignal"
+            lineNumber: 17
             Parameter { name: "myObject"; type: "QObject"; isPointer: true }
             Parameter { name: "myConstObject"; type: "QObject"; isPointer: true; isTypeConstant: true }
             Parameter { name: "myConstObject2"; type: "QObject"; isPointer: true; isTypeConstant: true }
@@ -594,6 +628,7 @@ void tst_qmltyperegistrar::hasIsConstantInParameters()
 
     QVERIFY(qmltypesData.contains(R"(Signal {
             name: "myVolatileSignal"
+            lineNumber: 19
             Parameter { name: "a"; type: "volatile QObject"; isPointer: true; isTypeConstant: true }
             Parameter { name: "b"; type: "volatile QObject"; isPointer: true; isTypeConstant: true }
             Parameter { name: "nonConst"; type: "volatile QObject"; isPointer: true }
@@ -783,9 +818,10 @@ void tst_qmltyperegistrar::constructibleValueType()
         Method {
             name: "Constructible"
             isConstructor: true
+            lineNumber: 564
             Parameter { name: "i"; type: "int" }
         }
-        Method { name: "Constructible"; isCloned: true; isConstructor: true }
+        Method { name: "Constructible"; isCloned: true; isConstructor: true; lineNumber: 564 }
     })"));
 }
 
@@ -800,7 +836,7 @@ void tst_qmltyperegistrar::structuredValueType()
         exports: ["QmlTypeRegistrarTest/structured 1.0"]
         isStructured: true
         exportMetaObjectRevisions: [256]
-        Property { name: "i"; type: "int"; index: 0; isFinal: true }
+        Property { name: "i"; type: "int"; index: 0; lineNumber: 575; isFinal: true }
     })"));
 }
 
@@ -842,56 +878,67 @@ void tst_qmltyperegistrar::typedEnum()
         Enum {
             name: "UChar"
             type: "quint8"
+            lineNumber: 604
             values: ["V0"]
         }
         Enum {
             name: "Int8_T"
             type: "qint8"
+            lineNumber: 606
             values: ["V1"]
         }
         Enum {
             name: "UInt8_T"
             type: "quint8"
+            lineNumber: 608
             values: ["V2"]
         }
         Enum {
             name: "Int16_T"
             type: "short"
+            lineNumber: 610
             values: ["V3"]
         }
         Enum {
             name: "UInt16_T"
             type: "ushort"
+            lineNumber: 612
             values: ["V4"]
         }
         Enum {
             name: "Int32_T"
             type: "int"
+            lineNumber: 614
             values: ["V5"]
         }
         Enum {
             name: "UInt32_T"
             type: "uint"
+            lineNumber: 616
             values: ["V6"]
         }
         Enum {
             name: "S"
             type: "short"
+            lineNumber: 622
             values: ["A", "B", "C"]
         }
         Enum {
             name: "T"
             type: "ushort"
+            lineNumber: 627
             values: ["D", "E", "F"]
         }
         Enum {
             name: "U"
             type: "qint8"
+            lineNumber: 632
             values: ["G", "H", "I"]
         }
         Enum {
             name: "V"
             type: "quint8"
+            lineNumber: 637
             values: ["J", "K", "L"]
         }
     })"));
@@ -908,6 +955,7 @@ void tst_qmltyperegistrar::listSignal()
         prototype: "QObject"
         Signal {
             name: "objectListHappened"
+            lineNumber: 649
             Parameter { type: "QObjectList" }
         }
     })"));
@@ -926,6 +974,7 @@ void tst_qmltyperegistrar::withNamespace()
             type: "int"
             read: "bar"
             index: 0
+            lineNumber: 655
             isReadonly: true
             isPropertyConstant: true
         }
@@ -944,6 +993,7 @@ void tst_qmltyperegistrar::withNamespace()
             type: "int"
             read: "bar"
             index: 0
+            lineNumber: 676
             isReadonly: true
             isPropertyConstant: true
         }
@@ -960,6 +1010,7 @@ void tst_qmltyperegistrar::withNamespace()
             type: "int"
             read: "foo"
             index: 0
+            lineNumber: 666
             isReadonly: true
             isPropertyConstant: true
         }
@@ -1003,7 +1054,15 @@ void tst_qmltyperegistrar::valueTypeSelfReference()
         lineNumber: 708
         name: "QPersistentModelIndexValueType"
         accessSemantics: "value"
-        Property { name: "row"; type: "int"; read: "row"; index: 0; isReadonly: true; isFinal: true }
+        Property {
+            name: "row"
+            type: "int"
+            read: "row"
+            index: 0
+            lineNumber: 711
+            isReadonly: true
+            isFinal: true
+        }
     })"));
 }
 
@@ -1115,10 +1174,10 @@ void tst_qmltyperegistrar::longNumberTypes()
         prototype: "QObject"
         exports: ["QmlTypeRegistrarTest/LongNumberTypes 1.0"]
         exportMetaObjectRevisions: [256]
-        Property { name: "a"; type: "qlonglong"; index: 0 }
-        Property { name: "b"; type: "qlonglong"; index: 1 }
-        Property { name: "c"; type: "qulonglong"; index: 2 }
-        Property { name: "d"; type: "qulonglong"; index: 3 }
+        Property { name: "a"; type: "qlonglong"; index: 0; lineNumber: 772 }
+        Property { name: "b"; type: "qlonglong"; index: 1; lineNumber: 773 }
+        Property { name: "c"; type: "qulonglong"; index: 2; lineNumber: 774 }
+        Property { name: "d"; type: "qulonglong"; index: 3; lineNumber: 775 }
     })"));
 }
 
@@ -1142,7 +1201,13 @@ void tst_qmltyperegistrar::constReturnType()
         prototype: "QObject"
         exports: ["QmlTypeRegistrarTest/ConstInvokable 1.0"]
         exportMetaObjectRevisions: [256]
-        Method { name: "getObject"; type: "QObject"; isPointer: true; isTypeConstant: true }
+        Method {
+            name: "getObject"
+            type: "QObject"
+            isPointer: true
+            isTypeConstant: true
+            lineNumber: 796
+        }
     })"));
 }
 
@@ -1156,7 +1221,15 @@ void tst_qmltyperegistrar::usingDeclaration()
         prototype: "QObject"
         exports: ["QmlTypeRegistrarTest/WithMyInt 1.0"]
         exportMetaObjectRevisions: [256]
-        Property { name: "a"; type: "int"; read: "a"; index: 0; isReadonly: true; isPropertyConstant: true }
+        Property {
+            name: "a"
+            type: "int"
+            read: "a"
+            index: 0
+            lineNumber: 812
+            isReadonly: true
+            isPropertyConstant: true
+        }
     })"));
 }
 
@@ -1205,9 +1278,9 @@ void tst_qmltyperegistrar::slotsBeforeInvokables()
         name: "SlotsBeforeInvokables"
         accessSemantics: "reference"
         prototype: "QObject"
-        Method { name: "bar" }
-        Method { name: "foo" }
-        Method { name: "baz" }
+        Method { name: "bar"; lineNumber: 833 }
+        Method { name: "foo"; lineNumber: 831 }
+        Method { name: "baz"; lineNumber: 835 }
     })"));
 }
 
@@ -1221,7 +1294,7 @@ void tst_qmltyperegistrar::omitQQmlV4FunctionPtrArg()
         prototype: "QObject"
         exports: ["QmlTypeRegistrarTest/JavaScriptFunction 1.0"]
         exportMetaObjectRevisions: [256]
-        Method { name: "jsfunc"; isJavaScriptFunction: true }
+        Method { name: "jsfunc"; isJavaScriptFunction: true; lineNumber: 844 }
     })"));
 }
 
@@ -1241,6 +1314,7 @@ void tst_qmltyperegistrar::preserveVoidStarPropTypes()
             isPointer: true
             read: "void1"
             index: 0
+            lineNumber: 858
             isReadonly: true
             isPropertyConstant: true
         }
@@ -1250,6 +1324,7 @@ void tst_qmltyperegistrar::preserveVoidStarPropTypes()
             isPointer: true
             read: "void2"
             index: 1
+            lineNumber: 859
             isReadonly: true
             isPropertyConstant: true
         }
@@ -1271,7 +1346,7 @@ void tst_qmltyperegistrar::inaccessibleBase()
         name: "InaccessibleBase"
         accessSemantics: "reference"
         prototype: "QObject"
-        Property { name: "a"; type: "int"; index: 0; isPropertyConstant: true }
+        Property { name: "a"; type: "int"; index: 0; lineNumber: 12; isPropertyConstant: true }
     })"));
 
     QVERIFY(!qmltypesData.contains(R"(name: "InaccessibleProperty")"));
@@ -1289,6 +1364,7 @@ void tst_qmltyperegistrar::inaccessibleBase()
             type: "InaccessibleProperty"
             isPointer: true
             index: 0
+            lineNumber: 872
             isPropertyConstant: true
         }
     })"));
@@ -1333,7 +1409,15 @@ void tst_qmltyperegistrar::derivedFromInvisible()
         prototype: "InvisibleBase"
         exports: ["QmlTypeRegistrarTest/DerivedFromInvisible 1.0"]
         exportMetaObjectRevisions: [256]
-        Property { name: "b"; type: "int"; read: "b"; index: 0; isReadonly: true; isPropertyConstant: true }
+        Property {
+            name: "b"
+            type: "int"
+            read: "b"
+            index: 0
+            lineNumber: 890
+            isReadonly: true
+            isPropertyConstant: true
+        }
     })"));
 }
 
@@ -1351,6 +1435,7 @@ void tst_qmltyperegistrar::foreignNamespacedWithEnum()
         Enum {
             name: "Enum"
             isScoped: true
+            lineNumber: 903
             values: ["ValueA", "ValueB"]
         }
     })"));
@@ -1368,7 +1453,8 @@ void tst_qmltyperegistrar::verifyJsRoot()
     QTemporaryDir dir;
 
     QProcess jsrootProcess;
-    connect(&jsrootProcess, &QProcess::errorOccurred, [&](QProcess::ProcessError error) {
+    connect(&jsrootProcess, &QProcess::errorOccurred,
+            this, [&jsrootProcess](QProcess::ProcessError error) {
         qWarning() << error << jsrootProcess.errorString();
     });
     jsrootProcess.setWorkingDirectory(dir.path());
