@@ -69,6 +69,9 @@ class tst_qqmljsscope : public QQmlDataTest
         QQmlJSLogger logger;
         logger.setFilePath(resolvedFile);
         logger.setCode(sourceCode);
+        // some files shadow properties on purpose
+        logger.setCategoryIgnored(qmlShadow, true);
+        logger.setCategoryIgnored(qmlPropertyOverride, true);
         logger.setSilent(expectErrorsOrWarnings);
         QQmlJSScope::Ptr target = QQmlJSScope::create();
         target->setOwnModuleName(u"HelloModule"_s);
@@ -90,6 +93,8 @@ private Q_SLOTS:
     void initTestCase() override;
 
     void orderedBindings();
+    void scriptBindingValueType_data();
+    void scriptBindingValueType();
     void signalCreationDifferences();
     void allTypesAvailable();
     void shadowing();
@@ -121,6 +126,8 @@ private Q_SLOTS:
     void modulePrefixes();
     void javaScriptBuiltinFlag();
     void isRoot();
+    void idLocation();
+    void lineNumber();
 
 public:
     tst_qqmljsscope()
@@ -132,7 +139,7 @@ public:
                           // Note: to be able to import the QQmlJSScopeTests
                           // correctly, we need an additional import path. Use
                           // this application's binary directory as done by
-                          // QQmlImportDatabase
+                          // QQmlTypeLoader
                           QCoreApplication::applicationDirPath(),
                   },
                   nullptr)
@@ -185,6 +192,32 @@ void tst_qqmljsscope::orderedBindings()
     QCOMPARE(std::next(itemsBindingsBegin)->objectType()->baseTypeName(), u"Text"_s);
 }
 
+void tst_qqmljsscope::scriptBindingValueType_data()
+{
+    QTest::addColumn<ScriptBindingValueType>("expectedType");
+
+    QTest::addRow("myInt") << ScriptBindingValueType::ScriptValue_Unknown;
+    QTest::addRow("block") << ScriptBindingValueType::ScriptValue_Function;
+    QTest::addRow("alsoABlock") << ScriptBindingValueType::ScriptValue_Function;
+    QTest::addRow("lambda") << ScriptBindingValueType::ScriptValue_Function;
+    QTest::addRow("namedFunction") << ScriptBindingValueType::ScriptValue_Function;
+    QTest::addRow("arrow") << ScriptBindingValueType::ScriptValue_Function;
+    QTest::addRow("myUndefined") << ScriptBindingValueType::ScriptValue_Undefined;
+    QTest::addRow("emptyBlock") << ScriptBindingValueType::ScriptValue_Function;
+    QTest::addRow("emptyObject") << ScriptBindingValueType::ScriptValue_Unknown;
+}
+void tst_qqmljsscope::scriptBindingValueType()
+{
+    QFETCH(ScriptBindingValueType, expectedType);
+
+    QQmlJSScope::ConstPtr root = run(u"scriptBindingValueType.qml"_s);
+    QVERIFY(root);
+
+    auto [start, end] = root->ownPropertyBindings(QTest::currentDataTag());
+    QCOMPARE(std::distance(start, end), 1);
+    QCOMPARE(start->scriptValueType(), expectedType);
+}
+
 void tst_qqmljsscope::signalCreationDifferences()
 {
     QQmlJSScope::ConstPtr root = run(u"signalCreationDifferences.qml"_s, true);
@@ -460,13 +493,13 @@ void tst_qqmljsscope::attachedSignalHandler()
     QQmlJSScope::ConstPtr root = run(u"attachedSignalHandler.qml"_s);
     QVERIFY(root);
 
-    auto binding = std::find_if(root->childScopesBegin(), root->childScopesEnd(),
-                                [](const QQmlJSScope::ConstPtr &child) {
-                                    return child->baseTypeName() == "binding"_L1;
-                                });
+    const auto binding = std::find_if(root->childScopesBegin(), root->childScopesEnd(),
+                                      [](const QQmlJSScope::ConstPtr &child) {
+                                          return child->baseTypeName() == "signalHandler"_L1;
+                                      });
     QCOMPARE_NE(binding, root->childScopesEnd());
 
-    auto block = std::find_if(
+    const auto block = std::find_if(
             (**binding).childScopesBegin(), (**binding).childScopesEnd(),
             [](const QQmlJSScope::ConstPtr &child) { return child->baseTypeName() == "block"_L1; });
     QCOMPARE_NE(block, (**binding).childScopesEnd());
@@ -705,10 +738,19 @@ void tst_qqmljsscope::extensions()
     QCOMPARE(childScopes[4]->attachedTypeName(), QString());
     QVERIFY(!childScopes[4]->attachedType());
 
-    auto [owner, ownerKind] = QQmlJSScope::ownerOfProperty(childScopes[4], u"count"_s);
-    QVERIFY(owner);
-    QCOMPARE(ownerKind, QQmlJSScope::ExtensionType);
-    QCOMPARE(owner, childScopes[4]->baseType()->extensionType().scope);
+    {
+        auto [owner, ownerKind] = QQmlJSScope::ownerOfProperty(childScopes[4], u"count"_s);
+        QVERIFY(owner);
+        QCOMPARE(ownerKind, QQmlJSScope::ExtensionType);
+        QCOMPARE(owner, childScopes[4]->baseType()->extensionType().scope);
+    }
+
+    {
+        auto [owner, ownerKind] = QQmlJSScope::ownerOfMethod(childScopes[4], u"myInvokable"_s);
+        QVERIFY(owner);
+        QCOMPARE(ownerKind, QQmlJSScope::ExtensionType);
+        QCOMPARE(owner, childScopes[4]->baseType()->extensionType().scope);
+    }
 }
 
 void tst_qqmljsscope::emptyBlockBinding()
@@ -833,7 +875,7 @@ void tst_qqmljsscope::compilationUnitsAreCompatible()
     QVERIFY2(root, qPrintable(component.errorString()));
     QQmlComponentPrivate *cPriv = QQmlComponentPrivate::get(&component);
     QVERIFY(cPriv);
-    auto unit = cPriv->compilationUnit;
+    auto unit = cPriv->compilationUnit();
     QVERIFY(unit);
     QVERIFY(unit->unitData());
     getRuntimeInfoFromCompilationUnit(unit->unitData(), componentFunctions);
@@ -916,7 +958,7 @@ void tst_qqmljsscope::attachedTypeResolution()
     const auto &implicitImportDirectory = QQmlJSImportVisitor::implicitImportDirectory(
             logger->filePath(), importer.resourceFileMapper());
     QQmlJSImportVisitor v{
-        QQmlJSScope::create(), &importer, logger.get(), implicitImportDirectory, {}
+        &importer, logger.get(), implicitImportDirectory, {}
     };
 
     PassManagerPtr manager(
@@ -974,7 +1016,7 @@ void tst_qqmljsscope::builtinTypeResolution()
     const auto &implicitImportDirectory = QQmlJSImportVisitor::implicitImportDirectory({}, nullptr);
     QQmlJSLogger logger;
     QQmlJSImportVisitor v{
-        QQmlJSScope::create(), &importer, &logger, implicitImportDirectory, {}
+        &importer, &logger, implicitImportDirectory, {}
     };
 
     PassManagerPtr manager(
@@ -1022,7 +1064,6 @@ void tst_qqmljsscope::modulePrefixes()
     logger.setFilePath(url);
     logger.setCode(sourceCode);
 
-    QQmlJSScope::Ptr target = QQmlJSScope::create();
     QmlIR::Document document(url, url, false);
     QQmlJSSaveFunction noop([](auto &&...) { return true; });
     QQmlJSCompileError error;
@@ -1032,7 +1073,7 @@ void tst_qqmljsscope::modulePrefixes()
     if (!error.message.isEmpty())
         return;
 
-    QQmlJSImportVisitor visitor(target, &m_importer, &logger, dataDirectory());
+    QQmlJSImportVisitor visitor(&m_importer, &logger, dataDirectory());
     QQmlJSTypeResolver typeResolver{ &m_importer };
     typeResolver.init(&visitor, document.program);
 
@@ -1049,7 +1090,6 @@ void tst_qqmljsscope::javaScriptBuiltinFlag()
     logger.setFilePath(url);
     logger.setCode(loadFile(url));
 
-    QQmlJSScope::Ptr target = QQmlJSScope::create();
     QmlIR::Document document(url, url, false);
     QQmlJSSaveFunction noop([](auto &&...) { return true; });
     QQmlJSCompileError error;
@@ -1059,7 +1099,7 @@ void tst_qqmljsscope::javaScriptBuiltinFlag()
     if (!error.message.isEmpty())
         return;
 
-    QQmlJSImportVisitor visitor(target, &m_importer, &logger, dataDirectory());
+    QQmlJSImportVisitor visitor( &m_importer, &logger, dataDirectory());
     QQmlJSTypeResolver typeResolver{ &m_importer };
     typeResolver.init(&visitor, document.program);
 
@@ -1084,5 +1124,28 @@ void tst_qqmljsscope::isRoot()
     }
 }
 
+void tst_qqmljsscope::idLocation()
+{
+    auto jsscope = run(u"idSourceLocation.qml"_s, false);
+    const QQmlJS::SourceLocation actual = jsscope->idSourceLocation();
+    QVERIFY(actual.isValid());
+    QCOMPARE(actual.startLine, 7);
+    QCOMPARE(actual.startColumn, 25);
+    QCOMPARE(actual.length, 10);
+}
+
+void tst_qqmljsscope::lineNumber()
+{
+    const auto jsscope = run(u"UseTypeWithProperties.qml"_s, false);
+    const auto baseFromCpp = jsscope->baseType();
+    QVERIFY(baseFromCpp);
+    QCOMPARE(baseFromCpp->lineNumber(), 13);
+    QCOMPARE(baseFromCpp->property("a").sourceLocation().startLine, 18);
+    QCOMPARE(baseFromCpp->methods("myInvokable").front().sourceLocation().startLine, 26);
+    QCOMPARE(baseFromCpp->enumeration("MyEnum").lineNumber(), 28);
+    QCOMPARE(baseFromCpp->methods("aChanged").front().sourceLocation().startLine, 32);
+    QCOMPARE(baseFromCpp->methods("mySlot").front().sourceLocation().startLine, 34);
+}
+
 QTEST_MAIN(tst_qqmljsscope)
 #include "tst_qqmljsscope.moc"
