@@ -56,7 +56,7 @@ public:
     tst_qqmllistmodel()
         : QQmlDataTest(QT_QMLTEST_DATADIR)
     {
-        qRegisterMetaType<QVector<int> >();
+        qRegisterMetaType<QList<int> >();
     }
 
 private:
@@ -125,6 +125,7 @@ private slots:
     void deadModelData();
     void valuesOfInnerList();
     void arrayLikes();
+    void functionInNested();
 };
 
 bool tst_qqmllistmodel::compareVariantList(const QVariantList &testList, QVariant object)
@@ -816,7 +817,7 @@ void tst_qqmllistmodel::get()
     QList<QVariant> spyResult = spy.takeFirst();
     QCOMPARE(spyResult.at(0).value<QModelIndex>(), model->index(index, 0, QModelIndex()));
     QCOMPARE(spyResult.at(1).value<QModelIndex>(), model->index(index, 0, QModelIndex()));  // only 1 item is modified at a time
-    QCOMPARE(spyResult.at(2).value<QVector<int> >(), (QVector<int>() << role));
+    QCOMPARE(spyResult.at(2).value<QList<int> >(), (QList<int>() << role));
 }
 
 void tst_qqmllistmodel::get_data()
@@ -938,7 +939,7 @@ void tst_qqmllistmodel::get_nested()
         QList<QVariant> spyResult = spy.takeFirst();
         QCOMPARE(spyResult.at(0).value<QModelIndex>(), childModel->index(index, 0, QModelIndex()));
         QCOMPARE(spyResult.at(1).value<QModelIndex>(), childModel->index(index, 0, QModelIndex()));  // only 1 item is modified at a time
-        QCOMPARE(spyResult.at(2).value<QVector<int> >(), (QVector<int>() << role));
+        QCOMPARE(spyResult.at(2).value<QList<int> >(), (QList<int>() << role));
     }
 }
 
@@ -1747,7 +1748,7 @@ void tst_qqmllistmodel::objectDestroyed()
             QUrl());
 
     std::unique_ptr<QObject> obj = std::make_unique<QObject>();
-    connect(obj.get(), &QObject::destroyed, [&]() { obj.release(); });
+    connect(obj.get(), &QObject::destroyed, obj.get(), [&obj] { obj.release(); });
 
     engine.rootContext()->setContextProperty(u"contextObject"_s, obj.get());
     engine.setObjectOwnership(obj.get(), QJSEngine::JavaScriptOwnership);
@@ -2198,6 +2199,18 @@ void tst_qqmllistmodel::arrayLikes()
     QCOMPARE(o->property("s").toString(), "2t");
 }
 
+void tst_qqmllistmodel::functionInNested()
+{
+    QQmlEngine engine;
+    QQmlComponent c(&engine, testFileUrl("functionInNested.qml"));
+    QVERIFY2(c.isReady(), qPrintable(c.errorString()));
+
+    QTest::ignoreMessage(QtDebugMsg, "called");
+
+    QScopedPointer<QObject> o(c.create());
+    QVERIFY(!o.isNull());
+}
+
 QTEST_MAIN(tst_qqmllistmodel)
 
 #include "tst_qqmllistmodel.moc"
