@@ -38,6 +38,23 @@ private slots:
     void overriddenSignals();
     void duplicateIdsAndGeneralizedGroupProperties();
 
+    void appendPropertyAttr_logging_data();
+    void appendPropertyAttr_logging();
+
+    void appendPropertyAttr_InvalidOverride_data();
+    void appendPropertyAttr_InvalidOverride();
+
+    void appendPropertyAttr_ValidOverride_data();
+    void appendPropertyAttr_ValidOverride();
+
+    void handleOverride_data();
+    void handleOverride();
+
+    void nonExistentGeneralizedGroup();
+
+    void append_propertyAttr_data();
+    void append_propertyAttr();
+
 private:
     QQmlEngine engine;
 };
@@ -558,8 +575,6 @@ class TestClassWithClassInfo : public QObject
     Q_CLASSINFO("Key", "Value")
 };
 
-#include "tst_qqmlpropertycache.moc"
-
 template <typename T, typename = void>
 struct SizeofOffsetsAndSizes_helper
 {
@@ -789,4 +804,351 @@ void tst_qqmlpropertycache::duplicateIdsAndGeneralizedGroupProperties()
     QScopedPointer<QObject> o(c.create());
 }
 
+// TODO consider moving to test utils
+static inline auto propertyWithFlags(QQmlPropertyData::Flags &&flags = QQmlPropertyData::Flags())
+        -> QQmlPropertyData
+{
+    QQmlPropertyData property{};
+    property.setFlags(std::move(flags));
+    return property;
+};
+
+using namespace OverrideSemantics;
+
+static inline auto newPropertyCache(HandlerRef fakeOverrideHandler) -> QQmlPropertyCache::Ptr
+{
+    return QQmlPropertyCache::Ptr(new QQmlPropertyCache(fakeOverrideHandler),
+                                  QQmlPropertyCache::Ptr::Adopt);
+}
+
+void tst_qqmlpropertycache::appendPropertyAttr_logging_data()
+{
+    QTest::addColumn<OverrideSemantics::Status>("overrideStatus");
+    QTest::addColumn<QString>("warningPattern");
+    QTest::addColumn<QtMsgType>("level");
+
+    QTest::newRow("NoOverride") << Status::NoOverride << "" << QtInfoMsg;
+    QTest::newRow("Valid") << Status::Valid << "" << QtInfoMsg;
+
+    QTest::newRow("MissingBase")
+            << Status::MissingBase
+            << "Member (.*) of the object (.*) does not override anything. Consider "
+               "removing \"override\"."
+            << QtWarningMsg;
+    QTest::newRow("OverridingFinal")
+            << Status::OverridingFinal
+            << "Final member (.*) is overridden in class (.*). The override won't be used."
+            << QtWarningMsg;
+    QTest::newRow("OverridingNonVirtual")
+            << Status::OverridingNonVirtual
+            << "Member (.*) of the object (.*) overrides a non-virtual "
+               "member. Consider renaming it or mark it virtual in the base object"
+            << QtDebugMsg;
+    QTest::newRow("OverridingNonVirtualError")
+            << Status::OverridingNonVirtualError
+            << "Member (.*) of the object (.*) overrides a non-virtual "
+               "member. Consider renaming it or mark it virtual in the base object"
+            << QtWarningMsg;
+    QTest::newRow("MissingOverrideSpecifier")
+            << Status::MissingOverrideOrFinalSpecifier
+            << "Member (.*) of the object (.*) overrides a member of the base object. "
+               "Consider renaming it or adding final or override specifier"
+            << QtWarningMsg;
+}
+
+static QLoggingCategory::CategoryFilter parentFilter;
+void logFilter(QLoggingCategory *category)
+{
+    if (qstrcmp(category->categoryName(), "qt.qml.propertyCache.append") == 0)
+        category->setEnabled(QtDebugMsg, true);
+    else if (parentFilter)
+        parentFilter(category);
+}
+
+
+void tst_qqmlpropertycache::appendPropertyAttr_logging()
+{
+    QFETCH(OverrideSemantics::Status, overrideStatus);
+    QFETCH(QString, warningPattern);
+    QFETCH(QtMsgType, level);
+
+    parentFilter = QLoggingCategory::installFilter(logFilter);
+    const auto restoreFilter = qScopeGuard([]() { QLoggingCategory::installFilter(parentFilter); });
+
+    const auto fakeOverrideHandler = [&overrideStatus](QQmlPropertyData &, QQmlPropertyData *,
+                                                       CheckMode) -> Status {
+        return overrideStatus;
+    };
+
+    if (!warningPattern.isEmpty())
+        QTest::ignoreMessage(level, QRegularExpression(warningPattern.toLatin1()));
+
+    newPropertyCache(fakeOverrideHandler)->appendPropertyAttr("p", QQmlPropertyData());
+}
+
+void tst_qqmlpropertycache::appendPropertyAttr_InvalidOverride_data()
+{
+    QTest::addColumn<OverrideSemantics::Status>("overrideStatus");
+
+    QTest::newRow("MissingBase") << Status::MissingBase;
+    QTest::newRow("OverridingFinal") << Status::OverridingFinal;
+    QTest::newRow("OverridingNonVirtualError") << Status::OverridingNonVirtualError;
+}
+
+void tst_qqmlpropertycache::appendPropertyAttr_InvalidOverride()
+{
+    QFETCH(OverrideSemantics::Status, overrideStatus);
+
+    QVERIFY(!isValidOverride(overrideStatus));
+
+    const auto fakeOverrideHandler = [&overrideStatus](QQmlPropertyData &, QQmlPropertyData *,
+                                                       CheckMode) -> Status {
+        return overrideStatus;
+    };
+
+    QQmlPropertyCache::Ptr cache = newPropertyCache(fakeOverrideHandler);
+    QCOMPARE(cache->propertyCount(), 0);
+
+    const auto res = cache->appendPropertyAttr("p", QQmlPropertyData());
+
+    QVERIFY(!res);
+    QCOMPARE(res.error(), overrideStatus);
+    QCOMPARE(cache->propertyCount(), 1);
+
+    // since append is invalid the property is not accessible by name
+    QVERIFY(!cacheProperty(cache, "p"));
+}
+
+void tst_qqmlpropertycache::appendPropertyAttr_ValidOverride_data()
+{
+    QTest::addColumn<OverrideSemantics::Status>("overrideStatus");
+
+    QTest::newRow("NoOverride") << Status::NoOverride;
+    QTest::newRow("Valid") << Status::Valid;
+    QTest::newRow("MissingOverrideSpecifier") << Status::MissingOverrideOrFinalSpecifier;
+    QTest::newRow("OverridingNonVirtual") << Status::OverridingNonVirtual;
+}
+
+void tst_qqmlpropertycache::appendPropertyAttr_ValidOverride()
+{
+    QFETCH(OverrideSemantics::Status, overrideStatus);
+
+    QVERIFY(isValidOverride(overrideStatus));
+
+    const auto fakeOverrideHandler = [&overrideStatus](QQmlPropertyData &, QQmlPropertyData *,
+                                                       CheckMode) -> Status {
+        return overrideStatus;
+    };
+
+    QQmlPropertyCache::Ptr cache = newPropertyCache(fakeOverrideHandler);
+    auto propCount = 0;
+    QCOMPARE(cache->propertyCount(), propCount);
+
+    const QString propName = "p";
+    {
+        QQmlPropertyData data{};
+        // set just to verify upon query that it's the same data
+        data.setCoreIndex(6);
+
+        const auto res = cache->appendPropertyAttr(propName, QQmlPropertyData(data));
+
+        QVERIFY(res);
+        QCOMPARE(cache->propertyCount(), ++propCount);
+        const auto *addedProperty = cache->property(propName, nullptr, nullptr);
+        QVERIFY(addedProperty);
+        QCOMPARE(*addedProperty, data);
+    }
+    // Now override the property
+    {
+        QQmlPropertyData data{};
+        // set just to verify upon query that it's the same data
+        data.setCoreIndex(3);
+        data.setWritable(false);
+
+        const auto res = cache->appendPropertyAttr(propName, QQmlPropertyData(data));
+
+        QVERIFY(res);
+        QCOMPARE(cache->propertyCount(), ++propCount);
+        const auto *addedProperty = cache->property(propName, nullptr, nullptr);
+        QVERIFY(addedProperty);
+        QCOMPARE(*addedProperty, data);
+    }
+}
+
+void tst_qqmlpropertycache::handleOverride_data()
+{
+    QTest::addColumn<QQmlPropertyData>("overridingProperty");
+    QTest::addColumn<std::optional<QQmlPropertyData>>("existingProperty");
+    QTest::addColumn<CheckMode>("checkMode");
+    QTest::addColumn<Status>("expectedResult");
+
+    const auto makeNullopt = []() -> std::optional<QQmlPropertyData> { return std::nullopt; };
+
+    // Minimal checks
+    QTest::newRow("Derived{property var p} Base{}")
+            << propertyWithFlags() << makeNullopt() << CheckMode::Minimal << Status::NoOverride;
+
+    QQmlPropertyData::Flags final{};
+    final.setIsFinal(true);
+    const auto finalProperty = propertyWithFlags(std::move(final));
+    QTest::newRow("Derived{property var p} Base{final property var p}")
+            << propertyWithFlags() << std::make_optional(finalProperty) << CheckMode::Minimal
+            << Status::OverridingFinal;
+
+    QTest::newRow("Derived{property var p} Base{ property var p}")
+            << propertyWithFlags() << std::make_optional(propertyWithFlags()) << CheckMode::Minimal
+            << Status::Valid;
+
+    // Full
+    QTest::newRow("Derived{property var p} Base{} full")
+            << propertyWithFlags() << makeNullopt() << CheckMode::Full << Status::NoOverride;
+
+    QQmlPropertyData::Flags override{};
+    override.setDoesOverride(true);
+    const auto propertyWithOverride = propertyWithFlags(std::move(override));
+
+    QQmlPropertyData::Flags virtual_{};
+    virtual_.setIsVirtual(true);
+    const auto propertyWithVirtual = propertyWithFlags(std::move(virtual_));
+
+    QTest::newRow("Derived{override property var p} Base{}")
+            << propertyWithOverride << makeNullopt() << CheckMode::Full << Status::MissingBase;
+
+    QTest::newRow("Derived{property var p} Base{final property var p} full")
+            << propertyWithFlags() << std::make_optional(finalProperty) << CheckMode::Full
+            << Status::OverridingFinal;
+    QTest::newRow("Derived{override property var p} Base{final property var p}")
+            << propertyWithOverride << std::make_optional(finalProperty) << CheckMode::Full
+            << Status::OverridingFinal;
+
+    QTest::newRow("Derived{property var p} Base{property var p}")
+            << propertyWithFlags() << std::make_optional(propertyWithFlags()) << CheckMode::Full
+            << Status::OverridingNonVirtual;
+
+    QTest::newRow("Derived{final property var p} Base{property var p}")
+            << finalProperty << std::make_optional(propertyWithFlags()) << CheckMode::Full
+            << Status::OverridingNonVirtual;
+
+    QTest::newRow("Derived{virtual property var p} Base{property var p}")
+            << propertyWithVirtual << std::make_optional(propertyWithFlags()) << CheckMode::Full
+            << Status::OverridingNonVirtual;
+
+    QTest::newRow("Derived{override property var p} Base{property var p}")
+            << propertyWithOverride << std::make_optional(propertyWithFlags()) << CheckMode::Full
+            << Status::OverridingNonVirtualError;
+
+    QTest::newRow("Derived{property var p} Base{virtual property var p}")
+            << propertyWithFlags() << std::make_optional(propertyWithVirtual) << CheckMode::Full
+            << Status::MissingOverrideOrFinalSpecifier;
+
+    QTest::newRow("Derived{virtual property var p} Base{virtual property var p}")
+            << propertyWithVirtual << std::make_optional(propertyWithVirtual) << CheckMode::Full
+            << Status::MissingOverrideOrFinalSpecifier;
+
+    QTest::newRow("Derived{override property var p} Base{virtual property var p}")
+            << propertyWithOverride << std::make_optional(propertyWithVirtual) << CheckMode::Full
+            << Status::Valid;
+
+    QTest::newRow("Derived{final property var p} Base{virtual property var p}")
+            << finalProperty << std::make_optional(propertyWithVirtual) << CheckMode::Full
+            << Status::Valid;
+
+    QQmlPropertyData::Flags invokable{};
+    invokable.setType(QQmlPropertyData::Flags::Type::FunctionType);
+    auto funcProperty = propertyWithFlags(std::move(invokable));
+
+    QTest::newRow("Derived{override p()} Base{virtual property var p}")
+            << funcProperty << std::make_optional(propertyWithFlags(std::move(virtual_)))
+            << CheckMode::Full << Status::InvokabilityMismatch;
+
+    QTest::newRow("Derived{override p} Base{p()}")
+            << propertyWithOverride << std::make_optional(funcProperty) << CheckMode::Full
+            << Status::InvokabilityMismatch;
+}
+
+void tst_qqmlpropertycache::handleOverride()
+{
+    QFETCH(QQmlPropertyData, overridingProperty);
+    QFETCH(std::optional<QQmlPropertyData>, existingProperty);
+    QFETCH(CheckMode, checkMode);
+    QFETCH(Status, expectedResult);
+
+    auto *existing = existingProperty.has_value() ? &existingProperty.value() : nullptr;
+    const auto res = ::handleOverride(overridingProperty, existing, checkMode);
+    QCOMPARE(res, expectedResult);
+    if (isValidOverride(res) && existing) {
+        // test markOverrideOf
+        QCOMPARE(overridingProperty.isVirtual(), existing->isVirtual());
+        QCOMPARE(overridingProperty.overrideIndexIsProperty(), !existing->isFunction());
+        QCOMPARE(overridingProperty.overrideIndex(), existing->coreIndex());
+        QVERIFY(existing->isOverridden());
+    }
+}
+
+void tst_qqmlpropertycache::nonExistentGeneralizedGroup()
+{
+    QQmlEngine engine;
+    QQmlComponent component(&engine);
+    component.setData(R"(
+        import QtQml
+        QtObject{grid{id:grid}}
+    )", QUrl());
+    QVERIFY(component.isError());
+    QVERIFY(component.errorString().contains("Cannot assign to non-existent property \"grid\""));
+}
+
+void tst_qqmlpropertycache::append_propertyAttr_data()
+{
+    appendPropertyAttr_logging_data();
+}
+
+/*
+ * Because of the tightly coupling of the QQmlPropertyCache, this case covers very
+ * narrowed (far from exhaustive) pieces of "append" method through publicly exposed "update"
+ * method.
+ * It verifies that depending on the Override Semantics the corresponding logging takes
+ * place. It also verifies that the property is being added and accessible depending on the status of
+ * override semantics.
+ */
+void tst_qqmlpropertycache::append_propertyAttr()
+{
+    QFETCH(OverrideSemantics::Status, overrideStatus);
+    QFETCH(QString, warningPattern);
+    QFETCH(QtMsgType, level);
+
+    parentFilter = QLoggingCategory::installFilter(logFilter);
+    const auto restoreFilter = qScopeGuard([]() { QLoggingCategory::installFilter(parentFilter); });
+
+
+    const auto fakeOverrideHandler = [&overrideStatus](QQmlPropertyData &, QQmlPropertyData *,
+                                                       CheckMode) -> Status {
+        return overrideStatus;
+    };
+
+    QQmlPropertyCache::Ptr cache = newPropertyCache(fakeOverrideHandler);
+    QCOMPARE(cache->propertyCount(), 0);
+
+    QMetaObjectBuilder moBuilder;
+    const QString propName = "p";
+    moBuilder.addProperty(propName.toUtf8(), "int", -1);
+    const auto *mo = moBuilder.toMetaObject();
+
+    if (!warningPattern.isEmpty())
+        QTest::ignoreMessage(level, QRegularExpression(warningPattern.toLatin1()));
+    cache->update(mo);
+
+    // no matter what the property is added
+    QCOMPARE(cache->propertyCount(), mo->propertyCount());
+    const auto *addedProperty = cache->property(propName, nullptr, nullptr);
+
+    // however it is accessible by name only in cases of valid override
+    if (isValidOverride(overrideStatus)) {
+        QVERIFY(addedProperty);
+    } else {
+        QVERIFY(!addedProperty);
+    }
+}
+
 QTEST_MAIN(tst_qqmlpropertycache)
+
+#include "tst_qqmlpropertycache.moc"
