@@ -7,6 +7,8 @@
 
 #include <QtGui/qaccessible.h>
 #include <QtGui/private/qguiapplication_p.h>
+#include <QtGui/private/qaccessiblecache_p.h>
+
 #include <qpa/qplatformnativeinterface.h>
 #include <qpa/qplatformintegration.h>
 #include <qpa/qplatformaccessibility.h>
@@ -20,11 +22,13 @@
 #include <QtQuick/private/qquicklistview_p.h>
 #include <QtQuick/private/qquicktext_p.h>
 #include <QtQuick/private/qquicktextinput_p.h>
+#include <QtQuick/private/qaccessiblequickitem_p.h>
 
 #include <QtQuickTestUtils/private/qmlutils_p.h>
 #include <QtQuickTestUtils/private/visualtestutils_p.h>
 
 #include <QQmlComponent>
+#include <QQmlApplicationEngine>
 
 using namespace Qt::StringLiterals;
 
@@ -70,6 +74,11 @@ private slots:
     void eventTest();
     void relations_data();
     void relations();
+    void controlsThatShouldSendObjectShow_data();
+    void controlsThatShouldSendObjectShow();
+
+    void editableTextInterface_data();
+    void editableTextInterface();
 };
 
 tst_QQuickAccessible::tst_QQuickAccessible()
@@ -106,7 +115,7 @@ void tst_QQuickAccessible::init()
 
 void tst_QQuickAccessible::cleanup()
 {
-    const EventList list = QTestAccessibility::events();
+    const auto list = QTestAccessibility::events();
     if (!list.isEmpty()) {
         qWarning().noquote() << list.size()
                              << "accessibility event(s) were not handled in testfunction '"
@@ -496,7 +505,7 @@ void tst_QQuickAccessible::basicPropertiesTest()
 
     QAccessibleInterface *item = iface->child(0);
     QVERIFY(item);
-    QCOMPARE(item->childCount(), 6);
+    QCOMPARE(item->childCount(), 7);
     QCOMPARE(item->rect().size(), QSize(400, 400));
     QCOMPARE(item->role(), QAccessible::Client);
     QCOMPARE(iface->indexOfChild(item), 0);
@@ -542,11 +551,19 @@ void tst_QQuickAccessible::basicPropertiesTest()
     auto textInterface = textInput->textInterface();
     QVERIFY(textInterface);
     auto editableTextInterface = textInput->editableTextInterface();
-    QEXPECT_FAIL("", "EditableTextInterface is not implemented", Continue);
     QVERIFY(editableTextInterface);
     auto newText = QString("a new text");
     textInput->setText(QAccessible::Value, newText);
     QCOMPARE(textInput->text(QAccessible::Value), newText);
+    QCOMPARE(textInterface->selectionCount(), 0);
+    textInterface->setSelection(0, 1, 4);
+    QCOMPARE(textInterface->selectionCount(), 1);
+    int selectionStartOffset = 0, selectionEndOffset = 0;
+    textInterface->selection(0, &selectionStartOffset, &selectionEndOffset);
+    QCOMPARE(selectionStartOffset, 1);
+    QCOMPARE(selectionEndOffset, 4);
+    textInterface->removeSelection(0);
+    QCOMPARE(textInterface->selectionCount(), 0);
 
     // TextEdit
     QAccessibleInterface *textEdit = item->child(3);
@@ -571,7 +588,6 @@ void tst_QQuickAccessible::basicPropertiesTest()
     QCOMPARE(textEditTextInterface->selectionCount(), 0);
 
     auto textEditEditableTextInterface = textEdit->editableTextInterface();
-    QEXPECT_FAIL("", "EditableTextInterface is not implemented", Continue);
     QVERIFY(textEditEditableTextInterface);
     textEdit->setText(QAccessible::Value, newText);
     QCOMPARE(textEdit->text(QAccessible::Value), newText);
@@ -595,7 +611,7 @@ void tst_QQuickAccessible::basicPropertiesTest()
 
     // Text "Rich text"
     QAccessibleInterface *richText = item->child(5);
-    QVERIFY(text3);
+    QVERIFY(richText);
     QCOMPARE(richText->childCount(), 2);
     QCOMPARE(richText->text(QAccessible::Name), QLatin1String("Rich text with links:\nWebsite or blog"));
     QCOMPARE(richText->role(), QAccessible::StaticText);
@@ -618,6 +634,17 @@ void tst_QQuickAccessible::basicPropertiesTest()
         QCOMPARE(link->anchorTarget(), QLatin1String(linkUrls[i][1]));
     }
 
+    // TextField
+    QAccessibleInterface *textField = item->child(6);
+    QVERIFY(textField);
+    QCOMPARE(textField->childCount(), 0);
+    QCOMPARE(textField->text(QAccessible::Value), QLatin1String("text is TextField"));
+    QCOMPARE(textField->text(QAccessible::Description), QLatin1String("This is the TextField description"));
+    QCOMPARE(textField->role(), QAccessible::EditableText);
+    QCOMPARE(item->indexOfChild(textField), 6);
+    QVERIFY(textField->state().editable);
+    QVERIFY(!textField->state().readOnly);
+
     // see if implicit changes back
     attached->setRole(QAccessible::EditableText);
     QEXPECT_FAIL("", "EditableText does not implicitly set readOnly to false", Continue);
@@ -860,10 +887,9 @@ void tst_QQuickAccessible::announceTest()
 
 void tst_QQuickAccessible::eventTest()
 {
-    std::unique_ptr<QQuickView, void(*)(QQuickView *)> window(new QQuickView, [](QQuickView *ptr) {
-        delete ptr;
-        QTestAccessibility::clearEvents();
-    });
+    auto clearEvents = qScopeGuard([]{ QTestAccessibility::clearEvents(); });
+    std::unique_ptr<QQuickView> window(new QQuickView());
+
     window->setSource(testFileUrl("eventTest.qml"));
     window->show();
     QVERIFY(QTest::qWaitForWindowExposed(window.get()));
@@ -881,11 +907,20 @@ void tst_QQuickAccessible::eventTest()
 
     // move an item that is accessible
     QQuickItem *buttonItem = rootItem->findChild<QQuickItem*>(QLatin1String("button"));
+    auto buttonIface = QAccessible::queryAccessibleInterface(buttonItem);
+    QVERIFY(buttonIface);
     QTestAccessibility::clearEvents();
     buttonItem->setX(buttonItem->x() + 2);
     QCOMPARE(QTestAccessibility::events().size(), 1);
     QAccessibleEvent ev(buttonItem, QAccessible::LocationChanged);
     QTestAccessibility::verifyEvent(&ev);
+
+    auto windowIface = QAccessible::queryAccessibleInterface(window.get());
+    QAccessibleObjectDestroyedEvent ev2(buttonIface);
+    QAccessibleObjectDestroyedEvent ev3(windowIface);
+    window.reset();
+    QTestAccessibility::verifyEvent(&ev2);
+    QTestAccessibility::verifyEvent(&ev3);
 }
 
 void tst_QQuickAccessible::relations_data()
@@ -934,6 +969,122 @@ void tst_QQuickAccessible::relations()
     QVERIFY(!otherRelations.isEmpty());
 }
 
+void tst_QQuickAccessible::controlsThatShouldSendObjectShow_data()
+{
+    QTest::addColumn<QByteArray>("qmlSnippet");
+
+    QTest::newRow("BusyIndicator") << QByteArray("BusyIndicator { running: true }");
+    QTest::newRow("Button") << QByteArray("Button { text: 'Button' }");
+    QTest::newRow("ComboBox") << QByteArray("ComboBox { model: 3 }");
+    QTest::newRow("Dial") << QByteArray("Dial { value: 0.5 }");
+    // Label without a text is a bit unusual,
+    // but the background can be an image with meaningful info...
+    QTest::newRow("Label") << QByteArray("Label {\nbackground: Rectangle {\ncolor: 'red'\n}\nwidth: 50\nheight: 20}");
+    QTest::newRow("ProgressBar") << QByteArray("ProgressBar { value: 0.5 }");
+    QTest::newRow("RangeSlider") << QByteArray("RangeSlider { from: 1; to: 100; second.value: 50 }");
+    QTest::newRow("RoundButton") << QByteArray("RoundButton { text: 'Yes, please' }");
+    QTest::newRow("Slider") << QByteArray("Slider { value: 0.5 }");
+    QTest::newRow("Switch") << QByteArray("Switch { text: 'Switch me' }");
+    QTest::newRow("TextArea") << QByteArray("TextArea { width: 50}");
+    QTest::newRow("TextField") << QByteArray("TextField { width: 50}");
+
+    QTest::newRow("CheckBox") << QByteArray("CheckBox { text: 'checkBox' }");
+    QTest::newRow("DelayButton") << QByteArray("DelayButton { text: 'Are you really sure?' }");
+    QTest::newRow("RadioButton") << QByteArray("RadioButton { text: 'RadioButton' }");
+    QTest::newRow("TabButton") << QByteArray("TabButton { text: 'Home' }");
+}
+
+void tst_QQuickAccessible::controlsThatShouldSendObjectShow()
+{
+    QFETCH(QByteArray, qmlSnippet);
+
+    auto clearEvents = qScopeGuard([]{ QTestAccessibility::clearEvents(); });
+    QQmlApplicationEngine engine;
+    engine.loadData(QByteArray(R"(import QtQuick
+import QtQuick.Controls
+Window { visible: true
+)") + qmlSnippet + "}",
+                    QUrl());
+
+    QVERIFY(engine.rootObjects().count() > 0);
+    QQuickWindow *window = qobject_cast<QQuickWindow*>(engine.rootObjects().first());
+    QQuickItem *contentItem = window->contentItem();
+    QVERIFY(contentItem);
+    QQuickItem *rootItem = contentItem->childItems().first();
+    QVERIFY(rootItem);
+
+    QAccessibleEvent ev(rootItem, QAccessible::ObjectShow);
+    // Don't use QVERIFY_EVENT, because it gets very noisy when additional events are found
+    QVERIFY(QTestAccessibility::containsEvent(&ev));
+}
+
+void tst_QQuickAccessible::editableTextInterface_data()
+{
+    QTest::addColumn<QString>("objectName");
+    QTest::addColumn<bool>("editable");
+    QTest::addColumn<bool>("readOnly");
+
+    QTest::newRow("Label") << "label" << false << true;
+    QTest::newRow("TextInput") << "textInput" << true << false;
+    QTest::newRow("TextInput_readOnly") << "textInput" << true << true;
+    QTest::newRow("TextField") << "textField" << true << false;
+    QTest::newRow("TextField_readOnly") << "textField" << true << true;
+    QTest::newRow("TextEdit") << "textEdit" << true << false;
+    QTest::newRow("TextEdit_readOnly") << "textEdit" << true << true;
+    QTest::newRow("TextArea") << "textArea" << true << false;
+    QTest::newRow("TextArea_readOnly") << "textArea" << true << true;
+}
+
+void tst_QQuickAccessible::editableTextInterface()
+{
+    QFETCH(QString, objectName);
+    QFETCH(bool, editable);
+    QFETCH(bool, readOnly);
+
+    auto clearEvents = qScopeGuard([]{ QTestAccessibility::clearEvents(); });
+
+    auto window = std::make_unique<QQuickView>();
+    window->setSource(testFileUrl("textInterfaces.qml"));
+    window->show();
+    QVERIFY(QTest::qWaitForWindowActive(window.get()));
+
+    QQuickItem *item = window->findChild<QQuickItem *>(objectName);
+    QVERIFY(item);
+
+    QAccessibleInterface *itemIface = QAccessible::queryAccessibleInterface(item);
+    QVERIFY(itemIface);
+
+    QAccessibleQuickItem *accessibleQuickItem = static_cast<QAccessibleQuickItem *>(itemIface);
+    QVERIFY(accessibleQuickItem);
+
+    QAccessibleTextInterface *textIface = itemIface->textInterface();
+    QVERIFY(textIface);
+
+    // The read-only text items does not have editable text interface.
+    // But they could be editable, for example a read-only TextEdit.
+    if (editable)
+        QVERIFY2(item->setProperty("readOnly", readOnly), "Failed to set readOnly property");
+
+    QAccessibleEditableTextInterface *editableIface = itemIface->editableTextInterface();
+    if (!editable || readOnly) {
+        QVERIFY(editableIface == nullptr);
+    } else {
+        QVERIFY(editableIface != nullptr);
+
+        itemIface->setText(QAccessible::Value, QString("This is a text"));
+        QCOMPARE(accessibleQuickItem->text(QAccessible::Value), QString("This is a text"));
+
+        editableIface->replaceText(10, 14, "test");
+        QCOMPARE(accessibleQuickItem->text(QAccessible::Value), QString("This is a test"));
+
+        editableIface->insertText(9, " new");
+        QCOMPARE(accessibleQuickItem->text(QAccessible::Value), QString("This is a new test"));
+
+        editableIface->deleteText(9, 13);
+        QCOMPARE(accessibleQuickItem->text(QAccessible::Value), QString("This is a test"));
+    }
+}
+
 QTEST_MAIN(tst_QQuickAccessible)
 
 #include "tst_qquickaccessible.moc"
