@@ -235,6 +235,7 @@ private slots:
     void transformChanged();
 
     void grabImage();
+    void invisibleItemCursorShape();
 
 private:
 
@@ -973,12 +974,12 @@ void tst_qquickitem::activeFocusChangedOrder()
         QTRY_VERIFY(scope1.hasActiveFocus());
 
         int counter = 0;
-        connect(&scope1, &QQuickItem::activeFocusChanged, [&counter, &scope1](bool focus) {
+        connect(&scope1, &QQuickItem::activeFocusChanged, this, [&counter, &scope1](bool focus) {
             QCOMPARE(scope1.childItems().front()->hasActiveFocus(), focus);
             QCOMPARE(counter, 0);
             counter++;
         });
-        connect(&scope2, &QQuickItem::activeFocusChanged, [&counter, &scope2](bool focus) {
+        connect(&scope2, &QQuickItem::activeFocusChanged, this, [&counter, &scope2](bool focus) {
             QCOMPARE(scope2.childItems().front()->hasActiveFocus(), focus);
             QCOMPARE(counter, 1);
             counter++;
@@ -1012,12 +1013,12 @@ void tst_qquickitem::activeFocusChangedOrder()
         QTRY_VERIFY(item1.hasActiveFocus());
 
         int counter = 0;
-        connect(&item1, &QQuickItem::activeFocusChanged, [&counter](bool focus) {
+        connect(&item1, &QQuickItem::activeFocusChanged, this, [&counter](bool focus) {
             QVERIFY(!focus);
             QCOMPARE(counter, 0);
             counter++;
         });
-        connect(&item2, &QQuickItem::activeFocusChanged, [&counter](bool focus) {
+        connect(&item2, &QQuickItem::activeFocusChanged, this, [&counter](bool focus) {
             QVERIFY(focus);
             QCOMPARE(counter, 1);
             counter++;
@@ -1521,7 +1522,7 @@ struct PolishItemSpan {
  * For instance, two consecutive spans {99,0} and {1,2000} } instructs to
  * construct 99 items with no repolish, and 1 item with 2000 repolishes (in that sibling order)
  */
-typedef QVector<PolishItemSpan> PolishItemSpans;
+typedef QList<PolishItemSpan> PolishItemSpans;
 
 Q_DECLARE_METATYPE(PolishItemSpan)
 Q_DECLARE_METATYPE(PolishItemSpans)
@@ -2449,7 +2450,7 @@ void tst_qquickitem::objectCastInDestructor()
     QQuickItem *item = view.findChild<QQuickItem *>("testRectangle");
     QVERIFY(item);
     bool destroyed = false;
-    connect(item, &QObject::destroyed, [&]{
+    connect(item, &QObject::destroyed, this, [&destroyed, item] {
         destroyed = true;
         QCOMPARE(qobject_cast<QQuickItem *>(item), nullptr);
         QCOMPARE(qobject_cast<QQuickRectangle *>(item), nullptr);
@@ -2696,6 +2697,17 @@ void tst_qquickitem::grabImage()
     QVERIFY(result);
 }
 
+void tst_qquickitem::invisibleItemCursorShape()
+{
+    QQuickView window;
+    QVERIFY(QQuickTest::showView(window, testFileUrl("invisibleItemCursorShape.qml")));
+    QQuickItem *root = qobject_cast<QQuickItem *>(window.rootObject());
+    QVERIFY(root);
+
+    QTest::mouseMove(&window, QPoint(100, 100));
+    QCOMPARE(window.cursor().shape(), Qt::ArrowCursor);
+}
+
 QTEST_MAIN(tst_qquickitem)
 
 #include "tst_qquickitem.moc"
