@@ -1,13 +1,17 @@
 // Copyright (C) 2021 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
-#include <QtTest/QtTest>
+#include "delegatemodelkinds.h"
+
+#include <QtTest/QTest>
+#include <QtQmlModels/private/qqmlobjectmodel_p.h>
 #include <QtQuick/qquickview.h>
 #include <QtQuick/private/qquickitemview_p_p.h>
 #include <QtQuick/private/qquicklistview_p.h>
 #include <QtQuickTest/QtQuickTest>
 #include <QStringListModel>
 #include <QQmlApplicationEngine>
+#include <QtQml/QQmlComponent>
 
 #include <QtQuickTestUtils/private/viewtestutils_p.h>
 #include <QtQuickTestUtils/private/visualtestutils_p.h>
@@ -70,11 +74,21 @@ private slots:
     void bindingDirectlyOnPositionInHeaderAndFooterDelegates();
 
     void clearObjectListModel();
+    void gadgetModelSections();
     void visibleBoundToCountGreaterThanZero();
 
+    void setDelegateAfterModel();
+
+    void delegateModelAccess_data();
+    void delegateModelAccess();
+    void removeAndDestroyObjectModelItem_data();
+    void removeAndDestroyObjectModelItem();
+
+    void fastMouseWheel();
+
 private:
     void flickWithTouch(QQuickWindow *window, const QPoint &from, const QPoint &to);
-    QScopedPointer<QPointingDevice> touchDevice = QScopedPointer<QPointingDevice>(QTest::createTouchDevice());
+    std::unique_ptr<QPointingDevice> touchscreen{QTest::createTouchDevice()};
 };
 
 tst_QQuickListView2::tst_QQuickListView2()
@@ -462,6 +476,7 @@ void tst_QQuickListView2::tapDelegateDuringFlicking() // QTBUG-103832
     flickWithTouch(&window, {100, 400}, {100, 100});
     QTRY_VERIFY(listView->contentY() > 501); // let it flick some distance
     QVERIFY(listView->isFlicking()); // we want to test the case when it's still moving while we tap
+    QVERIFY(listView->isMoving());
     // @y = 400 we pressed the 4th delegate; started flicking, and the press was canceled
     QCOMPARE(listView->property("pressedDelegates").toList().first(), 4);
     // At first glance one would expect MouseArea and Button would be consistent about this;
@@ -476,10 +491,20 @@ void tst_QQuickListView2::tapDelegateDuringFlicking() // QTBUG-103832
     QCOMPARE(listView->property("releasedDelegates").toList().size(), 0);
 
     // press a delegate during flicking (at y > 501 + 100, so likely delegate 6)
-    QTest::touchEvent(&window, touchDevice.data()).press(0, {100, 100});
+    QTest::touchEvent(&window, touchscreen.get()).press(0, {100, 100});
     QQuickTouchUtils::flush(&window);
-    QTest::touchEvent(&window, touchDevice.data()).release(0, {100, 100});
+    // The press will stop listView from flicking, but it will still be "moving", which
+    // means that the user is still interacting with it
+    QVERIFY(!listView->isFlicking());
+    QVERIFY(!listView->isDragging());
+    QVERIFY(listView->isMoving());
+    QTest::touchEvent(&window, touchscreen.get()).release(0, {100, 100});
     QQuickTouchUtils::flush(&window);
+    // Releasing while "flicking" is false will stop the flicking
+    // session, and set "moving" to false
+    QVERIFY(!listView->isMoving());
+    QVERIFY(!listView->isFlicking());
+    QVERIFY(!listView->isDragging());
 
     const QVariantList pressedDelegates = listView->property("pressedDelegates").toList();
     const QVariantList releasedDelegates = listView->property("releasedDelegates").toList();
@@ -491,11 +516,13 @@ void tst_QQuickListView2::tapDelegateDuringFlicking() // QTBUG-103832
     qCDebug(lcTests) << "tapped" << tappedDelegates;
     qCDebug(lcTests) << "canceled" << canceledDelegates;
 
-    // which delegate received the second press, during flicking?
+    // Since the flickable was already moving (that is, the user was interacting with
+    // it), the second tap was only used to stop the flicking. Hence, no delegates
+    // received any pointer events.
     const int lastPressed = pressedDelegates.last().toInt();
-    QVERIFY(lastPressed > 5);
-    QCOMPARE(releasedDelegates.last(), lastPressed);
-    QCOMPARE(tappedDelegates.last(), lastPressed);
+    QCOMPARE(lastPressed, 4); // Nothing changed since the beginning
+    QVERIFY(releasedDelegates.isEmpty());
+    QVERIFY(tappedDelegates.isEmpty());
     QCOMPARE(canceledDelegates.size(), expectCanceled ? 1 : 0); // only the first press was canceled, not the second
 }
 
@@ -541,38 +568,38 @@ void tst_QQuickListView2::flickDuringFlicking() // QTBUG-103832
 
     // flick again during flicking, and make sure that it doesn't jump back to the first delegate,
     // but flicks incrementally further from the position at that time
-    QTest::touchEvent(&window, touchDevice.data()).press(0, {100, 400});
+    QTest::touchEvent(&window, touchscreen.get()).press(0, {100, 400});
     QQuickTouchUtils::flush(&window);
     qCDebug(lcTests) << "second press: contentY" << posBeforeSecondFlick << "->" << listView->contentY();
     qCDebug(lcTests) << "pressed delegates" << listView->property("pressedDelegates").toList();
     QVERIFY(listView->contentY() >= posBeforeSecondFlick);
 
     QTest::qWait(20);
-    QTest::touchEvent(&window, touchDevice.data()).move(0, {100, 300});
+    QTest::touchEvent(&window, touchscreen.get()).move(0, {100, 300});
     QQuickTouchUtils::flush(&window);
     qCDebug(lcTests) << "first move after second press: contentY" << posBeforeSecondFlick << "->" << listView->contentY();
     QVERIFY(listView->contentY() >= posBeforeSecondFlick);
 
     QTest::qWait(20);
-    QTest::touchEvent(&window, touchDevice.data()).move(0, {100, 200});
+    QTest::touchEvent(&window, touchscreen.get()).move(0, {100, 200});
     QQuickTouchUtils::flush(&window);
     qCDebug(lcTests) << "second move after second press: contentY" << posBeforeSecondFlick << "->" << listView->contentY();
     QVERIFY(listView->contentY() >= posBeforeSecondFlick + 100);
 
-    QTest::touchEvent(&window, touchDevice.data()).release(0, {100, 100});
+    QTest::touchEvent(&window, touchscreen.get()).release(0, {100, 100});
 }
 
 void tst_QQuickListView2::flickWithTouch(QQuickWindow *window, const QPoint &from, const QPoint &to)
 {
-    QTest::touchEvent(window, touchDevice.data()).press(0, from, window);
+    QTest::touchEvent(window, touchscreen.get()).press(0, from, window);
     QQuickTouchUtils::flush(window);
 
     QPoint diff = to - from;
     for (int i = 1; i <= 8; ++i) {
-        QTest::touchEvent(window, touchDevice.data()).move(0, from + i * diff / 8, window);
+        QTest::touchEvent(window, touchscreen.get()).move(0, from + i * diff / 8, window);
         QQuickTouchUtils::flush(window);
     }
-    QTest::touchEvent(window, touchDevice.data()).release(0, to, window);
+    QTest::touchEvent(window, touchscreen.get()).release(0, to, window);
     QQuickTouchUtils::flush(window);
 }
 
@@ -1023,6 +1050,7 @@ void tst_QQuickListView2::isCurrentItem_NoRegressionWithDelegateModelGroups()
     // Press left arrow key -> Item 1 should become current, Item 3 should not
     // be current anymore. After a previous fix of QTBUG-86744 it was working
     // incorrectly - see QTBUG-98315
+    QVERIFY(QTest::qWaitForWindowActive(window.get()));
     QTest::keyPress(window.get(), Qt::Key_Left);
 
     QTRY_COMPARE(item0->property("isCurrent").toBool(), true);
@@ -1315,6 +1343,17 @@ void tst_QQuickListView2::clearObjectListModel()
     QVERIFY(!list.itemAtIndex(0));
 }
 
+void tst_QQuickListView2::gadgetModelSections()
+{
+    QQuickView window;
+    QVERIFY(QQuickTest::showView(window, testFileUrl("gadgetlist.qml")));
+    QQuickListView *listview = qobject_cast<QQuickListView*>(window.rootObject());
+    QTRY_VERIFY(listview);
+    QVERIFY(QQuickTest::qWaitForPolish(listview));
+    QCOMPARE(listview->count(), 4);
+    QCOMPARE(listview->currentSection(), "small");
+}
+
 void tst_QQuickListView2::visibleBoundToCountGreaterThanZero()
 {
     QQuickView window;
@@ -1332,6 +1371,217 @@ void tst_QQuickListView2::visibleBoundToCountGreaterThanZero()
     QVERIFY(listView->isVisible());
 }
 
+void tst_QQuickListView2::setDelegateAfterModel()
+{
+    QQmlEngine engine;
+    const QUrl url = testFileUrl("setDelegateAfterModel.qml");
+    QQmlComponent component(&engine, url);
+    QVERIFY2(component.isReady(), qPrintable(component.errorString()));
+
+    QScopedPointer<QObject> object(component.create());
+    QVERIFY(!object.isNull());
+
+    // If the model was lost by setting the delegate, the count would be 0.
+    QCOMPARE(object->property("count").toInt(), 3);
+
+    QTest::ignoreMessage(
+            QtWarningMsg,
+            qPrintable(url.toString() + ":17:5: QML ListView: Cannot retain explicitly set "
+                                        "delegate on non-DelegateModel"));
+    object->setProperty("useObjectModel", QVariant::fromValue<bool>(true));
+    QCOMPARE(object->property("count").toInt(), 2);
+
+    // The old model must not mess with the view anymore.
+    QTest::failOnWarning(qPrintable(url.toString() + ":17:5: QML ListView: Explicitly set delegate "
+                                                     "is externally overridden"));
+    QMetaObject::invokeMethod(object.data(), "plantDelegate");
+    QCOMPARE(object->property("count").toInt(), 4);
+}
+
+template<typename Enum>
+const char *enumKey(Enum value) {
+    const QMetaObject *mo = qt_getEnumMetaObject(value);
+    const QMetaEnum metaEnum = mo->enumerator(mo->indexOfEnumerator(qt_getEnumName(value)));
+    return metaEnum.valueToKey(value);
+}
+
+void tst_QQuickListView2::delegateModelAccess_data()
+{
+    QTest::addColumn<QQmlDelegateModel::DelegateModelAccess>("access");
+    QTest::addColumn<Model::Kind>("modelKind");
+    QTest::addColumn<Delegate::Kind>("delegateKind");
+
+    using Access = QQmlDelegateModel::DelegateModelAccess;
+    for (auto access : { Access::Qt5ReadWrite, Access::ReadOnly, Access::ReadWrite }) {
+        for (auto model : { Model::Singular, Model::List, Model::Array, Model::Object }) {
+            for (auto delegate : { Delegate::Untyped, Delegate::Typed }) {
+                QTest::addRow("%s-%s-%s", enumKey(access), enumKey(model), enumKey(delegate))
+                    << access << model << delegate;
+            }
+        }
+    }
+}
+
+void tst_QQuickListView2::delegateModelAccess()
+{
+    QFETCH(QQmlDelegateModel::DelegateModelAccess, access);
+    QFETCH(Model::Kind, modelKind);
+    QFETCH(Delegate::Kind, delegateKind);
+
+    QQmlEngine engine;
+    const QUrl url = testFileUrl("delegateModelAccess.qml");
+    QQmlComponent c(&engine, url);
+    QVERIFY2(c.isReady(), qPrintable(c.errorString()));
+    QScopedPointer<QObject> object(c.create());
+
+    QQuickListView *listView = qobject_cast<QQuickListView *>(object.data());
+    QVERIFY(listView);
+
+    QSignalSpy modelChangedSpy(listView, &QQuickItemView::modelChanged);
+
+    if (delegateKind == Delegate::Untyped && modelKind == Model::Array)
+        QSKIP("Properties of objects in arrays are not exposed as context properties");
+
+    if (access == QQmlDelegateModel::ReadOnly) {
+        const QRegularExpression message(
+                url.toString() + ":[0-9]+: TypeError: Cannot assign to read-only property \"a\"");
+
+        QTest::ignoreMessage(QtWarningMsg, message);
+        if (delegateKind == Delegate::Untyped)
+            QTest::ignoreMessage(QtWarningMsg, message);
+    }
+    object->setProperty("delegateModelAccess", access);
+    object->setProperty("modelIndex", modelKind);
+    object->setProperty("delegateIndex", delegateKind);
+
+    listView->setCurrentIndex(0);
+    QObject *delegate = listView->itemAtIndex(0);
+    QVERIFY(delegate);
+
+    const bool modelWritable = access != QQmlDelegateModel::ReadOnly;
+    const bool immediateWritable = (delegateKind == Delegate::Untyped)
+            ? access != QQmlDelegateModel::ReadOnly
+            : access == QQmlDelegateModel::ReadWrite;
+
+    // Only the array is actually updated itself. The other models are pointers
+    const bool writeShouldSignal = modelKind == Model::Kind::Array;
+
+    double expected = 11;
+
+    // Initial setting of the model, signals one update
+    int expectedModelUpdates = 1;
+    QCOMPARE(modelChangedSpy.count(), expectedModelUpdates);
+
+    QCOMPARE(delegate->property("immediateX").toDouble(), expected);
+    QCOMPARE(delegate->property("modelX").toDouble(), expected);
+
+    if (modelWritable) {
+        expected = 3;
+        if (writeShouldSignal)
+            ++expectedModelUpdates;
+    }
+
+    QMetaObject::invokeMethod(delegate, "writeThroughModel");
+    QCOMPARE(delegate->property("immediateX").toDouble(), expected);
+    QCOMPARE(delegate->property("modelX").toDouble(), expected);
+    QCOMPARE(modelChangedSpy.count(), expectedModelUpdates);
+
+    if (immediateWritable) {
+        expected = 1;
+        if (writeShouldSignal)
+            ++expectedModelUpdates;
+    }
+
+    QMetaObject::invokeMethod(delegate, "writeImmediate");
+
+    // Writes to required properties always succeed, but might not be propagated to the model
+    QCOMPARE(delegate->property("immediateX").toDouble(),
+             delegateKind == Delegate::Untyped ? expected : 1);
+
+    QCOMPARE(delegate->property("modelX").toDouble(), expected);
+    QCOMPARE(modelChangedSpy.count(), expectedModelUpdates);
+}
+
+enum RemovalPolicy {
+    RemoveAndDestroy,
+    OnlyDestroy
+};
+
+void tst_QQuickListView2::removeAndDestroyObjectModelItem_data()
+{
+    QTest::addColumn<QString>("qmlFilePath");
+    QTest::addColumn<RemovalPolicy>("removalPolicy");
+
+    QTest::addRow("remove and destroy, no transitions")
+        << "removeAndDestroyObjectModelItem.qml" << RemoveAndDestroy;
+    QTest::addRow("destroy, no transitions")
+        << "removeAndDestroyObjectModelItem.qml" << OnlyDestroy;
+    QTest::addRow("remove and destroy, transitions")
+        << "removeAndDestroyObjectModelItemWithTransitions.qml" << RemoveAndDestroy;
+    QTest::addRow("destroy, transitions")
+        << "removeAndDestroyObjectModelItemWithTransitions.qml" << OnlyDestroy;
+}
+
+// QTBUG-46798, QTBUG-133256
+void tst_QQuickListView2::removeAndDestroyObjectModelItem()
+{
+    QFETCH(QString, qmlFilePath);
+    QFETCH(RemovalPolicy, removalPolicy);
+
+    QQuickView window;
+    QVERIFY(QQuickTest::showView(window, testFileUrl(qmlFilePath)));
+
+    QQuickListView *listView = qobject_cast<QQuickListView*>(window.rootObject());
+    QVERIFY(listView);
+
+    auto *objectModel = listView->model().value<QQmlObjectModel *>();
+    QVERIFY(objectModel);
+    QPointer<QObject> firstItem = objectModel->get(0);
+    QVERIFY(firstItem);
+    // Shouldn't crash.
+    if (removalPolicy == RemoveAndDestroy)
+        objectModel->remove(0);
+    firstItem->deleteLater();
+    QTRY_VERIFY(!firstItem);
+
+    // Now try moving the view. It also shouldn't crash.
+    if (removalPolicy == RemoveAndDestroy) {
+        if (QQuickTest::qIsPolishScheduled(listView))
+            QVERIFY(QQuickTest::qWaitForPolish(listView));
+    }
+    listView->positionViewAtEnd();
+}
+
+void tst_QQuickListView2::fastMouseWheel()
+{
+    QScopedPointer<QQuickView> window(createView());
+    QTRY_VERIFY(window);
+    window->setSource(testFileUrl("qtbug134502.qml"));
+    window->resize(640, 480);
+    window->show();
+    QQuickViewTestUtils::moveMouseAway(window.data());
+    QVERIFY(QTest::qWaitForWindowExposed(window.data()));
+    auto sendWheelEvent = [](QQuickView *window, const QPoint &angleDelta, quint64 timestamp) {
+        QPoint pos(100, 100);
+        QWheelEvent event(pos, window->mapToGlobal(pos), QPoint(), angleDelta, Qt::NoButton,
+                          Qt::NoModifier, Qt::NoScrollPhase, false);
+        event.setAccepted(false);
+        event.setTimestamp(timestamp);
+        QGuiApplication::sendEvent(window, &event);
+    };
+
+    QQuickListView *listview = findItem<QQuickListView>(window->rootObject(), "objects");
+    QTRY_VERIFY(listview != nullptr);
+
+    QGuiApplication::styleHints()->setWheelScrollLines(60);
+    sendWheelEvent(window.data(), QPoint(0, -120), 100);
+    QTRY_VERIFY(listview->isMoving() == false);
+    sendWheelEvent(window.data(), QPoint(0, -120), 132);
+    QTRY_VERIFY(listview->isMoving() == false);
+    sendWheelEvent(window.data(), QPoint(0, -240), 194);
+    QTRY_VERIFY(listview->isMoving() == false);
+}
+
 QTEST_MAIN(tst_QQuickListView2)
 
 #include "tst_qquicklistview2.moc"
