@@ -490,7 +490,7 @@ void tst_QQuickLoader::networkComponent()
                         "Item {\n"
                         " Component { id: comp; NW.Rect120x60 {} }\n"
                         " Loader { sourceComponent: comp } }";
-    component.setData(qml.toUtf8(), dataDirectory());
+    component.setData(qml.toUtf8(), dataDirectoryUrl());
     // The component may be loaded synchronously or asynchronously, so we cannot test for
     // status == Loading here. Also, it makes no sense to instruct the server to send here
     // because in the synchronous case we're already done loading.
@@ -734,13 +734,13 @@ void tst_QQuickLoader::initialPropertyValues_data()
 void tst_QQuickLoader::initialPropertyValues()
 {
     QFETCH(QUrl, qmlFile);
-    QFETCH(QStringList, expectedWarnings);
+    QFETCH(const QStringList, expectedWarnings);
     QFETCH(QStringList, propertyNames);
     QFETCH(QVariantList, propertyValues);
 
     ThreadedTestHTTPServer server(dataDirectory());
 
-    for (const QString &warning : std::as_const(expectedWarnings))
+    for (const QString &warning : expectedWarnings)
         QTest::ignoreMessage(QtWarningMsg, QRegularExpression(warning.toLatin1().constData()));
 
     QQmlEngine engine;
@@ -799,9 +799,9 @@ void tst_QQuickLoader::initialPropertyValuesError_data()
 void tst_QQuickLoader::initialPropertyValuesError()
 {
     QFETCH(QUrl, qmlFile);
-    QFETCH(QStringList, expectedWarnings);
+    QFETCH(const QStringList, expectedWarnings);
 
-    for (const QString &warning : std::as_const(expectedWarnings))
+    for (const QString &warning : expectedWarnings)
         QTest::ignoreMessage(QtWarningMsg, warning.toUtf8().constData());
 
     QQmlEngine engine;
@@ -943,7 +943,7 @@ void tst_QQuickLoader::asynchronous_data()
 void tst_QQuickLoader::asynchronous()
 {
     QFETCH(QUrl, qmlFile);
-    QFETCH(QStringList, expectedWarnings);
+    QFETCH(const QStringList, expectedWarnings);
 
     QQmlEngine engine;
     QScopedPointer<PeriodicIncubationController> controller(new PeriodicIncubationController);
@@ -958,7 +958,7 @@ void tst_QQuickLoader::asynchronous()
     QQuickLoader *loader = root->findChild<QQuickLoader*>("loader");
     QVERIFY(loader);
 
-    for (const QString &warning : std::as_const(expectedWarnings))
+    for (const QString &warning : expectedWarnings)
         QTest::ignoreMessage(QtWarningMsg, warning.toUtf8().constData());
 
     QVERIFY(!loader->item());
@@ -1249,9 +1249,9 @@ void tst_QQuickLoader::transientWindow() // QTBUG-52944
     QElapsedTimer timer;
     qint64 viewVisibleTime = -1;
     qint64 loadedWindowVisibleTime = -1;
-    connect(&view, &QWindow::visibleChanged,
+    connect(&view, &QWindow::visibleChanged, this,
             [&viewVisibleTime, &timer]() { viewVisibleTime = timer.elapsed(); } );
-    connect(loadedWindow, &QQuickWindowQmlImpl::visibilityChanged,
+    connect(loadedWindow, &QQuickWindowQmlImpl::visibilityChanged, this,
             [&loadedWindowVisibleTime, &timer]() { loadedWindowVisibleTime = timer.elapsed(); } );
     timer.start();
     view.show();
@@ -1290,9 +1290,9 @@ void tst_QQuickLoader::nestedTransientWindow() // QTBUG-52944
     QElapsedTimer timer;
     qint64 viewVisibleTime = -1;
     qint64 loadedWindowVisibleTime = -1;
-    connect(&view, &QWindow::visibleChanged,
+    connect(&view, &QWindow::visibleChanged, this,
             [&viewVisibleTime, &timer]() { viewVisibleTime = timer.elapsed(); } );
-    connect(loadedWindow, &QQuickWindowQmlImpl::visibilityChanged,
+    connect(loadedWindow, &QQuickWindowQmlImpl::visibilityChanged, this,
             [&loadedWindowVisibleTime, &timer]() { loadedWindowVisibleTime = timer.elapsed(); } );
     timer.start();
     view.show();
