@@ -1,8 +1,9 @@
 // Copyright (C) 2020 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 
-#include <QtTest/QtTest>
+#include <QtTest/QTest>
+#include <QtTest/QSignalSpy>
 #include <QDebug>
 
 #include <QtGui/qstylehints.h>
@@ -244,6 +245,8 @@ private slots:
 
     void strayTouchDoesntAutograb();
 
+    void noDoubleClickWithInterveningTouch();
+
 protected:
     bool eventFilter(QObject *, QEvent *event) override
     {
@@ -259,7 +262,7 @@ protected:
 
 private:
     QQuickView *createView();
-    QPointingDevice *device = QTest::createTouchDevice();
+    std::unique_ptr<QPointingDevice> touchscreen{QTest::createTouchDevice()};
     QList<Event> filteredEventList;
     GrabMonitor grabMonitor;
 };
@@ -274,7 +277,7 @@ void tst_TouchMouse::initTestCase()
 {
     QQmlDataTest::initTestCase();
     qmlRegisterType<EventItem>("Qt.test", 1, 0, "EventItem");
-    connect(device, &QPointingDevice::grabChanged, &grabMonitor, &GrabMonitor::onGrabChanged);
+    connect(touchscreen.get(), &QPointingDevice::grabChanged, &grabMonitor, &GrabMonitor::onGrabChanged);
 }
 
 void tst_TouchMouse::simpleTouchEvent_data()
@@ -294,24 +297,24 @@ void tst_TouchMouse::simpleTouchEvent()
 
     EventItem *eventItem1 = window.rootObject()->findChild<EventItem*>("eventItem1");
     QVERIFY(eventItem1);
-    auto devPriv = QPointingDevicePrivate::get(device);
+    auto devPriv = QPointingDevicePrivate::get(touchscreen.get());
 
     // Do not accept touch or mouse
     eventItem1->setAcceptTouchEvents(false);
     QPoint p1;
     p1 = QPoint(20, 20);
-    QTest::touchEvent(&window, device).press(0, p1, &window);
+    QTest::touchEvent(&window, touchscreen.get()).press(0, p1, &window);
     QQuickTouchUtils::flush(&window);
     // Get a synth-mouse event if allowed
     QCOMPARE(eventItem1->eventList.size(), synthMouse ? 1 : 0);
     if (synthMouse)
         QCOMPARE(eventItem1->eventList.at(0).type, QEvent::MouseButtonPress);
     p1 += QPoint(10, 0);
-    QTest::touchEvent(&window, device).move(0, p1, &window);
+    QTest::touchEvent(&window, touchscreen.get()).move(0, p1, &window);
     QQuickTouchUtils::flush(&window);
     // Not accepted, no updates
     QCOMPARE(eventItem1->eventList.size(), synthMouse ? 1 : 0);
-    QTest::touchEvent(&window, device).release(0, p1, &window);
+    QTest::touchEvent(&window, touchscreen.get()).release(0, p1, &window);
     QQuickTouchUtils::flush(&window);
     QCOMPARE(eventItem1->eventList.size(), synthMouse ? 1 : 0);
     eventItem1->eventList.clear();
@@ -320,14 +323,14 @@ void tst_TouchMouse::simpleTouchEvent()
     eventItem1->setAcceptTouchEvents(true);
     eventItem1->acceptTouch = true;
     p1 = QPoint(20, 20);
-    QTest::touchEvent(&window, device).press(0, p1, &window);
+    QTest::touchEvent(&window, touchscreen.get()).press(0, p1, &window);
     QQuickTouchUtils::flush(&window);
     QCOMPARE(eventItem1->eventList.size(), 1);
     p1 += QPoint(10, 0);
-    QTest::touchEvent(&window, device).move(0, p1, &window);
+    QTest::touchEvent(&window, touchscreen.get()).move(0, p1, &window);
     QQuickTouchUtils::flush(&window);
     QCOMPARE(eventItem1->eventList.size(), 2);
-    QTest::touchEvent(&window, device).release(0, p1, &window);
+    QTest::touchEvent(&window, touchscreen.get()).release(0, p1, &window);
     QQuickTouchUtils::flush(&window);
     QCOMPARE(eventItem1->eventList.size(), 3);
     eventItem1->eventList.clear();
@@ -341,7 +344,7 @@ void tst_TouchMouse::simpleTouchEvent()
     eventItem1->setAcceptTouchEvents(false);
     eventItem1->setAcceptedMouseButtons(Qt::LeftButton);
     p1 = QPoint(20, 20);
-    QTest::touchEvent(&window, device).press(0, p1, &window);
+    QTest::touchEvent(&window, touchscreen.get()).press(0, p1, &window);
     QQuickTouchUtils::flush(&window);
     QCOMPARE(eventItem1->eventList.size(), synthMouse ? 1 : 0);
     if (synthMouse)
@@ -356,14 +359,14 @@ void tst_TouchMouse::simpleTouchEvent()
     }
 
     p1 += QPoint(10, 0);
-    QTest::touchEvent(&window, device).move(0, p1, &window);
+    QTest::touchEvent(&window, touchscreen.get()).move(0, p1, &window);
     QQuickTouchUtils::flush(&window);
     QCOMPARE(eventItem1->eventList.size(), synthMouse ? 2 : 0);
     if (synthMouse)
         QCOMPARE(eventItem1->eventList.at(1).type, QEvent::MouseMove);
     // else, if there was no synth-mouse and we didn't accept the touch,
     // TouchUpdate was not sent to eventItem1 either.
-    QTest::touchEvent(&window, device).release(0, p1, &window);
+    QTest::touchEvent(&window, touchscreen.get()).release(0, p1, &window);
     QQuickTouchUtils::flush(&window);
     QCOMPARE(eventItem1->eventList.size(), synthMouse ? 4 : 0);
     if (synthMouse) {
@@ -382,16 +385,16 @@ void tst_TouchMouse::simpleTouchEvent()
     eventItem1->acceptMouse = false;
     eventItem1->setAcceptedMouseButtons(Qt::LeftButton);
     p1 = QPoint(20, 20);
-    QTest::touchEvent(&window, device).press(0, p1, &window);
+    QTest::touchEvent(&window, touchscreen.get()).press(0, p1, &window);
     QQuickTouchUtils::flush(&window);
     QCOMPARE(eventItem1->eventList.size(), synthMouse ? 1 : 0);
     if (synthMouse)
         QCOMPARE(eventItem1->eventList.at(0).type, QEvent::MouseButtonPress);
     p1 += QPoint(10, 0);
-    QTest::touchEvent(&window, device).move(0, p1, &window);
+    QTest::touchEvent(&window, touchscreen.get()).move(0, p1, &window);
     QQuickTouchUtils::flush(&window);
     QCOMPARE(eventItem1->eventList.size(), synthMouse ? 1 : 0);
-    QTest::touchEvent(&window, device).release(0, p1, &window);
+    QTest::touchEvent(&window, touchscreen.get()).release(0, p1, &window);
     QQuickTouchUtils::flush(&window);
     QCOMPARE(eventItem1->eventList.size(), synthMouse ? 1 : 0);
     eventItem1->eventList.clear();
@@ -404,16 +407,16 @@ void tst_TouchMouse::simpleTouchEvent()
     eventItem1->setAcceptTouchEvents(true);
     eventItem1->setAcceptedMouseButtons(Qt::LeftButton);
     p1 = QPoint(20, 20);
-    QTest::touchEvent(&window, device).press(0, p1, &window);
+    QTest::touchEvent(&window, touchscreen.get()).press(0, p1, &window);
     QQuickTouchUtils::flush(&window);
     QCOMPARE(eventItem1->eventList.size(), 1);
     QCOMPARE(eventItem1->eventList.at(0).type, QEvent::TouchBegin);
     p1 += QPoint(10, 0);
-    QTest::touchEvent(&window, device).move(0, p1, &window);
+    QTest::touchEvent(&window, touchscreen.get()).move(0, p1, &window);
     QQuickTouchUtils::flush(&window);
     QCOMPARE(eventItem1->eventList.size(), 2);
     QCOMPARE(eventItem1->eventList.at(1).type, QEvent::TouchUpdate);
-    QTest::touchEvent(&window, device).release(0, p1, &window);
+    QTest::touchEvent(&window, touchscreen.get()).release(0, p1, &window);
     QQuickTouchUtils::flush(&window);
     QCOMPARE(eventItem1->eventList.size(), 3);
     QCOMPARE(eventItem1->eventList.at(2).type, QEvent::TouchEnd);
@@ -439,11 +442,11 @@ void tst_TouchMouse::testEventFilter()
 //    eventItem1->installEventFilter(filter);
 
 //    QPoint p1 = QPoint(20, 20);
-//    QTest::touchEvent(&window, device).press(0, p1, &window);
+//    QTest::touchEvent(&window, touchscreen.get()).press(0, p1, &window);
 //    // QEXPECT_FAIL("", "We do not implement event filters correctly", Abort);
 //    QCOMPARE(eventItem1->eventList.size(), 0);
 //    QCOMPARE(filter->eventList.size(), 1);
-//    QTest::touchEvent(&window, device).release(0, p1, &window);
+//    QTest::touchEvent(&window, touchscreen.get()).release(0, p1, &window);
 //    QCOMPARE(eventItem1->eventList.size(), 0);
 //    QCOMPARE(filter->eventList.size(), 2);
 
@@ -471,7 +474,7 @@ void tst_TouchMouse::mouse()
     // item 2 doesn't accept anything, thus it sees a touch pass by
     eventItem2->setAcceptTouchEvents(false);
     QPoint p1 = QPoint(30, 30);
-    QTest::touchEvent(&window, device).press(0, p1, &window);
+    QTest::touchEvent(&window, touchscreen.get()).press(0, p1, &window);
     QQuickTouchUtils::flush(&window);
 
     QCOMPARE(eventItem1->eventList.size(), 1);
@@ -499,7 +502,7 @@ void tst_TouchMouse::touchOverMouse()
 
     QCOMPARE(eventItem1->eventList.size(), 0);
     QPoint p1 = QPoint(20, 20);
-    QTest::touchEvent(&window, device).press(0, p1, &window);
+    QTest::touchEvent(&window, touchscreen.get()).press(0, p1, &window);
     QQuickTouchUtils::flush(&window);
     qCDebug(lcTests) << "expected delivered events: press(mouse)" << eventItem1->eventList;
     QCOMPARE(eventItem1->eventList.size(), 1);
@@ -507,12 +510,12 @@ void tst_TouchMouse::touchOverMouse()
     QCOMPARE(eventItem2->eventList.size(), 1);
     QCOMPARE(eventItem2->eventList.at(0).type, QEvent::TouchBegin);
     p1 += QPoint(10, 0);
-    QTest::touchEvent(&window, device).move(0, p1, &window);
+    QTest::touchEvent(&window, touchscreen.get()).move(0, p1, &window);
     QQuickTouchUtils::flush(&window);
     qCDebug(lcTests) << "expected delivered events: press(mouse) move(mouse)" << eventItem1->eventList;
     QCOMPARE(eventItem1->eventList.size(), 2);
     QCOMPARE(eventItem1->eventList.at(1).type, QEvent::MouseMove);
-    QTest::touchEvent(&window, device).release(0, p1, &window);
+    QTest::touchEvent(&window, touchscreen.get()).release(0, p1, &window);
     QQuickTouchUtils::flush(&window);
     qCDebug(lcTests) << "expected delivered events: press(mouse) move(mouse) release(mouse) ungrab(mouse)" << eventItem1->eventList;
     QCOMPARE(eventItem1->eventList.size(), 4);
@@ -541,16 +544,16 @@ void tst_TouchMouse::mouseOverTouch()
 
     QPoint p1 = QPoint(20, 20);
     QTest::qWait(qApp->styleHints()->mouseDoubleClickInterval() + 10);
-    QTest::touchEvent(&window, device).press(0, p1, &window);
+    QTest::touchEvent(&window, touchscreen.get()).press(0, p1, &window);
     QQuickTouchUtils::flush(&window);
     QCOMPARE(eventItem1->eventList.size(), 0);
     QCOMPARE(eventItem2->eventList.size(), 1);
     QCOMPARE(eventItem2->eventList.at(0).type, QEvent::MouseButtonPress);
 
     p1 += QPoint(10, 0);
-    QTest::touchEvent(&window, device).move(0, p1, &window);
+    QTest::touchEvent(&window, touchscreen.get()).move(0, p1, &window);
     QCOMPARE(eventItem2->eventList.size(), 1);
-    QTest::touchEvent(&window, device).release(0, p1, &window);
+    QTest::touchEvent(&window, touchscreen.get()).release(0, p1, &window);
     qCDebug(lcTests) << "expected delivered events: press(mouse) move(mouse) release(mouse) ungrab(mouse)" << eventItem2->eventList;
     QCOMPARE(eventItem2->eventList.size(), 4);
     eventItem2->eventList.clear();
@@ -589,11 +592,11 @@ void tst_TouchMouse::buttonOnFlickable()
     // mouse button
     QCOMPARE(eventItem1->eventList.size(), 0);
     QPoint p1 = QPoint(20, 130);
-    QTest::touchEvent(&window, device).press(0, p1, &window);
+    QTest::touchEvent(&window, touchscreen.get()).press(0, p1, &window);
     QQuickTouchUtils::flush(&window);
     QTRY_COMPARE(eventItem1->eventList.size(), 1);
     QCOMPARE(eventItem1->eventList.at(0).type, QEvent::MouseButtonPress);
-    QTest::touchEvent(&window, device).release(0, p1, &window);
+    QTest::touchEvent(&window, touchscreen.get()).release(0, p1, &window);
     QQuickTouchUtils::flush(&window);
     QCOMPARE(eventItem1->eventList.size(), 3);
     QCOMPARE(eventItem1->eventList.at(1).type, QEvent::MouseButtonRelease);
@@ -602,11 +605,11 @@ void tst_TouchMouse::buttonOnFlickable()
 
     // touch button
     p1 = QPoint(10, 310);
-    QTest::touchEvent(&window, device).press(0, p1, &window);
+    QTest::touchEvent(&window, touchscreen.get()).press(0, p1, &window);
     QQuickTouchUtils::flush(&window);
     QCOMPARE(eventItem2->eventList.size(), 1);
     QCOMPARE(eventItem2->eventList.at(0).type, QEvent::TouchBegin);
-    QTest::touchEvent(&window, device).release(0, p1, &window);
+    QTest::touchEvent(&window, touchscreen.get()).release(0, p1, &window);
     QQuickTouchUtils::flush(&window);
     QCOMPARE(eventItem2->eventList.size(), 2);
     QCOMPARE(eventItem2->eventList.at(1).type, QEvent::TouchEnd);
@@ -618,10 +621,10 @@ void tst_TouchMouse::buttonOnFlickable()
 
     // click above button, no events please
     p1 = QPoint(10, 90);
-    QTest::touchEvent(&window, device).press(0, p1, &window);
+    QTest::touchEvent(&window, touchscreen.get()).press(0, p1, &window);
     QQuickTouchUtils::flush(&window);
     QCOMPARE(eventItem1->eventList.size(), 0);
-    QTest::touchEvent(&window, device).release(0, p1, &window);
+    QTest::touchEvent(&window, touchscreen.get()).release(0, p1, &window);
     QQuickTouchUtils::flush(&window);
     QCOMPARE(eventItem1->eventList.size(), 0);
     eventItem1->eventList.clear();
@@ -632,14 +635,14 @@ void tst_TouchMouse::buttonOnFlickable()
     // check that flickable moves - mouse button
     QCOMPARE(eventItem1->eventList.size(), 0);
     p1 = QPoint(10, 110);
-    QTest::touchEvent(&window, device).press(0, p1, &window);
+    QTest::touchEvent(&window, touchscreen.get()).press(0, p1, &window);
     QQuickTouchUtils::flush(&window);
     QCOMPARE(eventItem1->eventList.size(), 1);
     QCOMPARE(eventItem1->eventList.at(0).type, QEvent::MouseButtonPress);
 
     QQuickWindowPrivate *windowPriv = QQuickWindowPrivate::get(&window);
     QVERIFY(windowPriv->deliveryAgentPrivate()->touchMouseId != -1);
-    auto devPriv = QPointingDevicePrivate::get(device);
+    auto devPriv = QPointingDevicePrivate::get(touchscreen.get());
     QCOMPARE(devPriv->pointById(0)->exclusiveGrabber, eventItem1);
     QCOMPARE(grabMonitor.exclusiveGrabber, eventItem1);
 
@@ -649,7 +652,7 @@ void tst_TouchMouse::buttonOnFlickable()
     int i = 0;
     for (; i < 10 && !flickable->isMovingVertically(); ++i) {
         p1 += QPoint(0, dragDelta);
-        QTest::touchEvent(&window, device).move(0, p1, &window);
+        QTest::touchEvent(&window, touchscreen.get()).move(0, p1, &window);
         QQuickTouchUtils::flush(&window);
     }
     QVERIFY(flickable->isMovingVertically());
@@ -662,7 +665,7 @@ void tst_TouchMouse::buttonOnFlickable()
     QVERIFY(windowPriv->deliveryAgentPrivate()->touchMouseId != -1);
     QCOMPARE(devPriv->pointById(0)->exclusiveGrabber, flickable);
 
-    QTest::touchEvent(&window, device).release(0, p1, &window);
+    QTest::touchEvent(&window, touchscreen.get()).release(0, p1, &window);
     QQuickTouchUtils::flush(&window);
 }
 
@@ -687,14 +690,14 @@ void tst_TouchMouse::touchButtonOnFlickable()
     // press via touch, then drag: check that flickable moves and that the button gets ungrabbed
     QCOMPARE(eventItem2->eventList.size(), 0);
     QPoint p1 = QPoint(10, 310);
-    QTest::touchEvent(&window, device).press(0, p1, &window);
+    QTest::touchEvent(&window, touchscreen.get()).press(0, p1, &window);
     QQuickTouchUtils::flush(&window);
     QCOMPARE(eventItem2->eventList.size(), 1);
     QCOMPARE(eventItem2->eventList.at(0).type, QEvent::TouchBegin);
 
     QQuickWindowPrivate *windowPriv = QQuickWindowPrivate::get(&window);
     QVERIFY(windowPriv->deliveryAgentPrivate()->touchMouseId == -1);
-    auto devPriv = QPointingDevicePrivate::get(device);
+    auto devPriv = QPointingDevicePrivate::get(touchscreen.get());
     QCOMPARE(devPriv->pointById(0)->exclusiveGrabber, eventItem2);
     QCOMPARE(grabMonitor.exclusiveGrabber, eventItem2);
 
@@ -704,11 +707,11 @@ void tst_TouchMouse::touchButtonOnFlickable()
     QPoint p3 = p2 + QPoint(0, dragDelta);
 
     QQuickTouchUtils::flush(&window);
-    QTest::touchEvent(&window, device).move(0, p1, &window);
+    QTest::touchEvent(&window, touchscreen.get()).move(0, p1, &window);
     QQuickTouchUtils::flush(&window);
-    QTest::touchEvent(&window, device).move(0, p2, &window);
+    QTest::touchEvent(&window, touchscreen.get()).move(0, p2, &window);
     QQuickTouchUtils::flush(&window);
-    QTest::touchEvent(&window, device).move(0, p3, &window);
+    QTest::touchEvent(&window, touchscreen.get()).move(0, p3, &window);
     QQuickTouchUtils::flush(&window);
 
     QTRY_COMPARE(eventItem2->touchUngrabCount, 1);
@@ -721,7 +724,7 @@ void tst_TouchMouse::touchButtonOnFlickable()
     QCOMPARE(devPriv->pointById(0)->exclusiveGrabber, flickable);
     QVERIFY(flickable->isMovingVertically());
 
-    QTest::touchEvent(&window, device).release(0, p3, &window);
+    QTest::touchEvent(&window, touchscreen.get()).release(0, p3, &window);
     QQuickTouchUtils::flush(&window);
 }
 
@@ -784,7 +787,7 @@ void tst_TouchMouse::buttonOnDelayedPressFlickable()
 
     // touch press
     QPoint p1 = QPoint(10, 110);
-    QTest::touchEvent(&window, device).press(0, p1, &window);
+    QTest::touchEvent(&window, touchscreen.get()).press(0, p1, &window);
     QQuickTouchUtils::flush(&window);
 
     if (scrollBeforeDelayIsOver || releaseBeforeDelayIsOver) {
@@ -803,7 +806,7 @@ void tst_TouchMouse::buttonOnDelayedPressFlickable()
         // move the touchpoint: try to flick
         for (int i = 0; i < 3; ++i) {
             p1 += QPoint(0, -threshold);
-            QTest::touchEvent(&window, device).move(0, p1, &window);
+            QTest::touchEvent(&window, touchscreen.get()).move(0, p1, &window);
             QQuickTouchUtils::flush(&window);
         }
         QTRY_VERIFY(flickable->isMovingVertically());
@@ -824,11 +827,11 @@ void tst_TouchMouse::buttonOnDelayedPressFlickable()
 
         // flickable should have the touchpoint grab: it no longer relies on synth-mouse
         QCOMPARE(grabMonitor.exclusiveGrabber, flickable);
-        auto devPriv = QPointingDevicePrivate::get(device);
+        auto devPriv = QPointingDevicePrivate::get(touchscreen.get());
         QCOMPARE(devPriv->pointById(0)->exclusiveGrabber, flickable);
     }
 
-    QTest::touchEvent(&window, device).release(0, p1, &window);
+    QTest::touchEvent(&window, touchscreen.get()).release(0, p1, &window);
     QQuickTouchUtils::flush(&window);
 
     if (releaseBeforeDelayIsOver) {
@@ -886,7 +889,7 @@ void tst_TouchMouse::buttonOnTouch()
     EventItem *eventItem4 = window.rootObject()->findChild<EventItem*>("eventItem4");
     QVERIFY(eventItem4);
 
-    QTest::QTouchEventSequence touchSeq = QTest::touchEvent(&window, device, false);
+    QTest::QTouchEventSequence touchSeq = QTest::touchEvent(&window, touchscreen.get(), false);
 
     // Test the common case of a mouse area on top of pinch
     eventItem1->setAcceptedMouseButtons(Qt::LeftButton);
@@ -1013,25 +1016,25 @@ void tst_TouchMouse::pinchOnFlickable()
     // flick the flickable with one touchpoint
     QCOMPARE(flickable->contentX(), 0);
     QPoint p = QPoint(100, 100);
-    QTest::touchEvent(&window, device).press(0, p, &window);
+    QTest::touchEvent(&window, touchscreen.get()).press(0, p, &window);
     QQuickTouchUtils::flush(&window);
     QCOMPARE(rect->position(), QPointF(200, 200));
     for (int i = 0; i < 4; ++i) {
         p -= QPoint(threshold, 0);
-        QTest::touchEvent(&window, device).move(0, p, &window);
+        QTest::touchEvent(&window, touchscreen.get()).move(0, p, &window);
         QQuickTouchUtils::flush(&window);
         if (!flickable->isAtXBeginning()) // currently happens when i == 3
             qCDebug(lcTests, "flicking after %d moves: %lf", i + 1, flickable->contentX());
     }
-    QTest::touchEvent(&window, device).release(0, p, &window);
+    QTest::touchEvent(&window, touchscreen.get()).release(0, p, &window);
     QTRY_COMPARE(flickable->isAtXBeginning(), false);
     // wait until flicking is done
-    QTRY_COMPARE(flickable->isFlicking(), false);
+    QTRY_COMPARE_WITH_TIMEOUT(flickable->isFlicking(), false, 2s);
 
     // pinch with two touchpoints
     QPoint p1 = QPoint(40, 20);
     QPoint p2 = QPoint(60, 20);
-    QTest::QTouchEventSequence pinchSequence = QTest::touchEvent(&window, device);
+    QTest::QTouchEventSequence pinchSequence = QTest::touchEvent(&window, touchscreen.get());
     QQuickTouchUtils::flush(&window);
     pinchSequence.press(0, p1, &window).commit();
     QQuickTouchUtils::flush(&window);
@@ -1072,22 +1075,22 @@ void tst_TouchMouse::flickableOnPinch()
     // flickable - single touch point
     QCOMPARE(flickable->contentX(), 0.0);
     QPoint p = QPoint(100, 100);
-    QTest::touchEvent(&window, device).press(0, p, &window);
+    QTest::touchEvent(&window, touchscreen.get()).press(0, p, &window);
     QQuickTouchUtils::flush(&window);
     QCOMPARE(rect->position(), QPointF(200.0, 200.0));
     p -= QPoint(10, 0);
-    QTest::touchEvent(&window, device).move(0, p, &window);
+    QTest::touchEvent(&window, touchscreen.get()).move(0, p, &window);
     QQuickTouchUtils::flush(&window);
     p -= QPoint(10, 0);
-    QTest::touchEvent(&window, device).move(0, p, &window);
+    QTest::touchEvent(&window, touchscreen.get()).move(0, p, &window);
     QQuickTouchUtils::flush(&window);
 
     QTest::qWait(1000);
 
     p -= QPoint(10, 0);
-    QTest::touchEvent(&window, device).move(0, p, &window);
+    QTest::touchEvent(&window, touchscreen.get()).move(0, p, &window);
     QQuickTouchUtils::flush(&window);
-    QTest::touchEvent(&window, device).release(0, p, &window);
+    QTest::touchEvent(&window, touchscreen.get()).release(0, p, &window);
     QQuickTouchUtils::flush(&window);
 
     QTest::qWait(1000);
@@ -1100,7 +1103,7 @@ void tst_TouchMouse::flickableOnPinch()
     // pinch
     QPoint p1 = QPoint(40, 20);
     QPoint p2 = QPoint(60, 20);
-    QTest::QTouchEventSequence pinchSequence = QTest::touchEvent(&window, device);
+    QTest::QTouchEventSequence pinchSequence = QTest::touchEvent(&window, touchscreen.get());
     pinchSequence.press(0, p1, &window).commit();
     QQuickTouchUtils::flush(&window);
     // In order for the stationary point to remember its previous position,
@@ -1147,19 +1150,19 @@ void tst_TouchMouse::mouseOnFlickableOnPinch()
     // flickable - single touch point
     QCOMPARE(flickable->contentX(), 0.0);
     QPoint p = QPoint(100, 100);
-    QTest::touchEvent(&window, device).press(0, p, &window);
+    QTest::touchEvent(&window, touchscreen.get()).press(0, p, &window);
     QQuickTouchUtils::flush(&window);
     QCOMPARE(rect->position(), QPointF(200.0, 200.0));
     p -= QPoint(10, 0);
-    QTest::touchEvent(&window, device).move(0, p, &window);
+    QTest::touchEvent(&window, touchscreen.get()).move(0, p, &window);
     QQuickTouchUtils::flush(&window);
     p -= QPoint(10, 0);
-    QTest::touchEvent(&window, device).move(0, p, &window);
+    QTest::touchEvent(&window, touchscreen.get()).move(0, p, &window);
     QQuickTouchUtils::flush(&window);
     p -= QPoint(10, 0);
-    QTest::touchEvent(&window, device).move(0, p, &window);
+    QTest::touchEvent(&window, touchscreen.get()).move(0, p, &window);
     QQuickTouchUtils::flush(&window);
-    QTest::touchEvent(&window, device).release(0, p, &window);
+    QTest::touchEvent(&window, touchscreen.get()).release(0, p, &window);
     QQuickTouchUtils::flush(&window);
 
     QVERIFY(flickable->isMovingHorizontally());
@@ -1171,7 +1174,7 @@ void tst_TouchMouse::mouseOnFlickableOnPinch()
     // pinch
     QPoint p1 = QPoint(40, 20);
     QPoint p2 = QPoint(60, 20);
-    QTest::QTouchEventSequence pinchSequence = QTest::touchEvent(&window, device);
+    QTest::QTouchEventSequence pinchSequence = QTest::touchEvent(&window, touchscreen.get());
     pinchSequence.press(0, p1, &window).commit();
     QQuickTouchUtils::flush(&window);
     // In order for the stationary point to remember its previous position,
@@ -1272,17 +1275,17 @@ void tst_TouchMouse::tapOnDismissiveTopMouseAreaClicksBottomOne()
 
     // tap the front mouse area (see qml file)
     QPoint p1(20, 20);
-    QTest::touchEvent(&window, device).press(0, p1, &window);
+    QTest::touchEvent(&window, touchscreen.get()).press(0, p1, &window);
     QQuickTouchUtils::flush(&window);
-    QTest::touchEvent(&window, device).release(0, p1, &window);
+    QTest::touchEvent(&window, touchscreen.get()).release(0, p1, &window);
     QQuickTouchUtils::flush(&window);
 
     QCOMPARE(bottomClickedSpy.size(), 1);
     QCOMPARE(bottomDoubleClickedSpy.size(), 0);
 
-    QTest::touchEvent(&window, device).press(0, p1, &window);
+    QTest::touchEvent(&window, touchscreen.get()).press(0, p1, &window);
     QQuickTouchUtils::flush(&window);
-    QTest::touchEvent(&window, device).release(0, p1, &window);
+    QTest::touchEvent(&window, touchscreen.get()).release(0, p1, &window);
     QQuickTouchUtils::flush(&window);
 
     QCOMPARE(bottomClickedSpy.size(), 1);
@@ -1303,7 +1306,7 @@ void tst_TouchMouse::touchGrabCausesMouseUngrab()
     QVERIFY(leftItem);
 
     EventItem *rightItem = window.rootObject()->findChild<EventItem*>("rightItem");
-    QVERIFY(leftItem);
+    QVERIFY(rightItem);
 
     // Send a touch to the leftItem. But leftItem accepts only mouse events, thus
     // a mouse event will be synthesized out of this touch and will get accepted by
@@ -1312,7 +1315,7 @@ void tst_TouchMouse::touchGrabCausesMouseUngrab()
     leftItem->setAcceptedMouseButtons(Qt::LeftButton);
     QPoint p1;
     p1 = QPoint(leftItem->width() / 2, leftItem->height() / 2);
-    QTest::touchEvent(&window, device).press(0, p1, &window);
+    QTest::touchEvent(&window, touchscreen.get()).press(0, p1, &window);
     QQuickTouchUtils::flush(&window);
     qCDebug(lcTests) << "expected delivered events: press(mouse)" << leftItem->eventList;
     QCOMPARE(leftItem->eventList.size(), 1);
@@ -1321,7 +1324,7 @@ void tst_TouchMouse::touchGrabCausesMouseUngrab()
     leftItem->eventList.clear();
 
     rightItem->acceptTouch = true;
-    auto devPriv = QPointingDevicePrivate::get(device);
+    auto devPriv = QPointingDevicePrivate::get(touchscreen.get());
     auto epd = devPriv->queryPointById(0);
     QVERIFY(epd);
     devPriv->setExclusiveGrabber(nullptr, epd->eventPoint, rightItem);
@@ -1352,9 +1355,9 @@ void tst_TouchMouse::touchPointDeliveryOrder()
     QPoint pLeftMiddle = QPoint(200, 100);
     QPoint pRightMiddle = QPoint(350, 100);
 
-    QTest::QTouchEventSequence touchSeq = QTest::touchEvent(&window, device, false);
+    QTest::QTouchEventSequence touchSeq = QTest::touchEvent(&window, touchscreen.get(), false);
 
-    QVector<QQuickItem*> events;
+    QList<QQuickItem*> events;
     EventItem *background = window.rootObject()->findChild<EventItem*>("background");
     EventItem *left = window.rootObject()->findChild<EventItem*>("left");
     EventItem *middle = window.rootObject()->findChild<EventItem*>("middle");
@@ -1367,10 +1370,11 @@ void tst_TouchMouse::touchPointDeliveryOrder()
     left->setAcceptTouchEvents(true);
     middle->setAcceptTouchEvents(true);
     right->setAcceptTouchEvents(true);
-    connect(background, &EventItem::onTouchEvent, [&events](QQuickItem* receiver){ events.append(receiver); });
-    connect(left, &EventItem::onTouchEvent, [&events](QQuickItem* receiver){ events.append(receiver); });
-    connect(middle, &EventItem::onTouchEvent, [&events](QQuickItem* receiver){ events.append(receiver); });
-    connect(right, &EventItem::onTouchEvent, [&events](QQuickItem* receiver){ events.append(receiver); });
+    auto appendReceiver = [&events](QQuickItem* receiver) { events.append(receiver); };
+    connect(background, &EventItem::onTouchEvent, this, appendReceiver);
+    connect(left, &EventItem::onTouchEvent, this, appendReceiver);
+    connect(middle, &EventItem::onTouchEvent, this, appendReceiver);
+    connect(right, &EventItem::onTouchEvent, this, appendReceiver);
 
     touchSeq.press(0, pLeft, &window).commit();
     QQuickTouchUtils::flush(&window);
@@ -1467,7 +1471,7 @@ void tst_TouchMouse::hoverEnabled() // QTBUG-40856
     QVERIFY(!mouseArea2->hovered());
 
     // ------------------------- Touch click on mouseArea1
-    QTest::touchEvent(&window, device).press(0, p1, &window);
+    QTest::touchEvent(&window, touchscreen.get()).press(0, p1, &window);
     deliveryAgent->flushFrameSynchronousEvents(&window);
 
     QCOMPARE(enterSpy1.size(), 1);
@@ -1476,14 +1480,14 @@ void tst_TouchMouse::hoverEnabled() // QTBUG-40856
     QVERIFY(mouseArea1->hovered());
     QVERIFY(!mouseArea2->hovered());
 
-    QTest::touchEvent(&window, device).release(0, p1, &window);
+    QTest::touchEvent(&window, touchscreen.get()).release(0, p1, &window);
     deliveryAgent->flushFrameSynchronousEvents(&window);
     QVERIFY(clickSpy1.size() == 1);
     QVERIFY(mouseArea1->hovered());
     QVERIFY(!mouseArea2->hovered());
 
     // ------------------------- Touch click on mouseArea2
-    QTest::touchEvent(&window, device).press(0, p2, &window);
+    QTest::touchEvent(&window, touchscreen.get()).press(0, p2, &window);
     deliveryAgent->flushFrameSynchronousEvents(&window);
 
     QVERIFY(!mouseArea1->hovered());
@@ -1492,7 +1496,7 @@ void tst_TouchMouse::hoverEnabled() // QTBUG-40856
     QCOMPARE(enterSpy1.size(), 1);
     QCOMPARE(enterSpy2.size(), 1);
 
-    QTest::touchEvent(&window, device).release(0, p2, &window);
+    QTest::touchEvent(&window, touchscreen.get()).release(0, p2, &window);
     deliveryAgent->flushFrameSynchronousEvents(&window);
 
     QVERIFY(clickSpy2.size() == 1);
@@ -1502,7 +1506,7 @@ void tst_TouchMouse::hoverEnabled() // QTBUG-40856
     QCOMPARE(exitSpy2.size(), 1);
 
     // ------------------------- Another touch click on mouseArea1
-    QTest::touchEvent(&window, device).press(0, p1, &window);
+    QTest::touchEvent(&window, touchscreen.get()).press(0, p1, &window);
     deliveryAgent->flushFrameSynchronousEvents(&window);
 
     QCOMPARE(enterSpy1.size(), 2);
@@ -1511,7 +1515,7 @@ void tst_TouchMouse::hoverEnabled() // QTBUG-40856
     QVERIFY(mouseArea1->hovered());
     QVERIFY(!mouseArea2->hovered());
 
-    QTest::touchEvent(&window, device).release(0, p1, &window);
+    QTest::touchEvent(&window, touchscreen.get()).release(0, p1, &window);
     deliveryAgent->flushFrameSynchronousEvents(&window);
     QCOMPARE(clickSpy1.size(), 2);
     QVERIFY(mouseArea1->hovered());
@@ -1531,23 +1535,23 @@ void tst_TouchMouse::implicitUngrab()
     eventItem->acceptMouse = true;
     eventItem->setAcceptTouchEvents(false);
     QPoint p1(20, 20);
-    QTest::touchEvent(&window, device).press(0, p1);
+    QTest::touchEvent(&window, touchscreen.get()).press(0, p1);
 
     QCOMPARE(grabMonitor.exclusiveGrabber, eventItem);
     eventItem->eventList.clear();
     eventItem->setEnabled(false);
     QVERIFY(!eventItem->eventList.isEmpty());
     QCOMPARE(eventItem->eventList.at(0).type, QEvent::UngrabMouse);
-    QTest::touchEvent(&window, device).release(0, p1);   // clean up potential state
+    QTest::touchEvent(&window, touchscreen.get()).release(0, p1);   // clean up potential state
     QCOMPARE(windowPriv->deliveryAgentPrivate()->touchMouseId, -1);
 
     eventItem->setEnabled(true);
-    QTest::touchEvent(&window, device).press(0, p1);
+    QTest::touchEvent(&window, touchscreen.get()).press(0, p1);
     eventItem->eventList.clear();
     eventItem->setVisible(false);
     QVERIFY(!eventItem->eventList.isEmpty());
     QCOMPARE(eventItem->eventList.at(0).type, QEvent::UngrabMouse);
-    QTest::touchEvent(&window, device).release(0, p1);   // clean up potential state
+    QTest::touchEvent(&window, touchscreen.get()).release(0, p1);   // clean up potential state
 }
 
 void tst_TouchMouse::touchCancelWillCancelMousePress()
@@ -1563,12 +1567,12 @@ void tst_TouchMouse::touchCancelWillCancelMousePress()
     QPoint p1(20, 20);
 
     // Begin a new touch, that gets converted to a mouse press
-    QTest::touchEvent(&window, device).press(0, p1);
+    QTest::touchEvent(&window, touchscreen.get()).press(0, p1);
     QCOMPARE(eventItem->eventList.size(), 1);
     QCOMPARE(eventItem->eventList.at(0).type, QEvent::MouseButtonPress);
 
     // Cancel it...
-    QTouchEvent cancelEvent(QEvent::TouchCancel, device);
+    QTouchEvent cancelEvent(QEvent::TouchCancel, touchscreen.get());
     QCoreApplication::sendEvent(&window, &cancelEvent);
     QCOMPARE(eventItem->eventList.size(), 3);
     QCOMPARE(eventItem->eventList.at(1).type, QEvent::TouchCancel);
@@ -1576,11 +1580,11 @@ void tst_TouchMouse::touchCancelWillCancelMousePress()
 
     // Begin a second touch. Since the last one was cancelled, this
     // should end up as a new mouse press on the target item.
-    QTest::touchEvent(&window, device).press(0, p1);
+    QTest::touchEvent(&window, touchscreen.get()).press(0, p1);
     QVERIFY(eventItem->eventList.size() >= 5);
     QCOMPARE(eventItem->eventList.at(3).type, QEvent::MouseButtonPress);
 
-    QTest::touchEvent(&window, device).release(0, p1);   // clean up potential state
+    QTest::touchEvent(&window, touchscreen.get()).release(0, p1);   // clean up potential state
 }
 
 void tst_TouchMouse::oneTouchInsideAndOneOutside() // QTBUG-102996
@@ -1594,28 +1598,28 @@ void tst_TouchMouse::oneTouchInsideAndOneOutside() // QTBUG-102996
 
     // Press the MouseArea
     QPoint p1 = ma->mapToScene(ma->boundingRect().center()).toPoint();
-    QTest::touchEvent(&window, device).press(1, p1);
+    QTest::touchEvent(&window, touchscreen.get()).press(1, p1);
     QQuickTouchUtils::flush(&window);
     QVERIFY(ma->isPressed());
 
     // Tap outside the MouseArea with a second finger
     QPoint p2(100, 100);
-    QTest::touchEvent(&window, device).stationary(1).press(2, p2);
+    QTest::touchEvent(&window, touchscreen.get()).stationary(1).press(2, p2);
     QQuickTouchUtils::flush(&window);
-    QTest::touchEvent(&window, device).stationary(1).release(2, p2);
+    QTest::touchEvent(&window, touchscreen.get()).stationary(1).release(2, p2);
     QQuickTouchUtils::flush(&window);
     QVERIFY(ma->isPressed());
 
     // Press again outside the MouseArea with a second finger
-    QTest::touchEvent(&window, device).stationary(1).press(2, p2);
+    QTest::touchEvent(&window, touchscreen.get()).stationary(1).press(2, p2);
 
     // Release the first finger: MouseArea should be released
-    QTest::touchEvent(&window, device).release(1, p1).stationary(2);
+    QTest::touchEvent(&window, touchscreen.get()).release(1, p1).stationary(2);
     QQuickTouchUtils::flush(&window);
     QCOMPARE(ma->isPressed(), false);
 
     // Release the second finger
-    QTest::touchEvent(&window, device).release(2, p2);
+    QTest::touchEvent(&window, touchscreen.get()).release(2, p2);
     QQuickTouchUtils::flush(&window);
 }
 
@@ -1634,7 +1638,7 @@ void tst_TouchMouse::strayTouchDoesntAutograb() // QTBUG-107867
     grabMonitor.reset();
 
     // Begin a new touch, that gets converted to a mouse press
-    QTest::touchEvent(&window, device).press(0, p1);
+    QTest::touchEvent(&window, touchscreen.get()).press(0, p1);
     QQuickTouchUtils::flush(&window);
     qCDebug(lcTests) << "after touch press:" << eventItem->eventList;
     QCOMPARE(eventItem->eventList.size(), 1);
@@ -1643,7 +1647,7 @@ void tst_TouchMouse::strayTouchDoesntAutograb() // QTBUG-107867
 
     // Drag
     for (int i = 0; i < 3; ++i) {
-        QTest::touchEvent(&window, device).move(0, p1 + QPoint(i * 5, i * 5), &window);
+        QTest::touchEvent(&window, touchscreen.get()).move(0, p1 + QPoint(i * 5, i * 5), &window);
         QQuickTouchUtils::flush(&window);
         QCOMPARE(grabMonitor.transitionCount, 1); // no new grab
         QCOMPARE(eventItem->eventList.size(), i + 2);
@@ -1651,13 +1655,65 @@ void tst_TouchMouse::strayTouchDoesntAutograb() // QTBUG-107867
     }
 
     // Press an extra point: EventItem should see nothing
-    QTest::touchEvent(&window, device).stationary(0).press(1, p1);
+    QTest::touchEvent(&window, touchscreen.get()).stationary(0).press(1, p1);
     QQuickTouchUtils::flush(&window);
     qCDebug(lcTests) << "after press of second touchpoint:" << eventItem->eventList;
     QCOMPARE(eventItem->eventList.size(), 4);
     QCOMPARE(grabMonitor.transitionCount, 1); // no new grab
 
-    QTest::touchEvent(&window, device).release(0, p1).release(1, p1);
+    QTest::touchEvent(&window, touchscreen.get()).release(0, p1).release(1, p1);
+}
+
+void tst_TouchMouse::noDoubleClickWithInterveningTouch() // QTBUG-116442
+{
+    QQuickView window;
+    QVERIFY(QQuickTest::showView(window, testFileUrl("twosiblingitems.qml")));
+
+    EventItem *leftItem = window.rootObject()->findChild<EventItem*>("leftItem");
+    QVERIFY(leftItem);
+    // simulate a MouseArea: don't accept touch
+    leftItem->setAcceptedMouseButtons(Qt::LeftButton);
+    leftItem->acceptMouse = true;
+
+    EventItem *rightItem = window.rootObject()->findChild<EventItem*>("rightItem");
+    QVERIFY(rightItem);
+    // simulate an item that reacts to either touch or mouse
+    rightItem->setAcceptedMouseButtons(Qt::LeftButton);
+    rightItem->acceptMouse = true;
+    rightItem->setAcceptTouchEvents(true);
+    rightItem->acceptTouch = true;
+
+    const QPoint pLeft(80, 200);
+    const QPoint pRight(240, 200);
+
+    // tap left
+    QTest::touchEvent(&window, touchscreen.get()).press(1, pLeft, &window);
+    QTest::touchEvent(&window, touchscreen.get()).release(1, pLeft, &window);
+    QQuickTouchUtils::flush(&window);
+    qCDebug(lcTests) << "left tap" << leftItem->eventList;
+    QCOMPARE(leftItem->eventList.size(), 3);
+    QCOMPARE(leftItem->eventList.at(0).type, QEvent::MouseButtonPress);
+    leftItem->eventList.clear();
+
+    // tap right
+    QTest::touchEvent(&window, touchscreen.get()).press(1, pRight, &window);
+    QTest::touchEvent(&window, touchscreen.get()).release(1, pRight, &window);
+    QQuickTouchUtils::flush(&window);
+    qCDebug(lcTests) << "right tap" << rightItem->eventList;
+    QCOMPARE(rightItem->eventList.size(), 2);
+    QCOMPARE(rightItem->eventList.at(0).type, QEvent::TouchBegin);
+    rightItem->eventList.clear();
+
+    // tap left again: this is NOT a double-click, even though it's within time and space limits
+    QTest::touchEvent(&window, touchscreen.get()).press(3, pLeft, &window);
+    QTest::touchEvent(&window, touchscreen.get()).release(3, pLeft, &window);
+    QQuickTouchUtils::flush(&window);
+    qCDebug(lcTests) << "left tap again" << leftItem->eventList;
+    QCOMPARE(leftItem->eventList.size(), 3);
+    QCOMPARE(leftItem->eventList.at(0).type, QEvent::MouseButtonPress);
+    QCOMPARE(leftItem->eventList.at(1).type, QEvent::MouseButtonRelease);
+    QCOMPARE(leftItem->eventList.at(2).type, QEvent::UngrabMouse);
+    leftItem->eventList.clear();
 }
 
 QTEST_MAIN(tst_TouchMouse)
