@@ -6,8 +6,11 @@
 #include <QtQml/qqmlcomponent.h>
 #include <QtQml/qqmlcontext.h>
 #include <QtQuick/private/qquickitem_p.h>
+#include <QtQuickTest/quicktest.h>
 #include <QtQuickTestUtils/private/qmlutils_p.h>
+#include <QtQuickTemplates2/private/qquickbutton_p.h>
 #include <QtQuickTemplates2/private/qquickpopup_p.h>
+#include <QtQuickTemplates2/private/qquickdialogbuttonbox_p.h>
 
 #if QT_CONFIG(accessibility)
 #include <QtGui/private/qguiapplication_p.h>
@@ -40,6 +43,8 @@ private slots:
     void accessibleName();
     void locale();
 
+    void defaultButton();
+
 private:
     QQmlEngine engine;
 };
@@ -103,7 +108,7 @@ void tst_accessibility::a11y_data()
     QTest::newRow("Label") << "label" << QAccessible::StaticText << "Label";
     QTest::newRow("Menu") << "menu" << QAccessible::PopupMenu << "";
     QTest::newRow("MenuItem") << "menuitem" << QAccessible::MenuItem << "MenuItem";
-    QTest::newRow("Page") << "page" << QAccessible::PageTab << "Page";
+    QTest::newRow("Page") << "page" << QAccessible::Pane << "Page";
     QTest::newRow("PageIndicator") << "pageindicator" << QAccessible::Indicator << "";
     QTest::newRow("Pane") << "pane" << QAccessible::Pane << "";
     QTest::newRow("Popup") << "popup" << QAccessible::Dialog << "";
@@ -188,7 +193,7 @@ void tst_accessibility::override_data()
     QTest::newRow("Label") << QAccessible::StaticText;
     QTest::newRow("Menu") << QAccessible::PopupMenu;
     QTest::newRow("MenuItem") << QAccessible::MenuItem;
-    QTest::newRow("Page") << QAccessible::PageTab;
+    QTest::newRow("Page") << QAccessible::Pane;
     QTest::newRow("PageIndicator") << QAccessible::Indicator;
     QTest::newRow("Pane") << QAccessible::Pane;
     QTest::newRow("Popup") << QAccessible::Dialog;
@@ -471,6 +476,39 @@ void tst_accessibility::locale()
 #endif
 }
 
+void tst_accessibility::defaultButton()
+{
+#if QT_CONFIG(accessibility)
+    if (!QAccessible::isActive()) {
+        QPlatformAccessibility *accessibility = platformAccessibility();
+        if (!accessibility)
+            QSKIP("No QPlatformAccessibility available.");
+        accessibility->setActive(true);
+    }
+    QQmlEngine engine;
+    QQmlComponent component(&engine);
+    component.setData("import QtQuick.Controls\nDialogButtonBox {\n"
+                      "standardButtons: DialogButtonBox.Ok | DialogButtonBox.No\n"
+                      "defaultStandardButton: DialogButtonBox.Ok\n"
+                      "}", QUrl());
+    QScopedPointer<QObject> object(component.create());
+    QVERIFY2(!object.isNull(), qPrintable(component.errorString()));
+
+    QQuickDialogButtonBox *buttonBox = qobject_cast<QQuickDialogButtonBox *>(object.get());
+    QVERIFY(buttonBox);
+    QVERIFY(QQuickTest::qWaitForPolish(buttonBox));
+    QCOMPARE(buttonBox->count(), 2);
+
+    for (int i = 0; i < buttonBox->count(); ++i) {
+        QQuickButton *button = qobject_cast<QQuickButton *>(buttonBox->itemAt(i));
+        QVERIFY(button);
+        QAccessibleInterface *buttonAccessible = QAccessible::queryAccessibleInterface(button);
+        QVERIFY(buttonAccessible);
+        QCOMPARE(buttonAccessible->state().defaultButton, button->isHighlighted());
+    }
+#endif
+}
+
 QTEST_MAIN(tst_accessibility)
 
 #include "tst_accessibility.moc"
