@@ -3,6 +3,7 @@
 
 #include <QtTest/qtest.h>
 #include <QtTest/qsignalspy.h>
+#include <QtTest/private/qtesthelpers_p.h>
 
 #include <QtQuick/qquickwindow.h>
 #include <QtQuick/qquickitem.h>
@@ -52,7 +53,7 @@ public:
         return ret || (ret && event->exclusiveGrabber(point) != this);
     }
 
-    bool grabPoints(QPointerEvent * event, const QVector<QEventPoint> & points)
+    bool grabPoints(QPointerEvent * event, const QList<QEventPoint> & points)
     {
         if (points.isEmpty())
             return false;
@@ -238,6 +239,9 @@ void tst_QQuickOverlay::pressedAndReleased()
     QQuickApplicationWindow *window = helper.appWindow;
     window->show();
     QVERIFY(QTest::qWaitForWindowExposed(window));
+    // Verify that the position and framePosition are correct
+    // to avoid flakiness on openSuSe KDE.
+    QVERIFY(QTestPrivate::ensurePositionTopLeft(window));
 
     auto *overlay = QQuickOverlay::overlay(window);
     QVERIFY(overlay);
